@@ -42,7 +42,7 @@
  * @brief Blob slap worker utility
  */
 
-#include <config.h>
+#include "gear_config.h"
 #include <benchmark/benchmark.h>
 #include <boost/program_options.hpp>
 #include <cerrno>
@@ -85,7 +85,7 @@ int main(int args, char *argv[])
   uint32_t count= UINT_MAX;
   in_port_t port;
   std::string host;
-  std::vector<std::string>* functions= NULL;
+  std::vector<std::string> functions;
   std::string verbose_string;
   boost::program_options::options_description desc("Options");
   desc.add_options()
@@ -98,7 +98,7 @@ int main(int args, char *argv[])
     ("status,s", boost::program_options::bool_switch(&opt_status)->default_value(false), "Send status updates and sleep while running job")
     ("unique,u", boost::program_options::bool_switch(&opt_unique)->default_value(false), "When grabbing jobs, grab the uniqie id")
     ("daemon,d", boost::program_options::bool_switch(&opt_daemon)->default_value(false), "Daemonize")
-    ("function,f", boost::program_options::value(functions), "Function to use.")
+    ("function,f", boost::program_options::value(&functions), "Function to use.")
     ("verbose,v", boost::program_options::value(&verbose_string)->default_value("v"), "Increase verbosity level by one.")
     ("pid-file", boost::program_options::value(&pid_file), "File to write process ID out to.")
     ("log-file", boost::program_options::value(&log_file), "Create a log file.")
@@ -127,24 +127,15 @@ int main(int args, char *argv[])
     util::daemonize(false, true);
   }
 
-  if (not pid_file.empty())
+  util::Pidfile _pid_file(pid_file);
+
+  if (pid_file.empty() == false)
   {
-    if (access(pid_file.c_str(), F_OK) == 0)
+    if (_pid_file.create() == false)
     {
-      std::cerr << "pid_file already exists:" << pid_file << std::endl;
+      std::cerr << _pid_file.error_message().c_str();
       return EXIT_FAILURE;
     }
-
-    FILE *file= fopen(pid_file.c_str(), "w+");
-    if (file == NULL)
-    {
-      std::cerr << "Unable to open:" << pid_file << "(" << strerror(errno) << ")" << std::endl;
-      return EXIT_FAILURE;
-    }
-    fclose(file);
-
-    // We let the error from this happen later (if one was to occur)
-    unlink(pid_file.c_str());
   }
 
   if (not log_file.empty())
@@ -196,14 +187,6 @@ int main(int args, char *argv[])
     return EXIT_FAILURE;
   }
 
-
-  util::Pidfile _pid_file(pid_file);
-  if (not _pid_file.create())
-  {
-    log.log() << _pid_file.error_message() << std::endl;
-    return EXIT_FAILURE;
-  }
-
   gearman_function_t shutdown_function= gearman_function_create(shutdown_fn);
   if (gearman_failed(gearman_worker_define_function(worker,
                                                     util_literal_param("shutdown"), 
@@ -224,9 +207,9 @@ int main(int args, char *argv[])
     return EXIT_FAILURE;
   }
 
-  if (functions and functions->size())
+  if (functions.empty() == false)
   {
-    for (std::vector<std::string>::iterator iter= functions->begin(); iter != functions->end(); iter++)
+    for (std::vector<std::string>::iterator iter= functions.begin(); iter != functions.end(); ++iter)
     {
       if (gearman_failed(gearman_worker_add_function(worker,
                                                      (*iter).c_str(), 0,
