@@ -1,17 +1,42 @@
-/* Gearman server and library
- * Copyright (C) 2008 Brian Aker, Eric Day
- * All rights reserved.
+/*  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ * 
+ *  Gearmand client and server library.
+ *
+ *  Copyright (C) 2011-2012 Data Differential, http://datadifferential.com/
+ *  Copyright (C) 2008 Brian Aker, Eric Day
+ *  All rights reserved.
+ *
+ *  Redistribution and use in source and binary forms, with or without
+ *  modification, are permitted provided that the following conditions are
+ *  met:
+ *
+ *      * Redistributions of source code must retain the above copyright
+ *  notice, this list of conditions and the following disclaimer.
+ *
+ *      * Redistributions in binary form must reproduce the above
+ *  copyright notice, this list of conditions and the following disclaimer
+ *  in the documentation and/or other materials provided with the
+ *  distribution.
+ *
+ *      * The names of its contributors may not be used to endorse or
+ *  promote products derived from this software without specific prior
+ *  written permission.
+ *
+ *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+ *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
+ *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+ *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+ *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+ *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+ *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+ *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+ *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- * Use and distribution licensed under the BSD license.  See
- * the COPYING file in the parent directory for full text.
  */
 
-/**
- * @file
- * @brief Gearman Command Line Tool
- */
-
-#include <config.h>
+#include "gear_config.h"
 
 #include <assert.h>
 #include <errno.h>
@@ -29,11 +54,14 @@
 #include <vector>
 
 
-#include <libgearman/gearman.h>
+#include <libgearman-1.0/gearman.h>
 
 #include "bin/arguments.h"
-#include "bin/client.h"
-#include "bin/worker.h"
+
+#include "libgearman/client.hpp"
+#include "libgearman/worker.hpp"
+using namespace org::gearmand;
+
 #include "util/pidfile.hpp"
 #include "bin/error.h"
 
@@ -62,9 +90,11 @@ static void _client(Args &args);
 /**
  * Run client jobs.
  */
-static void _client_run(gearman_client_st& client, Args &args,
+static void _client_run(libgearman::Client& client, Args &args,
                         const void *workload, size_t workload_size);
 
+static gearman_return_t _client_created(gearman_task_st *task);
+
 /**
  * Client data/complete callback function.
  */
@@ -103,7 +133,7 @@ static void _read_workload(int fd, Bytes& workload);
 /**
  * Print usage information.
  */
-static void usage(char *name);
+static void usage(Args&, char *name);
 
 extern "C"
 {
@@ -125,13 +155,13 @@ int main(int argc, char *argv[])
 
   if (args.usage())
   {
-    usage(argv[0]);
+    usage(args, argv[0]);
     return EXIT_SUCCESS;
   }
 
   if (args.is_error())
   {
-    usage(argv[0]);
+    usage(args, argv[0]);
     return EXIT_FAILURE;
   }
 
@@ -212,20 +242,28 @@ int main(int argc, char *argv[])
 
 void _client(Args &args)
 {
-  Client local_client;
-  gearman_client_st &client= local_client.client();
+  libgearman::Client client;
   Bytes workload;
   if (args.timeout() >= 0)
   {
     gearman_client_set_timeout(&client, args.timeout());
   }
 
-  if (gearman_failed(gearman_client_add_server(&client, args.host(), args.port())))
+  if (getenv("GEARMAN_SERVER"))
   {
-    error::message("gearman_client_add_server", client);
+    if (gearman_failed(gearman_client_add_servers(&client, getenv("GEARMAN_SERVER"))))
+    {
+      error::message("Error occurred while parsing GEARMAN_SERVER", &client);
+      return;
+    }
+  }
+  else if (gearman_failed(gearman_client_add_server(&client, args.host(), args.port())))
+  {
+    error::message("gearman_client_add_server", &client);
     return;
   }
 
+  gearman_client_set_created_fn(&client, _client_created);
   gearman_client_set_data_fn(&client, _client_data);
   gearman_client_set_warning_fn(&client, _client_warning);
   gearman_client_set_status_fn(&client, _client_status);
@@ -275,14 +313,14 @@ void _client(Args &args)
   }
 }
 
-void _client_run(gearman_client_st& client, Args &args,
+void _client_run(libgearman::Client& client, Args &args,
                  const void *workload, size_t workload_size)
 {
   gearman_return_t ret;
 
   for (Function::vector::iterator iter= args.begin(); 
        iter != args.end();
-       iter++)
+       ++iter)
   {
     Function &function= *iter;
 
@@ -371,16 +409,28 @@ void _client_run(gearman_client_st& client, Args &args,
 
     if (gearman_failed(ret))
     {
-      error::message("gearman_client_add_task", client);
+      error::message("gearman_client_add_task", &client);
     }
   }
 
   if (gearman_failed(gearman_client_run_tasks(&client)))
   {
-    error::message("gearman_client_run_tasks", client);
+    error::message("gearman_client_run_tasks", &client);
   }
 }
 
+static gearman_return_t _client_created(gearman_task_st *task)
+{
+  const Args *args= static_cast<const Args*>(gearman_task_context(task));
+
+  if (args->verbose())
+  {
+    fprintf(stdout, "Task created: %s\n", gearman_task_job_handle(task));
+  }
+
+  return GEARMAN_SUCCESS;
+}
+
 static gearman_return_t _client_data(gearman_task_st *task)
 {
   const Args *args= static_cast<const Args*>(gearman_task_context(task));
@@ -449,17 +499,24 @@ static void _worker_free(void *, void *)
 
 void _worker(Args &args)
 {
-  Worker local_worker;
-  gearman_worker_st &worker= local_worker.worker();
+  libgearman::Worker worker;
 
   if (args.timeout() >= 0)
   {
     gearman_worker_set_timeout(&worker, args.timeout());
   }
 
-  if (gearman_failed(gearman_worker_add_server(&worker, args.host(), args.port())))
+  if (getenv("GEARMAN_SERVER"))
   {
-    error::message("gearman_worker_add_server", worker);
+    if (gearman_failed(gearman_worker_add_servers(&worker, getenv("GEARMAN_SERVER"))))
+    {
+      error::message("Error occurred while parsing GEARMAN_SERVER", &worker);
+      return;
+    }
+  }
+  else if (gearman_failed(gearman_worker_add_server(&worker, args.host(), args.port())))
+  {
+    error::message("gearman_worker_add_server", &worker);
     _exit(EXIT_FAILURE);
   }
 
@@ -467,13 +524,13 @@ void _worker(Args &args)
 
   for (Function::vector::iterator iter= args.begin(); 
        iter != args.end();
-       iter++)
+       ++iter)
   {
     Function &function= *iter;
     worker_argument_t pass(args, *iter);
     if (gearman_failed(gearman_worker_add_function(&worker, function.name(), 0, _worker_cb, &pass)))
     {
-      error::message("gearman_worker_add_function", worker);
+      error::message("gearman_worker_add_function", &worker);
       _exit(EXIT_FAILURE);
     }
   }
@@ -482,7 +539,7 @@ void _worker(Args &args)
   {
     if (gearman_failed(gearman_worker_work(&worker)))
     {
-      error::message("gearman_worker_work", worker);
+      error::message("gearman_worker_work", &worker);
     }
 
     if (args.count() > 0)
@@ -508,9 +565,6 @@ static void *_worker_cb(gearman_job_st *job, void *context,
                         size_t *result_size, gearman_return_t *ret_ptr)
 {
   worker_argument_t *arguments= static_cast<worker_argument_t *>(context);
-  int in_fds[2];
-  int out_fds[2];
-  int status;
 
   Args &args= arguments->args;
   Function &function= arguments->function;
@@ -529,7 +583,10 @@ static void *_worker_cb(gearman_job_st *job, void *context,
   }
   else
   {
-    if (pipe(in_fds) == -1 || pipe(out_fds) == -1)
+    int in_fds[2];
+    int out_fds[2];
+
+    if (pipe(in_fds) == -1 or pipe(out_fds) == -1)
     {
       error::perror("pipe");
     }
@@ -654,6 +711,7 @@ static void *_worker_cb(gearman_job_st *job, void *context,
       *result_size= function.buffer().size();
     }
 
+    int status;
     if (wait(&status) == -1)
     {
       error::perror("wait");
@@ -701,32 +759,45 @@ void _read_workload(int fd, Bytes& workload)
   }
 }
 
-static void usage(char *name)
+static void usage(Args& args, char *name)
 {
-  printf("Client mode: %s [options] [<data>]\n", name);
-  printf("Worker mode: %s -w [options] [<command> [<args> ...]]\n", name);
+  FILE *stream;
+  if (args.is_error())
+  {
+    stream= stderr;
+    fprintf(stream, "\n%s\tError in usage(%s).\n\n", name, args.arg_error());
+  }
+  else
+  {
+    stream= stdout;
+    fprintf(stream, "\n%s\tError in usage(%s).\n\n", name, args.arg_error());
+  }
 
-  printf("\nCommon options to both client and worker modes.\n");
-  printf("\t-f <function> - Function name to use for jobs (can give many)\n");
-  printf("\t-h <host>     - Job server host\n");
-  printf("\t-H            - Print this help menu\n");
-  printf("\t-p <port>     - Job server port\n");
-  printf("\t-t <timeout>  - Timeout in milliseconds\n");
-  printf("\t-i <pidfile>  - Create a pidfile for the process\n");
+  fprintf(stream, "Client mode: %s [options] [<data>]\n", name);
+  fprintf(stream, "Worker mode: %s -w [options] [<command> [<args> ...]]\n", name);
 
-  printf("\nClient options:\n");
-  printf("\t-b            - Run jobs in the background\n");
-  printf("\t-I            - Run jobs as high priority\n");
-  printf("\t-L            - Run jobs as low priority\n");
-  printf("\t-n            - Run one job per line\n");
-  printf("\t-N            - Same as -n, but strip off the newline\n");
-  printf("\t-P            - Prefix all output lines with functions names\n");
-  printf("\t-s            - Send job without reading from standard input\n");
-  printf("\t-u <unique>   - Unique key to use for job\n");
+  fprintf(stream, "\nCommon options to both client and worker modes.\n");
+  fprintf(stream, "\t-f <function> - Function name to use for jobs (can give many)\n");
+  fprintf(stream, "\t-h <host>     - Job server host\n");
+  fprintf(stream, "\t-H            - Print this help menu\n");
+  fprintf(stream, "\t-v            - Print diagnostic information to stdout(%s)\n", args.verbose() ? "true" : "false");
+  fprintf(stream, "\t-p <port>     - Job server port\n");
+  fprintf(stream, "\t-t <timeout>  - Timeout in milliseconds\n");
+  fprintf(stream, "\t-i <pidfile>  - Create a pidfile for the process\n");
 
-  printf("\nWorker options:\n");
-  printf("\t-c <count>    - Number of jobs for worker to run before exiting\n");
-  printf("\t-n            - Send data packet for each line\n");
-  printf("\t-N            - Same as -n, but strip off the newline\n");
-  printf("\t-w            - Run in worker mode\n");
+  fprintf(stream, "\nClient options:\n");
+  fprintf(stream, "\t-b            - Run jobs in the background(%s)\n", args.background() ? "true" : "false");
+  fprintf(stream, "\t-I            - Run jobs as high priority\n");
+  fprintf(stream, "\t-L            - Run jobs as low priority\n");
+  fprintf(stream, "\t-n            - Run one job per line(%s)\n", args.job_per_newline() ? "true" : "false");
+  fprintf(stream, "\t-N            - Same as -n, but strip off the newline(%s)\n", args.strip_newline() ? "true" : "false");
+  fprintf(stream, "\t-P            - Prefix all output lines with functions names\n");
+  fprintf(stream, "\t-s            - Send job without reading from standard input\n");
+  fprintf(stream, "\t-u <unique>   - Unique key to use for job\n");
+
+  fprintf(stream, "\nWorker options:\n");
+  fprintf(stream, "\t-c <count>    - Number of jobs for worker to run before exiting\n");
+  fprintf(stream, "\t-n            - Send data packet for each line(%s)\n", args.job_per_newline() ? "true" : "false");
+  fprintf(stream, "\t-N            - Same as -n, but strip off the newline(%s)\n", args.strip_newline() ? "true" : "false");
+  fprintf(stream, "\t-w            - Run in worker mode(%s)\n", args.worker() ? "true" : "false");
 }
