@@ -70,6 +70,7 @@ struct gearman_client_st
     bool unbuffered_result;
     bool no_new;
     bool free_tasks;
+    bool generate_unique;
   } options;
   enum gearman_client_t state;
   uint32_t new_tasks;
@@ -127,6 +128,9 @@ void gearman_client_free(gearman_client_st *client);
 GEARMAN_API
 const char *gearman_client_error(const gearman_client_st *client);
 
+GEARMAN_API
+gearman_return_t gearman_client_error_code(const gearman_client_st *client);
+
 /**
  * See gearman_errno() for details.
  */
@@ -328,9 +332,11 @@ gearman_return_t gearman_client_wait(gearman_client_st *client);
  *  caller is done using it.
  */
 GEARMAN_API
-void *gearman_client_do(gearman_client_st *client, const char *function_name,
-                        const char *unique, const void *workload,
-                        size_t workload_size, size_t *result_size,
+void *gearman_client_do(gearman_client_st *client,
+                        const char *function_name,
+                        const char *unique,
+                        const void *workload, size_t workload_size,
+                        size_t *result_size,
                         gearman_return_t *ret_ptr);
 
 /**
@@ -434,6 +440,17 @@ gearman_return_t gearman_client_job_status(gearman_client_st *client,
                                            uint32_t *numerator,
                                            uint32_t *denominator);
 
+GEARMAN_API
+gearman_status_t gearman_client_unique_status(gearman_client_st *client,
+                                              const char *unique, size_t unique_length);
+
+// This is not in the API yet, subject to change
+GEARMAN_API
+gearman_task_st *gearman_client_add_task_status_by_unique(gearman_client_st *client,
+                                                          gearman_task_st *task_ptr,
+                                                          const char *unique_handle,
+                                                          gearman_return_t *ret_ptr);
+
 /**
  * Send data to all job servers to see if they echo it back. This is a test
  * function to see if the job servers are responding properly.
@@ -554,30 +571,28 @@ gearman_task_st *gearman_client_add_task_background(gearman_client_st *client,
  * gearman_client_add_task() for details.
  */
 GEARMAN_API
-gearman_task_st *
-gearman_client_add_task_high_background(gearman_client_st *client,
-                                        gearman_task_st *task,
-                                        void *context,
-                                        const char *function_name,
-                                        const char *unique,
-                                        const void *workload,
-                                        size_t workload_size,
-                                        gearman_return_t *ret_ptr);
+gearman_task_st *gearman_client_add_task_high_background(gearman_client_st *client,
+                                                         gearman_task_st *task,
+                                                         void *context,
+                                                         const char *function_name,
+                                                         const char *unique,
+                                                         const void *workload,
+                                                         size_t workload_size,
+                                                         gearman_return_t *ret_ptr);
 
 /**
  * Add a low priority background task to be run in parallel. See
  * gearman_client_add_task() for details.
  */
 GEARMAN_API
-gearman_task_st *
-gearman_client_add_task_low_background(gearman_client_st *client,
-                                       gearman_task_st *task,
-                                       void *context,
-                                       const char *function_name,
-                                       const char *unique,
-                                       const void *workload,
-                                       size_t workload_size,
-                                       gearman_return_t *ret_ptr);
+gearman_task_st *gearman_client_add_task_low_background(gearman_client_st *client,
+                                                        gearman_task_st *task,
+                                                        void *context,
+                                                        const char *function_name,
+                                                        const char *unique,
+                                                        const void *workload,
+                                                        size_t workload_size,
+                                                        gearman_return_t *ret_ptr);
 
 /**
  * Add task to get the status for a backgound task in parallel.
@@ -705,18 +720,12 @@ void gearman_client_clear_fn(gearman_client_st *client);
 GEARMAN_API
 gearman_return_t gearman_client_run_tasks(gearman_client_st *client);
 
-GEARMAN_LOCAL
-gearman_return_t gearman_client_run_block_tasks(gearman_client_st *client);
-
 GEARMAN_API
 bool gearman_client_compare(const gearman_client_st *first, const gearman_client_st *second);
 
 GEARMAN_API
 bool gearman_client_set_server_option(gearman_client_st *self, const char *option_arg, size_t option_arg_size);
 
-GEARMAN_LOCAL
-size_t gearman_client_count_tasks(gearman_client_st *client);
-
 GEARMAN_API
 void gearman_client_set_namespace(gearman_client_st *self, const char *namespace_key, size_t namespace_key_size);
 
@@ -724,6 +733,8 @@ GEARMAN_API
 gearman_return_t gearman_client_set_identifier(gearman_client_st *client,
                                                const char *id, size_t id_size);
 
+const char *gearman_client_namespace(gearman_client_st *self);
+
 /** @} */
 
 #ifdef __cplusplus
