@@ -1,9 +1,39 @@
-/* Gearman server and library
- * Copyright (C) 2008 Brian Aker, Eric Day
- * All rights reserved.
+/*  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ * 
+ *  Gearmand client and server library.
+ *
+ *  Copyright (C) 2011-2012 Data Differential, http://datadifferential.com/
+ *  Copyright (C) 2008 Brian Aker, Eric Day
+ *  All rights reserved.
+ *
+ *  Redistribution and use in source and binary forms, with or without
+ *  modification, are permitted provided that the following conditions are
+ *  met:
+ *
+ *      * Redistributions of source code must retain the above copyright
+ *  notice, this list of conditions and the following disclaimer.
+ *
+ *      * Redistributions in binary form must reproduce the above
+ *  copyright notice, this list of conditions and the following disclaimer
+ *  in the documentation and/or other materials provided with the
+ *  distribution.
+ *
+ *      * The names of its contributors may not be used to endorse or
+ *  promote products derived from this software without specific prior
+ *  written permission.
+ *
+ *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+ *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
+ *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+ *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+ *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+ *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+ *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+ *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+ *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- * Use and distribution licensed under the BSD license.  See
- * the COPYING file in the parent directory for full text.
  */
 
 /**
@@ -20,6 +50,7 @@
 #endif
 
 #include <libgearman-1.0/limits.h>
+#include <libgearman-1.0/priority.h>
 #include <libgearman-server/verbose.h>
 
 /**
@@ -38,7 +69,7 @@
 #define GEARMAN_DEFAULT_SOCKET_SEND_SIZE 32768
 #define GEARMAN_DEFAULT_SOCKET_TIMEOUT 10
 #define GEARMAND_JOB_HANDLE_SIZE 64
-#define GEARMAND_JOB_HASH_SIZE 383
+#define GEARMAND_DEFAULT_HASH_SIZE 991
 #define GEARMAN_MAX_COMMAND_ARGS 8
 #define GEARMAN_MAX_FREE_SERVER_CLIENT 1000
 #define GEARMAN_MAX_FREE_SERVER_CON 1000
@@ -52,30 +83,14 @@
 #define GEARMAN_SEND_BUFFER_SIZE 8192
 #define GEARMAN_SERVER_CON_ID_SIZE 128
 #define GEARMAN_TEXT_RESPONSE_SIZE 8192
+#define GEARMAN_MAGIC_MEMORY (void*)(0x000001)
 
 /** @} */
 
-/**
- * @ingroup gearman_job
- * Priority levels for a job.
- */
-enum gearmand_job_priority_t
-{
-  GEARMAND_JOB_PRIORITY_HIGH,
-  GEARMAND_JOB_PRIORITY_NORMAL,
-  GEARMAND_JOB_PRIORITY_LOW,
-  GEARMAND_JOB_PRIORITY_MAX /* Always add new commands before this. */
-};
-
-#ifndef __cplusplus
-typedef enum gearmand_job_priority_t gearmand_job_priority_t;
-#endif
-
 /** @} */
 
 typedef enum
 {
-  GEARMAND_CON_READY,
   GEARMAND_CON_PACKET_IN_USE,
   GEARMAND_CON_EXTERNAL_FD,
   GEARMAND_CON_CLOSE_AFTER_FLUSH,
@@ -90,22 +105,6 @@ struct gearman_server_st;
 struct gearman_server_con_st;
 struct gearmand_io_st;
 
-#else /* Types. */
-
-typedef struct gearman_server_client_st gearman_server_client_st;
-typedef struct gearman_server_con_st gearman_server_con_st;
-typedef struct gearman_server_function_st gearman_server_function_st;
-typedef struct gearman_server_job_st gearman_server_job_st;
-typedef struct gearman_server_packet_st gearman_server_packet_st;
-typedef struct gearman_server_st gearman_server_st;
-typedef struct gearman_server_thread_st gearman_server_thread_st;
-typedef struct gearman_server_worker_st gearman_server_worker_st;
-typedef struct gearmand_con_st gearmand_con_st;
-typedef struct gearmand_io_st gearmand_io_st;
-typedef struct gearmand_port_st gearmand_port_st;
-typedef struct gearmand_st gearmand_st;
-typedef struct gearmand_thread_st gearmand_thread_st;
-
 #endif
 
 #ifdef __cplusplus
@@ -124,7 +123,7 @@ typedef gearmand_error_t (gearman_queue_add_fn)(gearman_server_st *server,
                                                 size_t function_name_size,
                                                 const void *data,
                                                 size_t data_size,
-                                                gearmand_job_priority_t priority,
+                                                gearman_job_priority_t priority,
                                                 int64_t when);
 
 typedef gearmand_error_t (gearman_queue_flush_fn)(gearman_server_st *server,
@@ -149,16 +148,6 @@ typedef void (gearman_log_server_fn)(const char *line, gearmand_verbose_t verbos
 typedef gearmand_error_t (gearmand_event_watch_fn)(gearmand_io_st *con,
                                                    short events, void *context);
 
-typedef struct gearmand_packet_st gearmand_packet_st;
-
-typedef size_t (gearmand_packet_pack_fn)(const gearmand_packet_st *packet,
-                                         gearman_server_con_st *con,
-                                         void *data, size_t data_size,
-                                         gearmand_error_t *ret_ptr);
-typedef size_t (gearmand_packet_unpack_fn)(gearmand_packet_st *packet,
-                                           gearman_server_con_st *con, const void *data,
-                                           size_t data_size,
-                                           gearmand_error_t *ret_ptr);
 typedef void (gearmand_connection_protocol_context_free_fn)(gearman_server_con_st *con,
                                                             void *context);
 
