@@ -58,11 +58,13 @@
 
 #include <libgearman-server/constants.h>
 #include <libgearman-server/wakeup.h>
-#include <libgearman-server/connection_list.h>
 #include <libgearman-server/byteorder.h>
 #include <libgearman-server/log.h>
 #include <libgearman-server/packet.h>
 #include <libgearman-server/connection.h>
+#ifdef __cplusplus
+#include <libgearman-server/connection.hpp>
+#endif
 #include <libgearman-server/function.h>
 #include <libgearman-server/client.h>
 #include <libgearman-server/worker.h>
@@ -101,13 +103,15 @@ gearmand_st *Gearmand(void);
  */
 GEARMAN_API
 gearmand_st *gearmand_create(const char *host,
-                             const char *port,
                              uint32_t threads,
                              int backlog,
-                             uint8_t job_retries,
+                             const uint32_t job_retries,
+                             const char *job_handle_prefix,
                              uint8_t worker_wakeup,
                              gearmand_log_fn *function, void *log_context, const gearmand_verbose_t verbose,
-                             bool round_robin);
+                             bool round_robin,
+                             bool exceptions_,
+                             uint32_t hashtable_buckets);
 
 /**
  * Free resources used by a server instace.
@@ -144,6 +148,8 @@ gearmand_error_t gearmand_port_add(gearmand_st *gearmand,
 GEARMAN_API
 gearmand_error_t gearmand_run(gearmand_st *gearmand);
 
+bool gearmand_exceptions(gearmand_st *gearmand);
+
 /**
  * Interrupt a running gearmand server from another thread. You should only
  * call this when another thread is currently running gearmand_run() and you
