@@ -1,9 +1,39 @@
-/* Gearman server and library
- * Copyright (C) 2008 Brian Aker, Eric Day
- * All rights reserved.
+/*  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ * 
+ *  Gearmand client and server library.
+ *
+ *  Copyright (C) 2011-2013 Data Differential, http://datadifferential.com/
+ *  Copyright (C) 2008 Brian Aker, Eric Day
+ *  All rights reserved.
+ *
+ *  Redistribution and use in source and binary forms, with or without
+ *  modification, are permitted provided that the following conditions are
+ *  met:
+ *
+ *      * Redistributions of source code must retain the above copyright
+ *  notice, this list of conditions and the following disclaimer.
+ *
+ *      * Redistributions in binary form must reproduce the above
+ *  copyright notice, this list of conditions and the following disclaimer
+ *  in the documentation and/or other materials provided with the
+ *  distribution.
+ *
+ *      * The names of its contributors may not be used to endorse or
+ *  promote products derived from this software without specific prior
+ *  written permission.
+ *
+ *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+ *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
+ *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+ *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+ *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+ *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+ *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+ *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+ *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- * Use and distribution licensed under the BSD license.  See
- * the COPYING file in the parent directory for full text.
  */
 
 /**
@@ -11,13 +41,18 @@
  * @brief Connection Definitions
  */
 
-#include <config.h>
-#include <libgearman-server/common.h>
+#include "gear_config.h"
+#include "libgearman-server/common.h"
+#include <libgearman-server/plugins/base.h>
 
 #include <cstring>
 #include <cerrno>
 #include <cassert>
 
+#ifndef SOCK_NONBLOCK 
+# define SOCK_NONBLOCK 0
+#endif
+
 static void _connection_close(gearmand_io_st *connection)
 {
   if (connection->fd == INVALID_SOCKET)
@@ -32,7 +67,7 @@ static void _connection_close(gearmand_io_st *connection)
   else
   {
     (void)gearmand_sockfd_close(connection->fd);
-    assert(! "We should never have an internal fd");
+    assert_msg(false, "We should never have an internal fd");
   }
 
   connection->_state= gearmand_io_st::GEARMAND_CON_UNIVERSAL_INVALID;
@@ -78,13 +113,14 @@ static size_t _connection_read(gearman_server_con_st *con, void *data, size_t da
     }
     else if (read_size == -1)
     {
-      switch (errno)
+      int local_errno= errno;
+      switch (local_errno)
       {
       case EAGAIN:
         ret= gearmand_io_set_events(con, POLLIN);
         if (gearmand_failed(ret))
         {
-          gearmand_perror("recv");
+          gearmand_perror(local_errno, "recv");
           return 0;
         }
 
@@ -97,16 +133,22 @@ static size_t _connection_read(gearman_server_con_st *con, void *data, size_t da
       case EPIPE:
       case ECONNRESET:
       case EHOSTDOWN:
-        gearmand_perror("lost connection to client recv(EPIPE || ECONNRESET || EHOSTDOWN)");
-        ret= GEARMAN_LOST_CONNECTION;
+        {
+          ret= GEARMAN_LOST_CONNECTION;
+          gearmand_log_info(GEARMAN_DEFAULT_LOG_PARAM, 
+                            "Peer connection has called close() %s:%s",
+                            connection->context == NULL ? "-" : connection->context->host,
+                            connection->context == NULL ? "-" : connection->context->port);
+          _connection_close(connection);
+          return 0;
+        }
         break;
 
       default:
-        gearmand_perror("recv");
         ret= GEARMAN_ERRNO;
       }
 
-      gearmand_error("closing connection due to previous errno error");
+      gearmand_perror(local_errno, "closing connection due to previous errno error");
       _connection_close(connection);
       return 0;
     }
@@ -118,7 +160,7 @@ static size_t _connection_read(gearman_server_con_st *con, void *data, size_t da
   return size_t(read_size);
 }
 
-gearmand_error_t gearmand_connection_recv_data(gearman_server_con_st *con, void *data, size_t data_size)
+static gearmand_error_t gearmand_connection_recv_data(gearman_server_con_st *con, void *data, size_t data_size)
 {
   gearmand_io_st *connection= &con->con;
 
@@ -136,9 +178,13 @@ gearmand_error_t gearmand_connection_recv_data(gearman_server_con_st *con, void
   if (connection->recv_buffer_size > 0)
   {
     if (connection->recv_buffer_size < data_size)
+    {
       recv_size= connection->recv_buffer_size;
+    }
     else
+    {
       recv_size= data_size;
+    }
 
     memcpy(data, connection->recv_buffer_ptr, recv_size);
     connection->recv_buffer_ptr+= recv_size;
@@ -187,24 +233,25 @@ static gearmand_error_t _connection_flush(gearman_server_con_st *con)
 
         if (write_size == 0) // detect infinite loop?
         {
-          gearmand_log_debug("send() sent zero bytes to peer %s:%s",
+          gearmand_log_debug(GEARMAN_DEFAULT_LOG_PARAM, "send() sent zero bytes to peer %s:%s",
                              connection->context == NULL ? "-" : connection->context->host,
                              connection->context == NULL ? "-" : connection->context->port);
           continue;
         }
         else if (write_size == -1)
         {
-          gearmand_error_t gret;
-
-          switch (errno)
+          int local_errno= errno;
+          switch (local_errno)
           {
           case EAGAIN:
-            gret= gearmand_io_set_events(con, POLLOUT);
-            if (gret != GEARMAN_SUCCESS)
             {
-              return gret;
+              gearmand_error_t gret= gearmand_io_set_events(con, POLLOUT);
+              if (gret != GEARMAN_SUCCESS)
+              {
+                return gret;
+              }
+              return GEARMAN_IO_WAIT;
             }
-            return GEARMAN_IO_WAIT;
 
           case EINTR:
             continue;
@@ -212,7 +259,7 @@ static gearmand_error_t _connection_flush(gearman_server_con_st *con)
           case EPIPE:
           case ECONNRESET:
           case EHOSTDOWN:
-            gearmand_perror("lost connection to client during send(EPIPE || ECONNRESET || EHOSTDOWN)");
+            gearmand_perror(local_errno, "lost connection to client during send(EPIPE || ECONNRESET || EHOSTDOWN)");
             _connection_close(connection);
             return GEARMAN_LOST_CONNECTION;
 
@@ -220,11 +267,16 @@ static gearmand_error_t _connection_flush(gearman_server_con_st *con)
             break;
           }
 
-          gearmand_perror("send() failed, closing connection");
+          gearmand_perror(local_errno, "send() failed, closing connection");
           _connection_close(connection);
           return GEARMAN_ERRNO;
         }
 
+        gearmand_log_debug(GEARMAN_DEFAULT_LOG_PARAM, "send() %u bytes to peer %s:%s",
+                           uint32_t(write_size),
+                           connection->context == NULL ? "-" : connection->context->host,
+                           connection->context == NULL ? "-" : connection->context->port);
+
         connection->send_buffer_size-= static_cast<size_t>(write_size);
         if (connection->send_state == gearmand_io_st::GEARMAND_CON_SEND_UNIVERSAL_FLUSH_DATA)
         {
@@ -302,12 +354,7 @@ void gearmand_connection_init(gearmand_connection_list_st *gearman,
   connection->recv_data_offset= 0;
   connection->universal= gearman;
 
-  if (gearman->con_list != NULL)
-    gearman->con_list->prev= connection;
-  connection->next= gearman->con_list;
-  connection->prev= NULL;
-  gearman->con_list= connection;
-  gearman->con_count++;
+  GEARMAN_LIST__ADD(gearman->con, connection);
 
   connection->context= dcon;
 
@@ -316,23 +363,45 @@ void gearmand_connection_init(gearmand_connection_list_st *gearman,
   connection->recv_buffer_ptr= connection->recv_buffer;
 }
 
+void gearmand_connection_list_st::list_free()
+{
+  while (con_list)
+  {
+    gearmand_io_free(con_list);
+  }
+}
+
+gearmand_connection_list_st::gearmand_connection_list_st() :
+  con_count(0),
+  con_list(NULL),
+  event_watch_fn(NULL),
+  event_watch_context(NULL)
+{
+}
+
+void gearmand_connection_list_st::init(gearmand_event_watch_fn *watch_fn, void *watch_context)
+{
+  ready_con_count= 0;
+  ready_con_list= NULL;
+  con_count= 0;
+  con_list= NULL;
+  event_watch_fn= watch_fn;
+  event_watch_context= watch_context;
+}
+
 void gearmand_io_free(gearmand_io_st *connection)
 {
   if (connection->fd != INVALID_SOCKET)
     _connection_close(connection);
 
+  if (connection->options.ready)
   {
-    if (connection->universal->con_list == connection)
-      connection->universal->con_list= connection->next;
-
-    if (connection->prev != NULL)
-      connection->prev->next= connection->next;
-
-    if (connection->next != NULL)
-      connection->next->prev= connection->prev;
-    connection->universal->con_count--;
+    connection->options.ready= false;
+    GEARMAN_LIST_DEL(connection->universal->ready_con, connection, ready_);
   }
 
+  GEARMAN_LIST__DEL(connection->universal->con, connection);
+
   if (connection->options.packet_in_use)
   {
     gearmand_packet_free(&(connection->packet));
@@ -345,9 +414,6 @@ gearmand_error_t gearman_io_set_option(gearmand_io_st *connection,
 {
   switch (options)
   {
-  case GEARMAND_CON_READY:
-    connection->options.ready= value;
-    break;
   case GEARMAND_CON_PACKET_IN_USE:
     connection->options.packet_in_use= value;
     break;
@@ -394,7 +460,6 @@ gearmand_con_st *gearman_io_context(const gearmand_io_st *connection)
 gearmand_error_t gearman_io_send(gearman_server_con_st *con,
                                  const gearmand_packet_st *packet, bool flush)
 {
-  gearmand_error_t ret;
   size_t send_size;
 
   gearmand_io_st *connection= &con->con;
@@ -411,10 +476,12 @@ gearmand_error_t gearman_io_send(gearman_server_con_st *con,
     /* Pack first part of packet, which is everything but the payload. */
     while (1)
     {
-      send_size= con->protocol.packet_pack_fn(packet, con,
-                                              connection->send_buffer + connection->send_buffer_size,
-                                              GEARMAN_SEND_BUFFER_SIZE -
-                                              connection->send_buffer_size, &ret);
+      gearmand_error_t ret;
+      send_size= con->protocol->pack(packet,
+                                     con,
+                                     connection->send_buffer +connection->send_buffer_size,
+                                     GEARMAN_SEND_BUFFER_SIZE -connection->send_buffer_size,
+                                     ret);
       if (ret == GEARMAN_SUCCESS)
       {
         connection->send_buffer_size+= send_size;
@@ -441,10 +508,12 @@ gearmand_error_t gearman_io_send(gearman_server_con_st *con,
       connection->send_state= gearmand_io_st::GEARMAND_CON_SEND_UNIVERSAL_PRE_FLUSH;
 
     case gearmand_io_st::GEARMAND_CON_SEND_UNIVERSAL_PRE_FLUSH:
-      ret= _connection_flush(con);
-      if (ret != GEARMAN_SUCCESS)
       {
-        return ret;
+        gearmand_error_t local_ret;
+        if ((local_ret= _connection_flush(con)) != GEARMAN_SUCCESS)
+        {
+          return local_ret;
+        }
       }
     }
 
@@ -459,9 +528,12 @@ gearmand_error_t gearman_io_send(gearman_server_con_st *con,
     {
       connection->send_data_offset= GEARMAN_SEND_BUFFER_SIZE - connection->send_buffer_size;
       if (connection->send_data_offset > packet->data_size)
+      {
         connection->send_data_offset= packet->data_size;
+      }
 
-      memcpy(connection->send_buffer + connection->send_buffer_size, packet->data,
+      memcpy(connection->send_buffer +connection->send_buffer_size,
+             packet->data,
              connection->send_data_offset);
       connection->send_buffer_size+= connection->send_data_offset;
 
@@ -477,10 +549,12 @@ gearmand_error_t gearman_io_send(gearman_server_con_st *con,
     connection->send_state= gearmand_io_st::GEARMAND_CON_SEND_UNIVERSAL_FORCE_FLUSH;
 
   case gearmand_io_st::GEARMAND_CON_SEND_UNIVERSAL_FORCE_FLUSH:
-    ret= _connection_flush(con);
-    if (ret != GEARMAN_SUCCESS)
     {
-      return ret;
+      gearmand_error_t local_ret;
+      if ((local_ret= _connection_flush(con)) != GEARMAN_SUCCESS)
+      {
+        return local_ret;
+      }
     }
 
     connection->send_data_size= packet->data_size;
@@ -509,27 +583,30 @@ gearmand_error_t gearman_io_send(gearman_server_con_st *con,
 
   case gearmand_io_st::GEARMAND_CON_SEND_UNIVERSAL_FLUSH:
   case gearmand_io_st::GEARMAND_CON_SEND_UNIVERSAL_FLUSH_DATA:
-    ret= _connection_flush(con);
-    if (ret == GEARMAN_SUCCESS && connection->options.close_after_flush)
     {
-      _connection_close(connection);
-      ret= GEARMAN_LOST_CONNECTION;
-      gearmand_debug("closing connection after flush by request");
+      gearmand_error_t local_ret= _connection_flush(con);
+      if (local_ret == GEARMAN_SUCCESS and
+          connection->options.close_after_flush)
+      {
+        _connection_close(connection);
+        local_ret= GEARMAN_LOST_CONNECTION;
+        gearmand_debug("closing connection after flush by request");
+      }
+      return local_ret;
     }
-    return ret;
   }
 
   if (flush)
   {
     connection->send_state= gearmand_io_st::GEARMAND_CON_SEND_UNIVERSAL_FLUSH;
-    ret= _connection_flush(con);
-    if (ret == GEARMAN_SUCCESS and connection->options.close_after_flush)
+    gearmand_error_t local_ret= _connection_flush(con);
+    if (local_ret == GEARMAN_SUCCESS and connection->options.close_after_flush)
     {
       _connection_close(connection);
-      ret= GEARMAN_LOST_CONNECTION;
+      local_ret= GEARMAN_LOST_CONNECTION;
       gearmand_debug("closing connection after flush by request");
     }
-    return ret;
+    return local_ret;
   }
 
   connection->send_state= gearmand_io_st::GEARMAND_CON_SEND_STATE_NONE;
@@ -557,7 +634,7 @@ gearmand_error_t gearman_io_recv(gearman_server_con_st *con, bool recv_data)
 
     connection->recv_packet= packet;
     // The options being passed in are just defaults.
-    gearmand_packet_init(connection->recv_packet, GEARMAN_MAGIC_TEXT, GEARMAN_COMMAND_TEXT);
+    connection->recv_packet->reset(GEARMAN_MAGIC_TEXT, GEARMAN_COMMAND_TEXT);
 
     connection->recv_state= gearmand_io_st::GEARMAND_CON_RECV_UNIVERSAL_READ;
 
@@ -568,9 +645,11 @@ gearmand_error_t gearman_io_recv(gearman_server_con_st *con, bool recv_data)
 
       if (connection->recv_buffer_size > 0)
       {
-	size_t recv_size= con->protocol.packet_unpack_fn(connection->recv_packet, con,
-							 connection->recv_buffer_ptr,
-							 connection->recv_buffer_size, &ret);
+        assert(con->protocol);
+        size_t recv_size= con->protocol->unpack(connection->recv_packet,
+                                                con,
+                                                connection->recv_buffer_ptr,
+                                                connection->recv_buffer_size, ret);
         connection->recv_buffer_ptr+= recv_size;
         connection->recv_buffer_size-= recv_size;
         if (gearmand_success(ret))
@@ -579,7 +658,7 @@ gearmand_error_t gearman_io_recv(gearman_server_con_st *con, bool recv_data)
         }
         else if (ret != GEARMAN_IO_WAIT)
         {
-	  gearmand_gerror_warn("protocol failure, closing connection", ret);
+          gearmand_gerror_warn("protocol failure, closing connection", ret);
           _connection_close(connection);
           return ret;
         }
@@ -692,7 +771,10 @@ gearmand_error_t gearmand_io_set_revents(gearman_server_con_st *con, short reven
   gearmand_io_st *connection= &con->con;
 
   if (revents != 0)
+  {
     connection->options.ready= true;
+    GEARMAN_LIST_ADD(connection->universal->ready_con, connection, ready_);
+  }
 
   connection->revents= revents;
 
@@ -724,108 +806,132 @@ gearmand_error_t gearmand_io_set_revents(gearman_server_con_st *con, short reven
 
 static gearmand_error_t _io_setsockopt(gearmand_io_st &connection)
 {
-  struct linger linger;
-  struct timeval waittime;
-
-  int setting= 1;
-  if (setsockopt(connection.fd, IPPROTO_TCP, TCP_NODELAY, &setting, (socklen_t)sizeof(int)) and errno != EOPNOTSUPP)
+  gearmand_log_debug(GEARMAN_DEFAULT_LOG_PARAM, "setsockopt() %d", connection.fd);
   {
-    gearmand_perror("setsockopt(TCP_NODELAY)");
-    return GEARMAN_ERRNO;
+    int setting= 1;
+    if (setsockopt(connection.fd, IPPROTO_TCP, TCP_NODELAY, &setting, (socklen_t)sizeof(int)) and errno != EOPNOTSUPP)
+    {
+      return gearmand_perror(errno, "setsockopt(TCP_NODELAY)");
+    }
   }
 
-  linger.l_onoff= 1;
-  linger.l_linger= GEARMAN_DEFAULT_SOCKET_TIMEOUT;
-  if (setsockopt(connection.fd, SOL_SOCKET, SO_LINGER, &linger, (socklen_t)sizeof(struct linger)))
   {
-    gearmand_perror("setsockopt(SO_LINGER)");
-    return GEARMAN_ERRNO;
+    struct linger linger;
+    linger.l_onoff= 1;
+    linger.l_linger= GEARMAN_DEFAULT_SOCKET_TIMEOUT;
+    if (setsockopt(connection.fd, SOL_SOCKET, SO_LINGER, &linger, (socklen_t)sizeof(struct linger)))
+    {
+      return gearmand_perror(errno, "setsockopt(SO_LINGER)");
+    }
   }
 
 #if defined(__MACH__) && defined(__APPLE__) || defined(__FreeBSD__)
   {
-    setting= 1;
+    int setting= 1;
 
     // This is not considered a fatal error 
     if (setsockopt(connection.fd, SOL_SOCKET, SO_NOSIGPIPE, (void *)&setting, sizeof(int)))
     {
-      gearmand_perror("setsockopt(SO_NOSIGPIPE)");
+      gearmand_perror(errno, "setsockopt(SO_NOSIGPIPE)");
     }
   }
 #endif
 
-  waittime.tv_sec= GEARMAN_DEFAULT_SOCKET_TIMEOUT;
-  waittime.tv_usec= 0;
-  if (setsockopt(connection.fd, SOL_SOCKET, SO_SNDTIMEO, &waittime, (socklen_t)sizeof(struct timeval)) and errno != ENOPROTOOPT)
+  if (0)
   {
-    gearmand_perror("setsockopt(SO_SNDTIMEO)");
-    return GEARMAN_ERRNO;
+    struct timeval waittime;
+    waittime.tv_sec= GEARMAN_DEFAULT_SOCKET_TIMEOUT;
+    waittime.tv_usec= 0;
+    if (setsockopt(connection.fd, SOL_SOCKET, SO_SNDTIMEO, &waittime, (socklen_t)sizeof(struct timeval)) and errno != ENOPROTOOPT)
+    {
+      return gearmand_perror(errno, "setsockopt(SO_SNDTIMEO)");
+    }
+
+    if (setsockopt(connection.fd, SOL_SOCKET, SO_RCVTIMEO, &waittime, (socklen_t)sizeof(struct timeval)) and errno != ENOPROTOOPT)
+    {
+      return gearmand_perror(errno, "setsockopt(SO_RCVTIMEO)");
+    }
   }
 
-  if (setsockopt(connection.fd, SOL_SOCKET, SO_RCVTIMEO, &waittime, (socklen_t)sizeof(struct timeval)) and errno != ENOPROTOOPT)
+  if (0)
   {
-    gearmand_error("setsockopt(SO_RCVTIMEO)");
-    return GEARMAN_ERRNO;
+    int setting= GEARMAN_DEFAULT_SOCKET_SEND_SIZE;
+    if (setsockopt(connection.fd, SOL_SOCKET, SO_SNDBUF, &setting, (socklen_t)sizeof(int)))
+    {
+      return gearmand_perror(errno, "setsockopr(SO_SNDBUF)");
+    }
+
+    setting= GEARMAN_DEFAULT_SOCKET_RECV_SIZE;
+    if (setsockopt(connection.fd, SOL_SOCKET, SO_RCVBUF, &setting, (socklen_t)sizeof(int)))
+    {
+      return gearmand_perror(errno, "setsockopt(SO_RCVBUF)");
+    }
   }
 
-  setting= GEARMAN_DEFAULT_SOCKET_SEND_SIZE;
-  if (setsockopt(connection.fd, SOL_SOCKET, SO_SNDBUF, &setting, (socklen_t)sizeof(int)))
+  if (SOCK_NONBLOCK == 0)
   {
-    gearmand_perror("setsockopt(SO_SNDBUF)");
-    return GEARMAN_ERRNO;
-  }
+    int flags;
+    do
+    {
+      flags= fcntl(connection.fd, F_GETFL, 0);
+    } while (flags == -1 and (errno == EINTR or errno == EAGAIN));
 
-  setting= GEARMAN_DEFAULT_SOCKET_RECV_SIZE;
-  if (setsockopt(connection.fd, SOL_SOCKET, SO_RCVBUF, &setting, (socklen_t)sizeof(int)))
-  {
-    gearmand_perror("setsockopt(SO_RCVBUF)");
-    return GEARMAN_ERRNO;
-  }
+    if (flags == -1)
+    {
+      return gearmand_perror(errno, "fcntl(F_GETFL)");
+    }
+    else if ((flags & O_NONBLOCK) == 0)
+    {
+      int retval;
+      do
+      {
+        retval= fcntl(connection.fd, F_SETFL, flags | O_NONBLOCK);
+      } while (retval == -1 and (errno == EINTR or errno == EAGAIN));
 
-  int fcntl_flags;
-  if ((fcntl_flags= fcntl(connection.fd, F_GETFL, 0)) == -1)
-  {
-    gearmand_perror("fcntl(F_GETFL)");
-    return GEARMAN_ERRNO;
-  }
-
-  if ((fcntl(connection.fd, F_SETFL, fcntl_flags | O_NONBLOCK) == -1))
-  {
-    gearmand_perror("fcntl(F_SETFL)");
-    return GEARMAN_ERRNO;
+      if (retval == -1)
+      {
+        return gearmand_perror(errno, "fcntl(F_SETFL)");
+      }
+    }
   }
 
   return GEARMAN_SUCCESS;
 }
 
-void gearmand_sockfd_close(int sockfd)
+void gearmand_sockfd_close(int& sockfd)
 {
   if (sockfd == INVALID_SOCKET)
+  {
+    gearmand_error("gearmand_sockfd_close() called with an invalid socket");
     return;
+  }
 
   /* in case of death shutdown to avoid blocking at close() */
   if (shutdown(sockfd, SHUT_RDWR) == SOCKET_ERROR && get_socket_errno() != ENOTCONN)
   {
-    gearmand_perror("shutdown");
+    gearmand_perror(errno, "shutdown");
     assert(errno != ENOTSOCK);
-    return;
+  }
+  else if (closesocket(sockfd) == SOCKET_ERROR)
+  {
+    gearmand_perror(errno, "close");
   }
 
-  if (closesocket(sockfd) == SOCKET_ERROR)
-  {
-    gearmand_perror("close");
-  }
+  sockfd= INVALID_SOCKET;
 }
 
-void gearmand_pipe_close(int pipefd)
+void gearmand_pipe_close(int& pipefd)
 {
   if (pipefd == INVALID_SOCKET)
   {
+    gearmand_error("gearmand_pipe_close() called with an invalid socket");
     return;
   }
 
   if (closesocket(pipefd) == SOCKET_ERROR)
   {
-    gearmand_perror("close");
+    gearmand_perror(errno, "close");
   }
+
+  pipefd= -1;
 }
