@@ -1,9 +1,39 @@
-/* Gearman server and library
- * Copyright (C) 2008-2009 Brian Aker, Eric Day
- * All rights reserved.
+/*  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ * 
+ *  Gearmand client and server library.
+ *
+ *  Copyright (C) 2011-2013 Data Differential, http://datadifferential.com/
+ *  Copyright (C) 2008 Brian Aker, Eric Day
+ *  All rights reserved.
+ *
+ *  Redistribution and use in source and binary forms, with or without
+ *  modification, are permitted provided that the following conditions are
+ *  met:
+ *
+ *      * Redistributions of source code must retain the above copyright
+ *  notice, this list of conditions and the following disclaimer.
+ *
+ *      * Redistributions in binary form must reproduce the above
+ *  copyright notice, this list of conditions and the following disclaimer
+ *  in the documentation and/or other materials provided with the
+ *  distribution.
+ *
+ *      * The names of its contributors may not be used to endorse or
+ *  promote products derived from this software without specific prior
+ *  written permission.
+ *
+ *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+ *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
+ *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+ *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+ *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+ *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+ *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+ *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+ *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- * Use and distribution licensed under the BSD license.  See
- * the COPYING file in the parent directory for full text.
  */
 
 /**
@@ -11,19 +41,36 @@
  * @brief Gearman State Definitions
  */
 
-#include <config.h>
-#include <libgearman-server/common.h>
-#include <libgearman-server/timer.h>
+#include "gear_config.h"
+
+#include "libgearman-server/common.h"
+#include "libgearman-server/timer.h"
 
 #include <algorithm>
 #include <cerrno>
+#include <cstddef>
+#include <cstdlib>
 #include <cstring>
 #include <ctime>
+#include <pthread.h>
+
+#ifdef _WIN32
+# include <malloc.h>
+#else
+# include <alloca.h>
+#endif
+
+#ifndef __INTEL_COMPILER
+# pragma GCC diagnostic ignored "-Wold-style-cast"
+# pragma GCC diagnostic ignored "-Wformat-nonliteral"
+# pragma GCC diagnostic ignored "-Wformat-security"
+#endif
+
 
 static pthread_key_t logging_key;
 static pthread_once_t intitialize_log_once= PTHREAD_ONCE_INIT;
 
-static void delete_log(void *ptr)
+static void delete_log(void * ptr)
 {
   if (ptr)
   {
@@ -33,24 +80,40 @@ static void delete_log(void *ptr)
 
 static void create_log(void)
 {
-  (void) pthread_key_create(&logging_key, delete_log);
-}
-
-void gearmand_initialize_thread_logging(const char *identity)
-{
-  (void) pthread_once(&intitialize_log_once, create_log);
-
-  void *ptr;
-  if ((ptr= pthread_getspecific(logging_key)) == NULL)
+  int pthread_error;
+  if ((pthread_error= pthread_key_create(&logging_key, delete_log)))
   {
-    const char *key_to_use= strdup(identity);
-    (void) pthread_setspecific(logging_key, key_to_use);
+    gearmand_log_fatal_perror(GEARMAND_AT, "pthread_key_create", pthread_error, "pthread_key_create");
+    abort();
   }
 }
 
-#ifndef __INTEL_COMPILER
-#pragma GCC diagnostic ignored "-Wold-style-cast"
-#endif
+gearmand_error_t gearmand_initialize_thread_logging(const char *identity)
+{
+  if (identity)
+  {
+    int pthread_error;
+    if ((pthread_error= pthread_once(&intitialize_log_once, create_log)))
+    {
+      return gearmand_log_fatal_perror(GEARMAND_AT, "pthread_once", pthread_error, "identity: %s", identity);
+    }
+
+    if (pthread_getspecific(logging_key) == NULL)
+    {
+      const char *key_to_use= strdup(identity);
+      if ((pthread_error= pthread_setspecific(logging_key, key_to_use)))
+      {
+        return gearmand_log_fatal_perror(GEARMAND_AT, "pthread_setspecific", pthread_error, "identity: %s", identity);
+      }
+    }
+
+    return GEARMAN_SUCCESS;
+  }
+
+  gearmand_fatal("identity was NULL");
+
+  return GEARMAN_INVALID_ARGUMENT;
+}
 
 /**
  * Log a message.
@@ -80,9 +143,6 @@ static void gearmand_log(const char *position, const char *func /* func */,
   }
 
   struct tm current_tm;
-  const char *current_time_str_ptr;
-  char current_time_str[27];
-
   if (current_epoch.tv_sec == 0)
   {
     (void)gettimeofday(&current_epoch, NULL);
@@ -90,22 +150,7 @@ static void gearmand_log(const char *position, const char *func /* func */,
 
   if ((gmtime_r(&current_epoch.tv_sec, &current_tm) == NULL))
   {
-    current_time_str_ptr = "NA";
-  }
-  else
-  {
-    snprintf(current_time_str, sizeof(current_time_str), "%04d-%02d-%02d %02d:%02d:%02d.%06d",
-             int(1900 + current_tm.tm_year), current_tm.tm_mon, current_tm.tm_mday, current_tm.tm_hour,
-             current_tm.tm_min, current_tm.tm_sec, int(current_epoch.tv_usec));
-    current_time_str_ptr = current_time_str;
-  }
-
-  if (Gearmand() == NULL)
-  {
-    fprintf(stderr, "%s %s %7s: ", current_time_str_ptr, position,  gearmand_verbose_name(verbose));
-    vfprintf(stderr, format, args);
-    fprintf(stderr, "\n");
-    return;
+    memset(&current_epoch, 0, sizeof(current_epoch));
   }
 
   (void) pthread_once(&intitialize_log_once, create_log);
@@ -117,16 +162,19 @@ static void gearmand_log(const char *position, const char *func /* func */,
     identity= "[  main ]";
   }
 
+  char log_buffer[GEARMAN_MAX_ERROR_SIZE*2] = { 0 };
   if (Gearmand() && Gearmand()->log_fn)
   {
-    char log_buffer[GEARMAN_MAX_ERROR_SIZE*2];
     char *log_buffer_ptr= log_buffer;
     size_t remaining_size= sizeof(log_buffer);
 
     {
-      int length= snprintf(log_buffer, sizeof(log_buffer), "%s %s ", current_time_str_ptr, identity);
+      int length= snprintf(log_buffer, sizeof(log_buffer), "%04d-%02d-%02d %02d:%02d:%02d.%06d %s ",
+                           int(1900 +current_tm.tm_year), current_tm.tm_mon +1, current_tm.tm_mday, current_tm.tm_hour,
+                           current_tm.tm_min, current_tm.tm_sec, int(current_epoch.tv_usec),
+                           identity);
       // We just return whatever we have if this occurs
-      if (length < -1 || (size_t)length >= sizeof(log_buffer))
+      if (length <= 0 or (size_t)length >= sizeof(log_buffer))
       {
         remaining_size= 0;
       }
@@ -140,7 +188,7 @@ static void gearmand_log(const char *position, const char *func /* func */,
     if (remaining_size)
     {
       int length= vsnprintf(log_buffer_ptr, remaining_size, format, args);
-      if (length < -1 or size_t(length) >= remaining_size)
+      if (length <= 0 or size_t(length) >= remaining_size)
       { 
         remaining_size= 0;
       }
@@ -154,8 +202,10 @@ static void gearmand_log(const char *position, const char *func /* func */,
     if (remaining_size and error_arg != GEARMAN_SUCCESS)
     {
       int length= snprintf(log_buffer_ptr, remaining_size, " %s(%s)", func, gearmand_strerror(error_arg));
-      if (length < -1 or size_t(length) >= remaining_size)
-      { }
+      if (length <= 0 or size_t(length) >= remaining_size)
+      {
+        remaining_size= 0;
+      }
       else
       {
         remaining_size-= size_t(length);
@@ -165,60 +215,113 @@ static void gearmand_log(const char *position, const char *func /* func */,
 
     if (remaining_size and position and verbose != GEARMAND_VERBOSE_INFO)
     {
-      snprintf(log_buffer_ptr, remaining_size, " -> %s", position);
+      int length= snprintf(log_buffer_ptr, remaining_size, " -> %s", position);
+      if (length <= 0 or size_t(length) >= remaining_size)
+      {
+        remaining_size= 0;
+      }
     }
 
     // Make sure this is null terminated
-    log_buffer[sizeof(log_buffer)]= 0;
+    log_buffer[sizeof(log_buffer) -1]= 0;
+  }
 
+  if (Gearmand() and Gearmand()->log_fn)
+  {
     Gearmand()->log_fn(log_buffer, verbose, (void *)Gearmand()->log_context);
   }
   else
   {
-    fprintf(stderr, "%s %s %7s: ", current_time_str_ptr, identity,  gearmand_verbose_name(verbose));
+    fprintf(stderr, "%s -> %s",
+            log_buffer,  gearmand_verbose_name(verbose));
     vfprintf(stderr, format, args);
     fprintf(stderr, "\n");
   }
 }
 
 
-void gearmand_log_fatal(const char *position, const char *func, const char *format, ...)
+gearmand_error_t gearmand_log_fatal(const char *position, const char *func, const char *format, ...)
 {
-  va_list args;
-
-  if (not Gearmand() || Gearmand()->verbose >= GEARMAND_VERBOSE_FATAL)
+  if (Gearmand() and  Gearmand()->verbose < GEARMAND_VERBOSE_FATAL)
   {
+    return GEARMAN_ERRNO;
+  }
+
+  {
+    va_list args;
     va_start(args, format);
     gearmand_log(position, func, GEARMAND_VERBOSE_FATAL, GEARMAN_SUCCESS, format, args);
     va_end(args);
   }
+
+  return GEARMAN_ERRNO;
 }
 
-void gearmand_log_fatal_perror(const char *position, const char *function, const char *message)
+gearmand_error_t gearmand_log_fatal_perror(const char *position, const char *function, const int local_errno, const char *format, ...)
 {
-  if (not Gearmand() || Gearmand()->verbose >= GEARMAND_VERBOSE_FATAL)
+  if (Gearmand() and  Gearmand()->verbose < GEARMAND_VERBOSE_FATAL)
+  {
+    return GEARMAN_ERRNO;
+  }
+
+  char* message_buffer= NULL;
+  {
+    va_list args;
+    va_start(args, format);
+
+    size_t ask= snprintf(0, 0, format);
+    ask++; // for null
+    message_buffer= (char*)alloca(sizeof(char) * ask);
+    if (message_buffer)
+    {
+      vsnprintf(message_buffer, ask, format, args);
+    }
+    va_end(args);
+  }
+
   {
     const char *errmsg_ptr;
     char errmsg[GEARMAN_MAX_ERROR_SIZE]; 
     errmsg[0]= 0; 
 
 #ifdef STRERROR_R_CHAR_P
-    errmsg_ptr= strerror_r(errno, errmsg, sizeof(errmsg));
+    errmsg_ptr= strerror_r(local_errno, errmsg, sizeof(errmsg));
 #else
-    strerror_r(errno, errmsg, sizeof(errmsg));
+    strerror_r(local_errno, errmsg, sizeof(errmsg));
     errmsg_ptr= errmsg;
 #endif
 
-    gearmand_log_fatal(position, function, "%s(%s)", message, errmsg_ptr);
+    if (message_buffer)
+    {
+      gearmand_log_fatal(position, function, "%s(%s)", message_buffer, errmsg_ptr);
+    }
+    else
+    {
+      gearmand_log_fatal(position, function, "%s", errmsg_ptr);
+    }
   }
+
+  switch (local_errno)
+  {
+  case ENOMEM:
+    return GEARMAN_MEMORY_ALLOCATION_FAILURE;
+
+  case ECONNRESET:
+  case EHOSTDOWN:
+    return GEARMAN_LOST_CONNECTION;
+
+  default:
+    break;
+  }
+
+  return GEARMAN_ERRNO;
 }
 
 gearmand_error_t gearmand_log_error(const char *position, const char *function, const char *format, ...)
 {
-  va_list args;
-
   if (not Gearmand() or Gearmand()->verbose >= GEARMAND_VERBOSE_ERROR)
   {
+    va_list args;
     va_start(args, format);
     gearmand_log(position, function, GEARMAND_VERBOSE_ERROR, GEARMAN_SUCCESS, format, args);
     va_end(args);
@@ -229,10 +332,9 @@ gearmand_error_t gearmand_log_error(const char *position, const char *function,
 
 void gearmand_log_warning(const char *position, const char *function, const char *format, ...)
 {
-  va_list args;
-
-  if (not Gearmand() || Gearmand()->verbose >= GEARMAND_VERBOSE_WARN)
+  if (not Gearmand() or Gearmand()->verbose >= GEARMAND_VERBOSE_WARN)
   {
+    va_list args;
     va_start(args, format);
     gearmand_log(position, function, GEARMAND_VERBOSE_WARN, GEARMAN_SUCCESS, format, args);
     va_end(args);
@@ -242,10 +344,9 @@ void gearmand_log_warning(const char *position, const char *function, const char
 // LOG_NOTICE is only used for reporting job status.
 void gearmand_log_notice(const char *position, const char *function, const char *format, ...)
 {
-  va_list args;
-
-  if (not Gearmand() || Gearmand()->verbose >= GEARMAND_VERBOSE_NOTICE)
+  if (not Gearmand() or Gearmand()->verbose >= GEARMAND_VERBOSE_NOTICE)
   {
+    va_list args;
     va_start(args, format);
     gearmand_log(position, function, GEARMAND_VERBOSE_NOTICE, GEARMAN_SUCCESS, format, args);
     va_end(args);
@@ -254,10 +355,9 @@ void gearmand_log_notice(const char *position, const char *function, const char
 
 void gearmand_log_info(const char *position, const char *function, const char *format, ...)
 {
-  va_list args;
-
-  if (not Gearmand() || Gearmand()->verbose >= GEARMAND_VERBOSE_INFO)
+  if (not Gearmand() or Gearmand()->verbose >= GEARMAND_VERBOSE_INFO)
   {
+    va_list args;
     va_start(args, format);
     gearmand_log(position, function, GEARMAND_VERBOSE_INFO, GEARMAN_SUCCESS, format, args);
     va_end(args);
@@ -276,21 +376,54 @@ void gearmand_log_debug(const char *position, const char *function, const char *
   }
 }
 
-gearmand_error_t gearmand_log_perror(const char *position, const char *function, const char *message)
+gearmand_error_t gearmand_log_perror(const char *position, const char *function, const int local_errno, const char *format, ...)
 {
   if (not Gearmand() or (Gearmand()->verbose >= GEARMAND_VERBOSE_ERROR))
   {
+    char* message_buffer= NULL;
+    {
+      va_list args;
+      va_start(args, format);
+
+      size_t ask= snprintf(0, 0, format);
+      ask++; // for null
+      message_buffer= (char*)alloca(sizeof(char) * ask);
+      vsnprintf(message_buffer, ask, format, args);
+
+      va_end(args);
+    }
+
     const char *errmsg_ptr;
     char errmsg[GEARMAN_MAX_ERROR_SIZE]; 
     errmsg[0]= 0; 
 
 #ifdef STRERROR_R_CHAR_P
-    errmsg_ptr= strerror_r(errno, errmsg, sizeof(errmsg));
+    errmsg_ptr= strerror_r(local_errno, errmsg, sizeof(errmsg));
 #else
-    strerror_r(errno, errmsg, sizeof(errmsg));
+    strerror_r(local_errno, errmsg, sizeof(errmsg));
     errmsg_ptr= errmsg;
 #endif
-    gearmand_log_error(position, function, "%s(%s)", message, errmsg_ptr);
+    if (message_buffer)
+    {
+      gearmand_log_error(position, function, "%s(%s)", message_buffer, errmsg_ptr);
+    }
+    else
+    {
+      gearmand_log_error(position, function, "%s", errmsg_ptr);
+    }
+  }
+
+  switch (local_errno)
+  {
+  case ENOMEM:
+    return GEARMAN_MEMORY_ALLOCATION_FAILURE;
+
+  case ECONNRESET:
+  case EHOSTDOWN:
+    return GEARMAN_LOST_CONNECTION;
+
+  default:
+    break;
   }
 
   return GEARMAN_ERRNO;
@@ -338,7 +471,7 @@ gearmand_error_t gearmand_log_gai_error(const char *position, const char *functi
 {
   if (rc == EAI_SYSTEM)
   {
-    return gearmand_log_perror(position, function, message);
+    return gearmand_log_perror(position, function, errno, message);
   }
 
   gearmand_log_error(position, function, "%s getaddrinfo(%s)", message, gai_strerror(rc));
