@@ -1,9 +1,39 @@
-/* Gearman server and library
- * Copyright (C) 2008 Brian Aker, Eric Day
- * All rights reserved.
+/*  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ * 
+ *  Gearmand client and server library.
+ *
+ *  Copyright (C) 2011-2012 Data Differential, http://datadifferential.com/
+ *  Copyright (C) 2008 Brian Aker, Eric Day
+ *  All rights reserved.
+ *
+ *  Redistribution and use in source and binary forms, with or without
+ *  modification, are permitted provided that the following conditions are
+ *  met:
+ *
+ *      * Redistributions of source code must retain the above copyright
+ *  notice, this list of conditions and the following disclaimer.
+ *
+ *      * Redistributions in binary form must reproduce the above
+ *  copyright notice, this list of conditions and the following disclaimer
+ *  in the documentation and/or other materials provided with the
+ *  distribution.
+ *
+ *      * The names of its contributors may not be used to endorse or
+ *  promote products derived from this software without specific prior
+ *  written permission.
+ *
+ *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+ *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
+ *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+ *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+ *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+ *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+ *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+ *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+ *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- * Use and distribution licensed under the BSD license.  See
- * the COPYING file in the parent directory for full text.
  */
 
 /**
@@ -11,14 +41,15 @@
  * @brief Server connection definitions
  */
 
-#include <config.h>
-#include <libgearman-server/common.h>
+#include "gear_config.h"
+#include "libgearman-server/common.h"
 
 #include <libgearman/command.h>
 
-#include <libgearman-server/fifo.h>
 #include <cassert>
+#include <cerrno>
 #include <cstring>
+#include <memory>
 
 #ifndef __INTEL_COMPILER
 #pragma GCC diagnostic ignored "-Wold-style-cast"
@@ -55,10 +86,10 @@ gearman_server_packet_create(gearman_server_thread_st *thread,
 
   if (server_packet == NULL)
   {
-    server_packet= (gearman_server_packet_st *)malloc(sizeof(gearman_server_packet_st));
+    server_packet= new (std::nothrow) gearman_server_packet_st;
     if (server_packet == NULL)
     {
-      gearmand_perror("malloc");
+      gearmand_perror(errno, "new() gearman_server_packet_st");
       return NULL;
     }
   }
@@ -68,11 +99,16 @@ gearman_server_packet_create(gearman_server_thread_st *thread,
   return server_packet;
 }
 
+void destroy_gearman_server_packet_st(gearman_server_packet_st *packet)
+{
+  delete packet;
+}
+
 void gearman_server_packet_free(gearman_server_packet_st *packet,
                                 gearman_server_thread_st *thread,
                                 bool from_thread)
 {
-  if (from_thread && Server->flags.threaded)
+  if (from_thread and Server->flags.threaded)
   {
     if (thread->free_packet_count < GEARMAN_MAX_FREE_SERVER_PACKET)
     {
@@ -82,8 +118,8 @@ void gearman_server_packet_free(gearman_server_packet_st *packet,
     }
     else
     {
-      gearmand_debug("free");
-      free(packet);
+      gearmand_debug("delete() gearman_server_packet_st");
+      delete packet;
     }
   }
   else
@@ -96,8 +132,8 @@ void gearman_server_packet_free(gearman_server_packet_st *packet,
     }
     else
     {
-      gearmand_debug("free");
-      free(packet);
+      gearmand_debug("delete() gearman_server_packet_st");
+      delete packet;
     }
   }
 }
@@ -112,10 +148,12 @@ gearmand_error_t gearman_server_io_packet_add(gearman_server_con_st *con,
   va_list ap;
 
   server_packet= gearman_server_packet_create(con->thread, false);
-  if (not server_packet)
+  if (server_packet == NULL)
+  {
     return GEARMAN_MEMORY_ALLOCATION_FAILURE;
+  }
 
-  gearmand_packet_init(&(server_packet->packet), magic, command);
+  server_packet->packet.reset(magic, command);
 
   va_start(ap, arg);
 
@@ -150,9 +188,19 @@ gearmand_error_t gearman_server_io_packet_add(gearman_server_con_st *con,
     server_packet->packet.options.free_data= true;
   }
 
-  (void) pthread_mutex_lock(&con->thread->lock);
-  gearmand_server_con_fifo_add(con, server_packet);
-  (void) pthread_mutex_unlock(&con->thread->lock);
+  int error;
+  if ((error= pthread_mutex_lock(&con->thread->lock)) == 0)
+  {
+    GEARMAN_FIFO__ADD(con->io_packet, server_packet);
+    if ((error= pthread_mutex_unlock(&con->thread->lock)))
+    {
+      gearmand_log_fatal_perror(GEARMAN_DEFAULT_LOG_PARAM, error, "pthread_mutex_unlock");
+    }
+  }
+  else
+  {
+    gearmand_log_fatal_perror(GEARMAN_DEFAULT_LOG_PARAM, error, "pthread_mutex_lock");
+  }
 
   gearman_server_con_io_add(con);
 
@@ -165,9 +213,19 @@ void gearman_server_io_packet_remove(gearman_server_con_st *con)
 
   gearmand_packet_free(&(server_packet->packet));
 
-  (void) pthread_mutex_lock(&con->thread->lock);
-  gearmand_server_con_fifo_free(con, server_packet);
-  (void) pthread_mutex_unlock(&con->thread->lock);
+  int error;
+  if ((error= pthread_mutex_lock(&con->thread->lock)) == 0)
+  {
+    GEARMAN_FIFO__DEL(con->io_packet, server_packet);
+    if ((error= pthread_mutex_unlock(&con->thread->lock)))
+    {
+      gearmand_log_fatal_perror(GEARMAN_DEFAULT_LOG_PARAM, error, "pthread_mutex_unlock");
+    }
+  }
+  else
+  {
+    gearmand_log_fatal_perror(GEARMAN_DEFAULT_LOG_PARAM, error, "pthread_mutex_lock");
+  }
 
   gearman_server_packet_free(server_packet, con->thread, true);
 }
@@ -175,9 +233,19 @@ void gearman_server_io_packet_remove(gearman_server_con_st *con)
 void gearman_server_proc_packet_add(gearman_server_con_st *con,
                                     gearman_server_packet_st *packet)
 {
-  (void) pthread_mutex_lock(&con->thread->lock);
-  gearmand_server_con_fifo_proc_add(con, packet);
-  (void) pthread_mutex_unlock(&con->thread->lock);
+  int error;
+  if ((error= pthread_mutex_lock(&con->thread->lock)) == 0)
+  {
+    GEARMAN_FIFO__ADD(con->proc_packet, packet);
+    if ((error= pthread_mutex_unlock(&con->thread->lock)))
+    {
+      gearmand_log_fatal_perror(GEARMAN_DEFAULT_LOG_PARAM, error, "pthread_mutex_unlock");
+    }
+  }
+  else
+  {
+    gearmand_log_fatal_perror(GEARMAN_DEFAULT_LOG_PARAM, error, "pthread_mutex_lock");
+  }
 
   gearman_server_con_proc_add(con);
 }
@@ -187,12 +255,22 @@ gearman_server_proc_packet_remove(gearman_server_con_st *con)
 {
   gearman_server_packet_st *server_packet= con->proc_packet_list;
 
-  if (server_packet == NULL)
-    return NULL;
-
-  (void) pthread_mutex_lock(&con->thread->lock);
-  gearmand_server_con_fifo_proc_free(con, server_packet);
-  (void) pthread_mutex_unlock(&con->thread->lock);
+  if (server_packet)
+  {
+    int error;
+    if ((error= pthread_mutex_lock(&con->thread->lock)) == 0)
+    {
+      GEARMAN_FIFO__DEL(con->proc_packet, server_packet);
+      if ((error= pthread_mutex_unlock(&con->thread->lock)) != 0)
+      {
+        gearmand_log_fatal_perror(GEARMAN_DEFAULT_LOG_PARAM, error, "pthread_mutex_unlock");
+      }
+    }
+    else
+    {
+      gearmand_log_fatal_perror(GEARMAN_DEFAULT_LOG_PARAM, error, "pthread_mutex_lock");
+    }
+  }
 
   return server_packet;
 }
@@ -206,10 +284,8 @@ const char *gearmand_strcommand(gearmand_packet_st *packet)
 inline static gearmand_error_t packet_create_arg(gearmand_packet_st *packet,
                                                  const void *arg, size_t arg_size)
 {
-  size_t offset;
-
-  if (packet->argc == gearman_command_info(packet->command)->argc &&
-      (not (gearman_command_info(packet->command)->data) ||
+  if (packet->argc == gearman_command_info(packet->command)->argc and
+      (not (gearman_command_info(packet->command)->data) or
        packet->data))
   {
     gearmand_log_error(GEARMAN_DEFAULT_LOG_PARAM, "too many arguments for command(%s)", gearman_command_info(packet->command)->name);
@@ -224,7 +300,9 @@ inline static gearmand_error_t packet_create_arg(gearmand_packet_st *packet,
   }
 
   if (packet->args_size == 0 and packet->magic != GEARMAN_MAGIC_TEXT)
+  {
     packet->args_size= GEARMAN_PACKET_HEADER_SIZE;
+  }
 
   if ((packet->args_size + arg_size) < GEARMAN_ARGS_BUFFER_SIZE)
   {
@@ -241,10 +319,9 @@ inline static gearmand_error_t packet_create_arg(gearmand_packet_st *packet,
     else
     {
       char *new_args= (char *)realloc(packet->args, packet->args_size + arg_size);
-      if (not new_args)
+      if (new_args == NULL)
       {
-        gearmand_perror("realloc");
-        return GEARMAN_MEMORY_ALLOCATION_FAILURE;
+        return gearmand_perror(errno, "realloc");
       }
       packet->args= new_args;
     }
@@ -255,6 +332,7 @@ inline static gearmand_error_t packet_create_arg(gearmand_packet_st *packet,
   packet->arg_size[packet->argc]= arg_size;
   packet->argc++;
 
+  size_t offset;
   if (packet->magic == GEARMAN_MAGIC_TEXT)
   {
     offset= 0;
@@ -264,7 +342,7 @@ inline static gearmand_error_t packet_create_arg(gearmand_packet_st *packet,
     offset= GEARMAN_PACKET_HEADER_SIZE;
   }
 
-  for (uint8_t x= 0; x < packet->argc; x++)
+  for (uint8_t x= 0; x < packet->argc; ++x)
   {
     packet->arg[x]= packet->args + offset;
     offset+= packet->arg_size[x];
@@ -280,21 +358,19 @@ inline static gearmand_error_t packet_create_arg(gearmand_packet_st *packet,
  */
 
 
-void gearmand_packet_init(gearmand_packet_st *packet, enum gearman_magic_t magic, gearman_command_t command)
+void gearmand_packet_st::reset(enum gearman_magic_t magic_, gearman_command_t command_)
 {
-  assert(packet);
+  options.complete= false;
+  options.free_data= false;
 
-  packet->options.complete= false;
-  packet->options.free_data= false;
+  magic= magic_;
+  command= command_;
+  argc= 0;
+  args_size= 0;
+  data_size= 0;
 
-  packet->magic= magic;
-  packet->command= command;
-  packet->argc= 0;
-  packet->args_size= 0;
-  packet->data_size= 0;
-
-  packet->args= NULL;
-  packet->data= NULL;
+  args= NULL;
+  data= NULL;
 }
 
 gearmand_error_t gearmand_packet_create(gearmand_packet_st *packet,
@@ -307,14 +383,14 @@ void gearmand_packet_free(gearmand_packet_st *packet)
 {
   if (packet->args != packet->args_buffer && packet->args != NULL)
   {
-    gearmand_debug("free");
+    gearmand_debug("free packet's args");
     free(packet->args);
     packet->args= NULL;
   }
 
   if (packet->options.free_data && packet->data != NULL)
   {
-    gearmand_debug("free");
+    gearmand_debug("free() packet's data");
     free((void *)packet->data); //@todo fix the need for the casting.
     packet->data= NULL;
   }
@@ -322,9 +398,6 @@ void gearmand_packet_free(gearmand_packet_st *packet)
 
 gearmand_error_t gearmand_packet_pack_header(gearmand_packet_st *packet)
 {
-  uint64_t length_64;
-  uint32_t tmp;
-
   if (packet->magic == GEARMAN_MAGIC_TEXT)
   {
     packet->options.complete= true;
@@ -362,11 +435,11 @@ gearmand_error_t gearmand_packet_pack_header(gearmand_packet_st *packet)
     return GEARMAN_INVALID_COMMAND;
   }
 
-  tmp= packet->command;
+  uint32_t tmp= packet->command;
   tmp= htonl(tmp);
   memcpy(packet->args + 4, &tmp, 4);
 
-  length_64= packet->args_size + packet->data_size - GEARMAN_PACKET_HEADER_SIZE;
+  uint64_t length_64= packet->args_size + packet->data_size - GEARMAN_PACKET_HEADER_SIZE;
 
   // Check for overflow on 32bit(portable?).
   if (length_64 >= UINT32_MAX || length_64 < packet->data_size)
@@ -383,179 +456,3 @@ gearmand_error_t gearmand_packet_pack_header(gearmand_packet_st *packet)
 
   return GEARMAN_SUCCESS;
 }
-
-static gearmand_error_t gearmand_packet_unpack_header(gearmand_packet_st *packet)
-{
-  uint32_t tmp;
-
-  if (not memcmp(packet->args, "\0REQ", 4))
-  {
-    packet->magic= GEARMAN_MAGIC_REQUEST;
-  }
-  else if (not memcmp(packet->args, "\0RES", 4))
-  {
-    packet->magic= GEARMAN_MAGIC_RESPONSE;
-  }
-  else
-  {
-    gearmand_error("invalid magic value");
-    return GEARMAN_INVALID_MAGIC;
-  }
-
-  memcpy(&tmp, packet->args + 4, 4);
-  packet->command= static_cast<gearman_command_t>(ntohl(tmp));
-
-  if (packet->command == GEARMAN_COMMAND_TEXT ||
-      packet->command >= GEARMAN_COMMAND_MAX)
-  {
-    gearmand_error("invalid command value");
-    return GEARMAN_INVALID_COMMAND;
-  }
-
-  memcpy(&tmp, packet->args + 8, 4);
-  packet->data_size= ntohl(tmp);
-
-  return GEARMAN_SUCCESS;
-}
-
-size_t gearmand_packet_pack(const gearmand_packet_st *packet,
-                            gearman_server_con_st *con __attribute__ ((unused)),
-                            void *data, size_t data_size,
-                            gearmand_error_t *ret_ptr)
-{
-  if (packet->args_size == 0)
-  {
-    *ret_ptr= GEARMAN_SUCCESS;
-    return 0;
-  }
-
-  if (packet->args_size > data_size)
-  {
-    *ret_ptr= GEARMAN_FLUSH_DATA;
-    return 0;
-  }
-
-  memcpy(data, packet->args, packet->args_size);
-  *ret_ptr= GEARMAN_SUCCESS;
-  return packet->args_size;
-}
-
-size_t gearmand_packet_unpack(gearmand_packet_st *packet,
-                              gearman_server_con_st *con __attribute__ ((unused)),
-                              const void *data, size_t data_size,
-                              gearmand_error_t *ret_ptr)
-{
-  uint8_t *ptr;
-  size_t used_size;
-
-  if (packet->args_size == 0)
-  {
-    if (data_size > 0 && ((uint8_t *)data)[0] != 0)
-    {
-      /* Try to parse a text-based command. */
-      ptr= (uint8_t *)memchr(data, '\n', data_size);
-      if (ptr == NULL)
-      {
-        *ret_ptr= GEARMAN_IO_WAIT;
-        return 0;
-      }
-
-      packet->magic= GEARMAN_MAGIC_TEXT;
-      packet->command= GEARMAN_COMMAND_TEXT;
-
-      used_size= (size_t)(ptr - ((uint8_t *)data)) + 1;
-      *ptr= 0;
-      if (used_size > 1 && *(ptr - 1) == '\r')
-        *(ptr - 1)= 0;
-
-      size_t arg_size;
-      for (arg_size= used_size, ptr= (uint8_t *)data; ptr != NULL; data= ptr)
-      {
-        ptr= (uint8_t *)memchr(data, ' ', arg_size);
-        if (ptr != NULL)
-        {
-          *ptr= 0;
-          ptr++;
-          while (*ptr == ' ')
-            ptr++;
-
-          arg_size-= (size_t)(ptr - ((uint8_t *)data));
-        }
-
-        *ret_ptr= packet_create_arg(packet, data, ptr == NULL ? arg_size :
-                                    (size_t)(ptr - ((uint8_t *)data)));
-        if (*ret_ptr != GEARMAN_SUCCESS)
-        {
-          return used_size;
-        }
-      }
-
-      return used_size;
-    }
-    else if (data_size < GEARMAN_PACKET_HEADER_SIZE)
-    {
-      *ret_ptr= GEARMAN_IO_WAIT;
-      return 0;
-    }
-
-    packet->args= packet->args_buffer;
-    packet->args_size= GEARMAN_PACKET_HEADER_SIZE;
-    memcpy(packet->args, data, GEARMAN_PACKET_HEADER_SIZE);
-
-    *ret_ptr= gearmand_packet_unpack_header(packet);
-    if (gearmand_failed(*ret_ptr))
-    {
-      return 0;
-    }
-
-    used_size= GEARMAN_PACKET_HEADER_SIZE;
-  }
-  else
-  {
-    used_size= 0;
-  }
-
-  while (packet->argc != gearman_command_info(packet->command)->argc)
-  {
-    if (packet->argc != (gearman_command_info(packet->command)->argc - 1) ||
-        gearman_command_info(packet->command)->data)
-    {
-      ptr= (uint8_t *)memchr(((uint8_t *)data) + used_size, 0, data_size - used_size);
-      if (not ptr)
-      {
-        gearmand_log_debug(GEARMAN_DEFAULT_LOG_PARAM, "Possible protocol error for %s, received only %u args", gearman_command_info(packet->command)->name, packet->argc);
-        *ret_ptr= GEARMAN_IO_WAIT;
-        return used_size;
-      }
-
-      size_t arg_size= (size_t)(ptr - (((uint8_t *)data) + used_size)) + 1;
-      *ret_ptr= packet_create_arg(packet, ((uint8_t *)data) + used_size, arg_size);
-
-      if (gearmand_failed(*ret_ptr))
-        return used_size;
-
-      packet->data_size-= arg_size;
-      used_size+= arg_size;
-    }
-    else
-    {
-      if ((data_size - used_size) < packet->data_size)
-      {
-        *ret_ptr= GEARMAN_IO_WAIT;
-        return used_size;
-      }
-
-      *ret_ptr= packet_create_arg(packet, ((uint8_t *)data) + used_size, packet->data_size);
-      if (gearmand_failed(*ret_ptr))
-      {
-        return used_size;
-      }
-
-      used_size+= packet->data_size;
-      packet->data_size= 0;
-    }
-  }
-
-  *ret_ptr= GEARMAN_SUCCESS;
-  return used_size;
-}
