@@ -35,12 +35,12 @@
  *
  */
 
-#include <config.h>
+#include "gear_config.h"
 #include <iostream>
 
 #include <libgearman-server/plugins.h>
 
-#include <libgearman-server/queue.h>
+#include <libgearman-server/queue.hpp>
 #include <libgearman-server/plugins/queue.h>
 
 namespace gearmand {
@@ -48,6 +48,8 @@ namespace plugins {
 
 void initialize(boost::program_options::options_description &all)
 {
+  queue::initialize_default();
+
   if (HAVE_LIBDRIZZLE)
   {
     queue::initialize_drizzle();
@@ -69,16 +71,20 @@ void initialize(boost::program_options::options_description &all)
   }
 
 
+  if (HAVE_HIREDIS)
+  {
 #ifdef HAVE_LIBHIREDIS
-  queue::initialize_redis();
+    queue::initialize_redis();
 #endif
+  }
 
-  if (HAVE_LIBTOKYOCABINET)
+
+  if (HAVE_TOKYOCABINET)
   {
     queue::initialize_tokyocabinet();
   }
 
-  if (HAVE_MYSQL_BUILD)
+  if (HAVE_LIBMYSQL_BUILD)
   {
     queue::initialize_mysql();
   }
