@@ -42,7 +42,7 @@
  * @brief libpq Queue Storage Definitions
  */
 
-#include <config.h>
+#include <gear_config.h>
 #include <libgearman-server/common.h>
 #include <libgearman-server/byte.h>
 
@@ -50,10 +50,11 @@
 #include <libgearman-server/plugins/queue/base.h>
 
 #if defined(HAVE_LIBPQ) and HAVE_LIBPQ
+#pragma GCC diagnostic ignored "-Wundef"
 #include <libpq-fe.h>
-#include <pg_config_manual.h>
 #endif
 
+#include <cerrno>
 
 /**
  * @addtogroup plugins::queue::Postgresatic Static libpq Queue Storage Definitions
@@ -69,7 +70,7 @@
 
 namespace gearmand { namespace plugins { namespace  queue { class Postgres; }}}
 
-static gearmand_error_t _initialize(gearman_server_st *server, gearmand::plugins::queue::Postgres *queue);
+static gearmand_error_t _initialize(gearman_server_st& server, gearmand::plugins::queue::Postgres *queue);
 
 namespace gearmand {
 namespace plugins {
@@ -131,7 +132,7 @@ gearmand_error_t Postgres::initialize()
   _create_query+= "CREATE TABLE " +table +" (unique_key VARCHAR" +"(" + TOSTRING(GEARMAN_UNIQUE_SIZE) +"), ";
   _create_query+= "function_name VARCHAR(255), priority INTEGER, data BYTEA, when_to_run INTEGER, UNIQUE (unique_key, function_name))";
 
-  gearmand_error_t ret= _initialize(&Gearmand()->server, this);
+  gearmand_error_t ret= _initialize(Gearmand()->server, this);
 
   _insert_query+= "INSERT INTO " +table +" (priority, unique_key, function_name, data, when_to_run) VALUES($1,$2,$3,$4::BYTEA,$5)";
 
@@ -160,7 +161,7 @@ static gearmand_error_t _libpq_add(gearman_server_st *server, void *context,
                                    const char *function_name,
                                    size_t function_name_size,
                                    const void *data, size_t data_size,
-                                   gearmand_job_priority_t priority,
+                                   gearman_job_priority_t priority,
                                    int64_t when);
 
 static gearmand_error_t _libpq_flush(gearman_server_st *server, void *context);
@@ -183,11 +184,9 @@ static gearmand_error_t _libpq_replay(gearman_server_st *server, void *context,
 
 #pragma GCC diagnostic ignored "-Wold-style-cast"
 
-gearmand_error_t _initialize(gearman_server_st *server,
+gearmand_error_t _initialize(gearman_server_st& server,
                              gearmand::plugins::queue::Postgres *queue)
 {
-  PGresult *result;
-
   gearmand_info("Initializing libpq module");
 
   gearman_server_set_queue(server, queue, _libpq_add, _libpq_flush, _libpq_done, _libpq_replay);
@@ -201,11 +200,11 @@ gearmand_error_t _initialize(gearman_server_st *server,
     return GEARMAN_QUEUE_ERROR;
   }
 
-  (void)PQsetNoticeProcessor(queue->con, _libpq_notice_processor, server);
+  (void)PQsetNoticeProcessor(queue->con, _libpq_notice_processor, &server);
 
   std::string query("SELECT tablename FROM pg_tables WHERE tablename='" +queue->table + "'");
 
-  result= PQexec(queue->con, query.c_str());
+  PGresult* result= PQexec(queue->con, query.c_str());
   if (result == NULL || PQresultStatus(result) != PGRES_TUPLES_OK)
   {
     std::string error_string= "PQexec:";
@@ -251,15 +250,14 @@ static void _libpq_notice_processor(void *arg, const char *message)
   gearmand_log_info(GEARMAN_DEFAULT_LOG_PARAM, "PostgreSQL %s", message);
 }
 
-static gearmand_error_t _libpq_add(gearman_server_st *server, void *context,
+static gearmand_error_t _libpq_add(gearman_server_st*, void *context,
                                    const char *unique, size_t unique_size,
                                    const char *function_name,
                                    size_t function_name_size,
                                    const void *data, size_t data_size,
-                                   gearmand_job_priority_t priority,
+                                   gearman_job_priority_t priority,
                                    int64_t when)
 {
-  (void)server;
   (void)when;
   gearmand::plugins::queue::Postgres *queue= (gearmand::plugins::queue::Postgres *)context;
 
@@ -306,13 +304,12 @@ static gearmand_error_t _libpq_flush(gearman_server_st *, void *)
   return GEARMAN_SUCCESS;
 }
 
-static gearmand_error_t _libpq_done(gearman_server_st *server, void *context,
+static gearmand_error_t _libpq_done(gearman_server_st*, void *context,
                                     const char *unique,
                                     size_t unique_size,
                                     const char *function_name,
                                     size_t function_name_size)
 {
-  (void)server;
   (void)function_name_size;
   gearmand::plugins::queue::Postgres *queue= (gearmand::plugins::queue::Postgres *)context;
   PGresult *result;
@@ -377,10 +374,10 @@ static gearmand_error_t _libpq_replay(gearman_server_st *server, void *context,
     {
       data_length= size_t(PQgetlength(result, row, 3));
       data= (char *)malloc(data_length);
-      if (not data)
+      if (data == NULL)
       {
         PQclear(result);
-        return gearmand_perror("malloc");
+        return gearmand_perror(errno, "malloc");
       }
 
       memcpy(data, PQgetvalue(result, row, 3), data_length);
@@ -392,7 +389,7 @@ static gearmand_error_t _libpq_replay(gearman_server_st *server, void *context,
                    PQgetvalue(result, row, 1),
                    (size_t)PQgetlength(result, row, 1),
                    data, data_length,
-                   (gearmand_job_priority_t)atoi(PQgetvalue(result, row, 2)),
+                   (gearman_job_priority_t)atoi(PQgetvalue(result, row, 2)),
                    atoll(PQgetvalue(result, row, 4)));
     if (gearmand_failed(ret))
     {
