@@ -37,6 +37,8 @@
 
 #pragma once
 
+#include "libgearman-server/plugins/base.h"
+
 struct gearmand_io_st
 {
   struct {
@@ -76,6 +78,8 @@ struct gearmand_io_st
   gearmand_connection_list_st *universal;
   gearmand_io_st *next;
   gearmand_io_st *prev;
+  gearmand_io_st *ready_next;
+  gearmand_io_st *ready_prev;
   gearmand_con_st *context;
   char *send_buffer_ptr;
   gearmand_packet_st *recv_packet;
@@ -86,6 +90,8 @@ struct gearmand_io_st
   char recv_buffer[GEARMAN_RECV_BUFFER_SIZE];
 };
 
+namespace gearmand { namespace protocol {class Context; } }
+
 /*
   Free list for these are stored in gearman_server_thread_st[], otherwise they are owned by gearmand_con_st[]
   */
@@ -125,11 +131,33 @@ struct gearman_server_con_st
   const char *_host; // client host
   const char *_port; // client port
   char id[GEARMAN_SERVER_CON_ID_SIZE];
-  struct {
-    void *context;
-    gearmand_connection_protocol_context_free_fn *context_free_fn;
-    gearmand_packet_pack_fn *packet_pack_fn;
-    gearmand_packet_unpack_fn *packet_unpack_fn;
-  } protocol;
+  gearmand::protocol::Context* protocol;
   struct event *timeout_event;
+
+  gearman_server_con_st()
+  {
+  }
+
+  ~gearman_server_con_st()
+  {
+  }
+
+  void set_protocol(gearmand::protocol::Context* arg)
+  {
+    protocol= arg;
+  }
+
+  void protocol_release()
+  {
+    if (protocol)
+    {
+      protocol->notify(this);
+      if (protocol->is_owner())
+      {
+        delete protocol;
+        protocol= NULL;
+      }
+      protocol= NULL;
+    }
+  }
 };
