@@ -35,12 +35,13 @@
  *
  */
 
-#include <config.h>
+#include "gear_config.h"
 #include <libgearman/common.h>
 #include <libgearman/add.hpp>
 #include <libgearman/universal.hpp>
 
-#include <cassert>
+#include "libgearman/assert.hpp"
+
 #include <cerrno>
 
 static inline gearman_command_t pick_command_by_priority(const gearman_job_priority_t &arg)
@@ -56,9 +57,13 @@ static inline gearman_command_t pick_command_by_priority(const gearman_job_prior
 static inline gearman_command_t pick_command_by_priority_background(const gearman_job_priority_t &arg)
 {
   if (arg == GEARMAN_JOB_PRIORITY_NORMAL)
+  {
     return GEARMAN_COMMAND_SUBMIT_JOB_BG;
+  }
   else if (arg == GEARMAN_JOB_PRIORITY_HIGH)
+  {
     return GEARMAN_COMMAND_SUBMIT_JOB_HIGH_BG;
+  }
 
   return GEARMAN_COMMAND_SUBMIT_JOB_LOW_BG;
 }
@@ -68,7 +73,7 @@ static inline gearman_command_t pick_command_by_priority_background(const gearma
 gearman_task_st *gearman_execute(gearman_client_st *client,
                                  const char *function_name, size_t function_length,
                                  const char *unique_str, size_t unique_length,
-                                 gearman_task_attr_t *workload,
+                                 gearman_task_attr_t *task_attr,
                                  gearman_argument_t *arguments,
                                  void *context)
 {
@@ -77,6 +82,12 @@ gearman_task_st *gearman_execute(gearman_client_st *client,
     return NULL;
   }
 
+  gearman_argument_t null_arg= gearman_argument_make(0, 0, 0, 0);
+  if (arguments == NULL)
+  {
+    arguments= &null_arg;
+  }
+
   if (function_name == NULL or function_length == 0)
   {
     gearman_error(client->universal, GEARMAN_INVALID_ARGUMENT, "function_name was NULL");
@@ -86,14 +97,14 @@ gearman_task_st *gearman_execute(gearman_client_st *client,
 
   gearman_task_st *task= NULL;
   gearman_unique_t unique= gearman_unique_make(unique_str, unique_length);
-  if (workload)
+  if (task_attr)
   {
-    switch (workload->kind)
+    switch (task_attr->kind)
     {
     case GEARMAN_TASK_ATTR_BACKGROUND:
       task= add_task(*client,
                      context,
-                     pick_command_by_priority_background(workload->priority),
+                     pick_command_by_priority_background(task_attr->priority),
                      function,
                      unique,
                      arguments->value,
@@ -108,14 +119,14 @@ gearman_task_st *gearman_execute(gearman_client_st *client,
                      function,
                      unique,
                      arguments->value,
-                     gearman_task_attr_has_epoch(workload),
+                     gearman_task_attr_has_epoch(task_attr),
                      gearman_actions_execute_defaults());
       break;
 
     case GEARMAN_TASK_ATTR_FOREGROUND:
       task= add_task(*client,
                      context,
-                     pick_command_by_priority(workload->priority),
+                     pick_command_by_priority(task_attr->priority),
                      function,
                      unique,
                      arguments->value,
@@ -153,23 +164,23 @@ gearman_task_st *gearman_execute_by_partition(gearman_client_st *client,
                                               const char *partition_function, const size_t partition_function_length,
                                               const char *function_name, const size_t function_name_length,
                                               const char *unique_str, const size_t unique_length,
-                                              gearman_task_attr_t *workload,
+                                              gearman_task_attr_t *task_attr,
                                               gearman_argument_t *arguments,
                                               void *context)
 {
-  if (not client)
+  if (client == NULL)
   {
     errno= EINVAL;
     return NULL;
   }
 
-  if (not partition_function or partition_function_length == 0)
+  if ((partition_function == NULL) or (partition_function_length == 0))
   {
-    gearman_error(client->universal, GEARMAN_INVALID_ARGUMENT, "function_name was NULL");
+    gearman_error(client->universal, GEARMAN_INVALID_ARGUMENT, "partition_function was NULL");
     return NULL;
   }
 
-  if (not function_name or function_name_length == 0)
+  if ((function_name == NULL) or (function_name_length == 0))
   {
     gearman_error(client->universal, GEARMAN_INVALID_ARGUMENT, "function_name was NULL");
     return NULL;
@@ -182,14 +193,14 @@ gearman_task_st *gearman_execute_by_partition(gearman_client_st *client,
   gearman_string_t function= { function_name, function_name_length };
   gearman_unique_t unique= gearman_unique_make(unique_str, unique_length);
 
-  if (workload)
+  if (task_attr)
   {
-    switch (workload->kind)
+    switch (task_attr->kind)
     {
     case GEARMAN_TASK_ATTR_BACKGROUND:
       task= add_reducer_task(client,
                              GEARMAN_COMMAND_SUBMIT_REDUCE_JOB_BACKGROUND,
-                             workload->priority,
+                             task_attr->priority,
                              partition,
                              function,
                              unique,
@@ -205,13 +216,13 @@ gearman_task_st *gearman_execute_by_partition(gearman_client_st *client,
 #if 0
       task= add_task(client,
                      GEARMAN_COMMAND_SUBMIT_REDUCE_JOB_BACKGROUND,
-                     workload->priority,
+                     task_attr->priority,
                      partition,
                      function,
                      unique,
                      arguments->value,
                      gearman_actions_execute_defaults(),
-                     gearman_work_epoch(workload),
+                     gearman_work_epoch(task_attr),
                      context);
 #endif
       break;
@@ -219,7 +230,7 @@ gearman_task_st *gearman_execute_by_partition(gearman_client_st *client,
     case GEARMAN_TASK_ATTR_FOREGROUND:
       task= add_reducer_task(client,
                              GEARMAN_COMMAND_SUBMIT_REDUCE_JOB,
-                             workload->priority,
+                             task_attr->priority,
                              partition,
                              function,
                              unique,
@@ -244,7 +255,7 @@ gearman_task_st *gearman_execute_by_partition(gearman_client_st *client,
                            NULL);
   }
 
-  if (not task)
+  if (task == NULL)
   {
     return NULL;
   }
