@@ -36,23 +36,34 @@
  *
  */
 
+#include "gear_config.h"
 #include <libgearman/common.h>
 
-#include <cassert>
+#include "libgearman/assert.hpp"
+
 #include <cstdio>
 #include <cstdlib>
+#include <cstring>
 
-gearman_return_t _client_run_task(gearman_client_st *client, gearman_task_st *task)
+gearman_return_t _client_run_task(gearman_task_st *task)
 {
+  // This should not be possible
+  assert_msg(task->client, "Programmer error, somehow an invalid task was specified");
+  if (task->client == NULL)
+  {
+    return gearman_universal_set_error(task->client->universal, GEARMAN_INVALID_ARGUMENT, GEARMAN_AT,
+                                       "Programmer error, somehow an invalid task was specified");
+  }
+
   switch(task->state)
   {
   case GEARMAN_TASK_STATE_NEW:
-    if (not task->client->universal.con_list)
+    
+    if (task->client->universal.con_list == NULL)
     {
-      client->new_tasks--;
-      client->running_tasks--;
-      gearman_universal_set_error(client->universal, GEARMAN_NO_SERVERS, GEARMAN_AT, "no servers added");
-      return GEARMAN_NO_SERVERS;
+      task->client->new_tasks--;
+      task->client->running_tasks--;
+      return gearman_universal_set_error(task->client->universal, GEARMAN_NO_SERVERS, GEARMAN_AT, "no servers added");
     }
 
     for (task->con= task->client->universal.con_list; task->con;
@@ -64,14 +75,13 @@ gearman_return_t _client_run_task(gearman_client_st *client, gearman_task_st *ta
       }
     }
 
-    if (not task->con)
+    if (task->con == NULL)
     {
-      client->options.no_new= true;
-      gearman_gerror(client->universal, GEARMAN_IO_WAIT);
-      return GEARMAN_IO_WAIT;
+      task->client->options.no_new= true;
+      return gearman_gerror(task->client->universal, GEARMAN_IO_WAIT);
     }
 
-    client->new_tasks--;
+    task->client->new_tasks--;
 
     if (task->send.command != GEARMAN_COMMAND_GET_STATUS)
     {
@@ -83,7 +93,7 @@ gearman_return_t _client_run_task(gearman_client_st *client, gearman_task_st *ta
     while (1)
     {
       assert(task->con);
-      gearman_return_t ret= task->con->send_packet(task->send, client->new_tasks == 0 ? true : false);
+      gearman_return_t ret= task->con->send_packet(task->send, task->client->new_tasks == 0 ? true : false);
 
       if (gearman_success(ret))
       {
@@ -123,12 +133,12 @@ gearman_return_t _client_run_task(gearman_client_st *client, gearman_task_st *ta
           if (ret == GEARMAN_COULD_NOT_CONNECT) // If no connection is found, we will let the user try again
           {
             task->state= GEARMAN_TASK_STATE_NEW;
-            client->new_tasks++;
+            task->client->new_tasks++;
           }
           else
           {
             task->state= GEARMAN_TASK_STATE_FAIL;
-            client->running_tasks--;
+            task->client->running_tasks--;
           }
           return ret;
         }
@@ -145,7 +155,7 @@ gearman_return_t _client_run_task(gearman_client_st *client, gearman_task_st *ta
     {
       if (not task->func.workload_fn)
       {
-        gearman_error(client->universal, GEARMAN_NEED_WORKLOAD_FN,
+        gearman_error(task->client->universal, GEARMAN_NEED_WORKLOAD_FN,
                       "workload size > 0, but no data pointer or workload_fn was given");
         return GEARMAN_NEED_WORKLOAD_FN;
       }
@@ -159,7 +169,7 @@ gearman_return_t _client_run_task(gearman_client_st *client, gearman_task_st *ta
       }
     }
 
-    client->options.no_new= false;
+    task->client->options.no_new= false;
     task->state= GEARMAN_TASK_STATE_WORK;
     task->con->set_events(POLLIN);
     return GEARMAN_SUCCESS;
@@ -221,22 +231,56 @@ gearman_return_t _client_run_task(gearman_client_st *client, gearman_task_st *ta
         }
       }
     }
-    else if (task->recv->command == GEARMAN_COMMAND_WORK_STATUS ||
+    else if (task->recv->command == GEARMAN_COMMAND_WORK_STATUS or
+             task->recv->command == GEARMAN_COMMAND_STATUS_RES_UNIQUE or
              task->recv->command == GEARMAN_COMMAND_STATUS_RES)
     {
       uint8_t x;
 
       if (task->recv->command == GEARMAN_COMMAND_STATUS_RES)
       {
+        task->result_rc= GEARMAN_SUCCESS;
         if (atoi(static_cast<char *>(task->recv->arg[1])) == 0)
+        {
           task->options.is_known= false;
+        }
         else
+        {
           task->options.is_known= true;
+        }
 
         if (atoi(static_cast<char *>(task->recv->arg[2])) == 0)
+        {
           task->options.is_running= false;
+        }
         else
+        {
           task->options.is_running= true;
+        }
+
+        x= 3;
+      }
+      else if (task->recv->command == GEARMAN_COMMAND_STATUS_RES_UNIQUE)
+      {
+        task->result_rc= GEARMAN_SUCCESS;
+        strncpy(task->unique, task->recv->arg[0], GEARMAN_MAX_UNIQUE_SIZE);
+        if (atoi(static_cast<char *>(task->recv->arg[1])) == 0)
+        {
+          task->options.is_known= false;
+        }
+        else
+        {
+          task->options.is_known= true;
+        }
+
+        if (atoi(static_cast<char *>(task->recv->arg[2])) == 0)
+        {
+          task->options.is_running= false;
+        }
+        else
+        {
+          task->options.is_running= true;
+        }
 
         x= 3;
       }
@@ -246,11 +290,25 @@ gearman_return_t _client_run_task(gearman_client_st *client, gearman_task_st *ta
       }
 
       task->numerator= uint32_t(atoi(static_cast<char *>(task->recv->arg[x])));
-      char status_buffer[11]; /* Max string size to hold a uint32_t. */
-      snprintf(status_buffer, 11, "%.*s",
-               int(task->recv->arg_size[x + 1]),
-               static_cast<char *>(task->recv->arg[x + 1]));
-      task->denominator= uint32_t(atoi(status_buffer));
+
+      // denomitor
+      {
+        char status_buffer[11]; /* Max string size to hold a uint32_t. */
+        snprintf(status_buffer, 11, "%.*s",
+                 int(task->recv->arg_size[x + 1]),
+                 static_cast<char *>(task->recv->arg[x + 1]));
+        task->denominator= uint32_t(atoi(status_buffer));
+      }
+
+      // client_count
+      if (task->recv->command == GEARMAN_COMMAND_STATUS_RES_UNIQUE)
+      {
+        char status_buffer[11]; /* Max string size to hold a uint32_t. */
+        snprintf(status_buffer, 11, "%.*s",
+                 int(task->recv->arg_size[x +2]),
+                 static_cast<char *>(task->recv->arg[x +2]));
+        task->client_count= uint32_t(atoi(status_buffer));
+      }
 
   case GEARMAN_TASK_STATE_STATUS:
       if (task->func.status_fn)
@@ -267,6 +325,11 @@ gearman_return_t _client_run_task(gearman_client_st *client, gearman_task_st *ta
       {
         break;
       }
+
+      if (task->send.command == GEARMAN_COMMAND_GET_STATUS_UNIQUE)
+      {
+        break;
+      }
     }
     else if (task->recv->command == GEARMAN_COMMAND_WORK_COMPLETE)
     {
@@ -331,10 +394,10 @@ gearman_return_t _client_run_task(gearman_client_st *client, gearman_task_st *ta
     break;
   }
 
-  client->running_tasks--;
+  task->client->running_tasks--;
   task->state= GEARMAN_TASK_STATE_FINISHED;
 
-  if (client->options.free_tasks and task->type == GEARMAN_TASK_KIND_ADD_TASK)
+  if (task->client->options.free_tasks and task->type == GEARMAN_TASK_KIND_ADD_TASK)
   {
     gearman_task_free(task);
   }
