@@ -35,5 +35,5 @@
  */
 
 
-#include <config.h>
+#include "gear_config.h"
 #include <libgearman/common.h>
