@@ -2,7 +2,7 @@
  * 
  *  Gearmand client and server library.
  *
- *  Copyright (C) 2011 Data Differential, http://datadifferential.com/
+ *  Copyright (C) 2011-2013 Data Differential, http://datadifferential.com/
  *  Copyright (C) 2008 Brian Aker, Eric Day
  *  All rights reserved.
  *
@@ -41,11 +41,12 @@
  * @brief Task Definitions
  */
 
-#include <config.h>
+#include "gear_config.h"
 #include <libgearman/common.h>
 
+#include "libgearman/assert.hpp"
+
 #include <cerrno>
-#include <cassert>
 #include <cstring>
 #include <memory>
 
@@ -53,6 +54,9 @@
  * Public Definitions
  */
 
+#define TASK_MAGIC 134
+#define TASK_ANTI_MAGIC 157
+
 gearman_task_st *gearman_task_internal_create(gearman_client_st *client, gearman_task_st *task)
 {
   assert(client);
@@ -71,6 +75,7 @@ gearman_task_st *gearman_task_internal_create(gearman_client_st *client, gearman
 
     task->options.allocated= true;
   }
+  task->options.is_initialized= true;
 
   task->options.send_in_use= false;
   task->options.is_known= false;
@@ -81,9 +86,11 @@ gearman_task_st *gearman_task_internal_create(gearman_client_st *client, gearman
   task->type= GEARMAN_TASK_KIND_ADD_TASK;
 
   task->state= GEARMAN_TASK_STATE_NEW;
+  task->magic_= TASK_MAGIC;
   task->created_id= 0;
   task->numerator= 0;
   task->denominator= 0;
+  task->client_count= 0;
   task->client= client;
 
   task->func= client->actions;
@@ -106,6 +113,8 @@ gearman_task_st *gearman_task_internal_create(gearman_client_st *client, gearman
   task->recv= NULL;
   task->result_ptr= NULL;
   task->job_handle[0]= 0;
+  task->unique_length= 0;
+  task->unique[0]= 0;
 
   return task;
 }
@@ -113,61 +122,79 @@ gearman_task_st *gearman_task_internal_create(gearman_client_st *client, gearman
 
 void gearman_task_free(gearman_task_st *task)
 {
-  if (task == NULL)
+  if (task)
   {
-    return;
-  }
+    if (task->options.is_initialized == false)
+    {
+      return;
+    }
 
-  delete task->result_ptr;
-  task->result_ptr= NULL;
+    assert(task->magic_ != TASK_ANTI_MAGIC);
+    assert(task->magic_ == TASK_MAGIC);
+    task->magic_= TASK_ANTI_MAGIC;
 
-  if (task->client == NULL)
-  {
-    return;
-  }
+    gearman_task_free_result(task);
 
-  if (task->options.send_in_use)
-  {
-    gearman_packet_free(&(task->send));
-  }
+    if (task->client)
+    {
+      if (task->options.send_in_use)
+      {
+        gearman_packet_free(&(task->send));
+      }
 
-  if (task->type != GEARMAN_TASK_KIND_DO  and task->context and  task->client->task_context_free_fn)
-  {
-    task->client->task_context_free_fn(task, static_cast<void *>(task->context));
-  }
+      if (task->type != GEARMAN_TASK_KIND_DO  and task->context and  task->client->task_context_free_fn)
+      {
+        task->client->task_context_free_fn(task, static_cast<void *>(task->context));
+      }
 
-  if (task->client->task_list == task)
-  {
-    task->client->task_list= task->next;
-  }
+      if (task->client->task_list == task)
+      {
+        task->client->task_list= task->next;
+      }
 
-  if (task->prev)
-  {
-    task->prev->next= task->next;
-  }
+      if (task->prev)
+      {
+        task->prev->next= task->next;
+      }
 
-  if (task->next)
-  {
-    task->next->prev= task->prev;
-  }
+      if (task->next)
+      {
+        task->next->prev= task->prev;
+      }
 
-  task->client->task_count--;
+      task->client->task_count--;
 
-  if (task->options.allocated)
-  {
-    delete task;
+      // If the task we are removing is a current task, remove it from the client
+      // structures.
+      if (task->client->task == task)
+      {
+        task->client->task= NULL;
+      }
+      task->client= NULL;
+    }
+    task->job_handle[0]= 0;
+
+    task->options.is_initialized= false;
+    if (task->options.allocated)
+    {
+      delete task;
+    }
   }
 }
 
 void gearman_task_free_result(gearman_task_st *task)
 {
   assert(task);
-  delete task->result_ptr;
-  task->result_ptr= NULL;
+  if (task)
+  {
+    delete task->result_ptr;
+    task->result_ptr= NULL;
+  }
 }
 
 bool gearman_task_is_active(const gearman_task_st *self)
 {
+  assert(self);
   switch (self->state)
   {
   case GEARMAN_TASK_STATE_NEW:
@@ -190,8 +217,13 @@ bool gearman_task_is_active(const gearman_task_st *self)
   return false;
 }
 
-const char *gearman_task_strstate(gearman_task_st *self)
+const char *gearman_task_strstate(const gearman_task_st *self)
 {
+  if (self == NULL)
+  {
+    return NULL;
+  }
+
   switch (self->state)
   {
   case GEARMAN_TASK_STATE_NEW: return "GEARMAN_TASK_STATE_NEW";
@@ -213,29 +245,38 @@ const char *gearman_task_strstate(gearman_task_st *self)
 
 void gearman_task_clear_fn(gearman_task_st *task)
 {
+  if (task == NULL)
+  {
+    return;
+  }
+
   task->func= gearman_actions_default();
 }
 
 void *gearman_task_context(const gearman_task_st *task)
 {
   if (task == NULL)
+  {
     return NULL;
+  }
 
   return const_cast<void *>(task->context);
 }
 
 void gearman_task_set_context(gearman_task_st *task, void *context)
 {
-  if (task == NULL)
-    return;
-
-  task->context= context;
+  if (task)
+  {
+    task->context= context;
+  }
 }
 
 const char *gearman_task_function_name(const gearman_task_st *task)
 {
   if (task == NULL)
-    return 0;
+  {
+    return NULL;
+  }
 
   return task->send.arg[0];
 }
@@ -243,16 +284,18 @@ const char *gearman_task_function_name(const gearman_task_st *task)
 const char *gearman_task_unique(const gearman_task_st *task)
 {
   if (task == NULL)
-    return 0;
+  {
+    return NULL;
+  }
 
-  return task->send.arg[1];
+  return task->unique;
 }
 
 const char *gearman_task_job_handle(const gearman_task_st *task)
 {
   if (task == NULL)
   {
-    return 0;
+    return NULL;
   }
 
   return task->job_handle;
@@ -261,7 +304,9 @@ const char *gearman_task_job_handle(const gearman_task_st *task)
 bool gearman_task_is_known(const gearman_task_st *task)
 {
   if (task == NULL)
+  {
     return false;
+  }
 
   return task->options.is_known;
 }
@@ -279,7 +324,9 @@ bool gearman_task_is_running(const gearman_task_st *task)
 uint32_t gearman_task_numerator(const gearman_task_st *task)
 {
   if (task == NULL)
+  {
     return 0;
+  }
 
   return task->numerator;
 }
@@ -287,7 +334,9 @@ uint32_t gearman_task_numerator(const gearman_task_st *task)
 uint32_t gearman_task_denominator(const gearman_task_st *task)
 {
   if (task == NULL)
+  {
     return 0;
+  }
 
   return task->denominator;
 }
@@ -295,10 +344,10 @@ uint32_t gearman_task_denominator(const gearman_task_st *task)
 void gearman_task_give_workload(gearman_task_st *task, const void *workload,
                                 size_t workload_size)
 {
-  if (task == NULL)
-    return;
-
-  gearman_packet_give_data(task->send, workload, workload_size);
+  if (task)
+  {
+    gearman_packet_give_data(task->send, workload, workload_size);
+  }
 }
 
 size_t gearman_task_send_workload(gearman_task_st *task, const void *workload,
@@ -306,46 +355,63 @@ size_t gearman_task_send_workload(gearman_task_st *task, const void *workload,
                                   gearman_return_t *ret_ptr)
 {
   if (task == NULL)
+  {
+    if (ret_ptr)
+    {
+      *ret_ptr= GEARMAN_INVALID_ARGUMENT;
+    }
     return 0;
+  }
 
   return task->con->send_and_flush(workload, workload_size, ret_ptr);
 }
 
 gearman_result_st *gearman_task_result(gearman_task_st *task)
 {
-  if (task == NULL)
-    return NULL;
+  if (task)
+  {
+    return task->result_ptr;
+  }
 
-  return task->result_ptr;
+  return NULL;
 }
 
 gearman_result_st *gearman_task_mutable_result(gearman_task_st *task)
 {
   assert(task); // Programmer error
-  if (not task->result_ptr)
-    task->result_ptr= new gearman_result_st();
+  if (task)
+  {
+    if (task->result_ptr == NULL)
+    {
+      task->result_ptr= new (std::nothrow) gearman_result_st();
+      assert(task->result_ptr);
+    }
 
-  return task->result_ptr;
+    return task->result_ptr;
+  }
+  
+  return NULL;
 }
 
 const void *gearman_task_data(const gearman_task_st *task)
 {
-  if (task == NULL)
-    return NULL;
-
-  if (task->recv and task->recv->data)
+  if (task and task->recv and task->recv->data)
+  {
     return task->recv->data;
+  }
 
-  return 0;
+  return NULL;
 }
 
 size_t gearman_task_data_size(const gearman_task_st *task)
 {
-  if (task == NULL)
-    return 0;
-
-  if (task->recv and task->recv->data_size)
-    return task->recv->data_size;
+  if (task)
+  {
+    if (task->recv and task->recv->data_size)
+    {
+      return task->recv->data_size;
+    }
+  }
 
   return 0;
 }
@@ -353,7 +419,9 @@ size_t gearman_task_data_size(const gearman_task_st *task)
 void *gearman_task_take_data(gearman_task_st *task, size_t *data_size)
 {
   if (task == NULL)
+  {
     return 0;
+  }
 
   return gearman_packet_take_data(*task->recv, data_size);
 }
@@ -362,18 +430,19 @@ size_t gearman_task_recv_data(gearman_task_st *task, void *data,
                                   size_t data_size,
                                   gearman_return_t *ret_ptr)
 {
+  gearman_return_t unused;
+  if (ret_ptr == NULL)
+  {
+    ret_ptr= &unused;
+  }
+
   if (task == NULL)
   {
+    *ret_ptr= GEARMAN_INVALID_ARGUMENT;
     return 0;
   }
 
-  if (not ret_ptr)
-  {
-    gearman_return_t unused;
-    return task->con->receiving(data, data_size, unused);
-  }
-
-  return task->con->receiving(data, data_size, *ret_ptr);
+  return task->con->receive_data(data, data_size, *ret_ptr);
 }
 
 const char *gearman_task_error(const gearman_task_st *task)
@@ -384,14 +453,15 @@ const char *gearman_task_error(const gearman_task_st *task)
   }
 
   if (task->result_rc == GEARMAN_UNKNOWN_STATE or task->result_rc == GEARMAN_SUCCESS)
+  {
     return NULL;
+  }
 
   return gearman_strerror(task->result_rc);
 }
 
 gearman_return_t gearman_task_return(const gearman_task_st *task)
 {
-  assert(task); // Only used internally.
   if (task == NULL)
   {
     return GEARMAN_INVALID_ARGUMENT;
