@@ -35,30 +35,31 @@
  *
  */
 
-#include <config.h>
+#include "gear_config.h"
 #include <libgearman/common.h>
 
-#include <cassert>
+#include "libgearman/assert.hpp"
+
 #include <cstring>
 #include <memory>
 
 gearman_task_attr_t gearman_task_attr_init(gearman_job_priority_t priority)
 {
-  gearman_task_attr_t local= { GEARMAN_TASK_ATTR_FOREGROUND, priority, {{0}}};
+  gearman_task_attr_t local= { GEARMAN_TASK_ATTR_FOREGROUND, priority, {{0}} };
 
   return local;
 }
 
 gearman_task_attr_t gearman_task_attr_init_background(gearman_job_priority_t priority)
 {
-  gearman_task_attr_t local= { GEARMAN_TASK_ATTR_BACKGROUND, priority, {{0}}};
+  gearman_task_attr_t local= { GEARMAN_TASK_ATTR_BACKGROUND, priority, {{0}} };
 
   return local;
 }
 
 gearman_task_attr_t gearman_task_attr_init_epoch(time_t epoch, gearman_job_priority_t priority)
 {
-  gearman_task_attr_t local= { GEARMAN_TASK_ATTR_BACKGROUND, priority, {{0}}};
+  gearman_task_attr_t local= { GEARMAN_TASK_ATTR_EPOCH, priority, {{0}} };
   local.options.epoch.value= epoch;
 
   return local;
@@ -66,27 +67,29 @@ gearman_task_attr_t gearman_task_attr_init_epoch(time_t epoch, gearman_job_prior
 
 time_t gearman_task_attr_has_epoch(const gearman_task_attr_t *self)
 {
-  if (not self)
-    return 0;
-
-  if (self->kind == GEARMAN_TASK_ATTR_BACKGROUND)
+  if (self and self->kind == GEARMAN_TASK_ATTR_EPOCH)
+  {
     return self->options.epoch.value;
+  }
 
   return 0;
 }
 
 gearman_job_priority_t gearman_task_attr_priority(const gearman_task_attr_t *self)
 {
-  if (not self)
+  if (self == NULL)
+  {
     return GEARMAN_JOB_PRIORITY_NORMAL;
+  }
 
   return self->priority;
 }
 
 bool gearman_task_attr_is_background(const gearman_task_attr_t *self)
 {
-  if (not self)
-    return false;
-
-  return (self->kind == GEARMAN_TASK_ATTR_BACKGROUND or self->kind == GEARMAN_TASK_ATTR_EPOCH);
+  if (self)
+  {
+    return (self->kind == GEARMAN_TASK_ATTR_BACKGROUND or self->kind == GEARMAN_TASK_ATTR_EPOCH);
+  }
+  return false;
 }
