@@ -39,52 +39,36 @@
 #pragma once
 
 // Get next connection that is ready for I/O.
-GEARMAN_LOCAL
 gearman_connection_st *gearman_ready(gearman_universal_st&);
 
-GEARMAN_LOCAL
 void gearman_universal_initialize(gearman_universal_st &self, const gearman_universal_options_t *options= NULL);
 
-GEARMAN_LOCAL
 void gearman_universal_clone(gearman_universal_st &destination, const gearman_universal_st &source, bool has_wakeup_fd= false);
 
-GEARMAN_LOCAL
 void gearman_universal_free(gearman_universal_st &gearman);
 
-GEARMAN_LOCAL
 void gearman_free_all_packets(gearman_universal_st &gearman);
 
-GEARMAN_LOCAL
-bool gearman_request_option(gearman_universal_st &universal, gearman_string_t &option);
-
-GEARMAN_LOCAL
 gearman_return_t gearman_universal_set_option(gearman_universal_st &self, gearman_universal_options_t option, bool value);
 
-GEARMAN_LOCAL
 void gearman_set_log_fn(gearman_universal_st &self, gearman_log_fn *function, void *context, gearman_verbose_t verbose);
 
-GEARMAN_LOCAL
 void gearman_universal_set_timeout(gearman_universal_st &self, int timeout);
 
-GEARMAN_LOCAL
 int gearman_universal_timeout(gearman_universal_st &self);
 
-GEARMAN_LOCAL
 void gearman_universal_set_namespace(gearman_universal_st &self, const char *namespace_key, size_t namespace_key_size);
 
-GEARMAN_LOCAL
 void gearman_reset(gearman_universal_st& universal);
 
 // Flush the send buffer for all connections.
-GEARMAN_LOCAL
-gearman_return_t gearman_flush_all(gearman_universal_st&);
+void gearman_flush_all(gearman_universal_st&);
 
 /**
  * Set custom memory allocation function for workloads. Normally gearman uses
  * the standard system malloc to allocate memory used with workloads. The
  * provided function will be used instead.
  */
-GEARMAN_LOCAL
 void gearman_set_workload_malloc_fn(gearman_universal_st&,
                                     gearman_malloc_fn *function,
                                     void *context);
@@ -94,30 +78,24 @@ void gearman_set_workload_malloc_fn(gearman_universal_st&,
  * standard system free to free memory used with workloads. The provided
  * function will be used instead.
  */
-GEARMAN_LOCAL
 void gearman_set_workload_free_fn(gearman_universal_st&,
                                   gearman_free_fn *function,
                                   void *context);
 
 // Free all connections for a gearman structure.
-GEARMAN_LOCAL
 void gearman_free_all_cons(gearman_universal_st&);
 
 // Test echo with all connections.
-GEARMAN_LOCAL
 gearman_return_t gearman_echo(gearman_universal_st&, const void *workload, size_t workload_size);
 
 /**
  * Wait for I/O on connections.
  *
  */
-GEARMAN_LOCAL
 gearman_return_t gearman_wait(gearman_universal_st&);
 
-GEARMAN_LOCAL
 void gearman_nap(gearman_universal_st &self);
 
-GEARMAN_LOCAL
 void gearman_nap(int arg);
 
 static inline void gearman_universal_add_options(gearman_universal_st &self, gearman_universal_options_t options)
@@ -160,3 +138,64 @@ gearman_id_t gearman_universal_id(gearman_universal_st &universal);
 gearman_return_t gearman_set_identifier(gearman_universal_st& universal,
                                         const char *id,
                                         size_t id_size);
+
+const char *gearman_univeral_namespace(gearman_universal_st& universal);
+
+#define PUSH(__original, __temp_value) Push _push((__original),(__temp_value));
+
+class Push {
+public:
+  Push(bool& original_, const bool temp_value) :
+    _saved(original_),
+    _origin(original_)
+  {
+    _origin= temp_value;
+  }
+
+  ~Push()
+  {
+    _origin= _saved;
+  }
+
+private:
+  bool _saved;
+  bool& _origin;
+};
+
+/**
+  Push the state of IO
+*/
+
+#define PUSH_BLOCKING(__univeral) Push push_blocking_((__univeral).options.non_blocking, false);
+
+#define PUSH_NON_BLOCKING(__univeral) Push push_non_blocking_((__univeral).options.non_blocking, true);
+
+class Check {
+public:
+  virtual gearman_return_t success(gearman_connection_st*)= 0;
+
+  virtual ~Check() {};
+};
+
+class EchoCheck : public Check {
+public:
+  EchoCheck(gearman_universal_st& universal_,
+            const void *workload_, const size_t workload_size_);
+
+  gearman_return_t success(gearman_connection_st* con);
+
+private:
+  gearman_universal_st& _universal;
+  const void *_workload;
+  const size_t _workload_size;
+};
+
+class OptionCheck : public Check {
+public:
+  OptionCheck(gearman_universal_st& universal_);
+
+  gearman_return_t success(gearman_connection_st* con);
+
+private:
+  gearman_universal_st& _universal;
+};
