@@ -34,20 +34,16 @@
  *
  */
 
-/*
-  Random realloc failing library for testing realloc failures.
-  LD_PRELOAD="/usr/lib/libdl.so ./util/libhostile_realloc.so" ./binary
-*/
+#include "gear_config.h"
 
-#define _GNU_SOURCE
-#include <dlfcn.h>
+#include <libhostile/function.h>
+#include <libhostile/initialize.h>
 
+#include <errno.h>
+#include <pthread.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <time.h>
-#include <errno.h>
-
-#include <libhostile/initialize.h>
 
 static int not_until= 10;
 
@@ -64,14 +60,21 @@ void *realloc(void *old_ptr, size_t size)
   hostile_initialize();
   (void) pthread_once(&function_lookup_once, set_local);
 
-  if (__function.frequency)
+  if (is_called() == false)
   {
-    if (--not_until < 0 && random() % __function.frequency)
+    if (__function.frequency)
     {
-      errno= ENOMEM;
-      return NULL;
+      if (--not_until < 0 && random() % __function.frequency)
+      {
+        errno= ENOMEM;
+        return NULL;
+      }
     }
   }
 
-  return __function.function.realloc(old_ptr, size);
+  set_called();
+  void *ret= __function.function.realloc(old_ptr, size);
+  reset_called();
+
+  return ret;
 }
