@@ -39,7 +39,11 @@
   Test that we are cycling the servers we are creating during testing.
 */
 
-#include <config.h>
+#include "gear_config.h"
+
+#include "libgearman/client.hpp"
+#include "libgearman/worker.hpp"
+using namespace org::gearmand;
 
 #include <libtest/test.hpp>
 using namespace libtest;
@@ -51,9 +55,10 @@ using namespace libtest;
 #pragma GCC diagnostic ignored "-Wstrict-aliasing"
 #endif
 
-#include <tests/workers.h>
 #include <tests/start_worker.h>
 
+#include "tests/workers/v2/echo_or_react.h"
+
 #define WORKER_FUNCTION_NAME "echo_function"
 
 namespace cli {
@@ -73,20 +78,37 @@ struct Context
 
   void shutdown_workers()
   {
-    for (std::vector<worker_handle_st *>::iterator iter= _workers.begin(); iter != _workers.end(); iter++)
+    for (std::vector<worker_handle_st *>::iterator iter= _workers.begin(); iter != _workers.end(); ++iter)
     {
       delete *iter;
     }
     _workers.clear();
   }
 
-  ~Context()
+  void clear()
   {
     shutdown_workers();
+    servers.clear();
+  }
+
+  void port(in_port_t port_)
+  {
+    _port= port_;
+  }
+
+  in_port_t port() const
+  {
+    return _port;
+  }
+
+  ~Context()
+  {
+    clear();
   }
 
 private:
   std::vector<worker_handle_st *>_workers;
+  in_port_t _port;
 
 };
 
@@ -96,164 +118,272 @@ static test_return_t gearman_help_test(void *)
 {
   const char *args[]= { "-H", 0 };
 
-  test_compare(EXIT_SUCCESS, exec_cmdline("bin/gearman", args, true));
+  ASSERT_EQ(EXIT_SUCCESS, exec_cmdline("bin/gearman", args, true));
   return TEST_SUCCESS;
 }
 
-static test_return_t gearman_unknown_test(void *)
+static test_return_t gearman_verbose_TEST(void *)
 {
+  const char *args[]= { "-H", "-v", 0 };
+
+  ASSERT_EQ(EXIT_SUCCESS, exec_cmdline("bin/gearman", args, true));
+  return TEST_SUCCESS;
+}
+
+static test_return_t gearman_unknown_test(void* object)
+{
+  cli::Context *context= (cli::Context*)object;
+
   char buffer[1024];
-  snprintf(buffer, sizeof(buffer), "-p %d", int(default_port()));
+  snprintf(buffer, sizeof(buffer), "-p %d", int(context->port()));
   const char *args[]= { buffer, "--unknown", 0 };
 
   // The argument doesn't exist, so we should see an error
-  test_compare(EXIT_FAILURE, exec_cmdline("bin/gearman", args, true));
+  ASSERT_EQ(EXIT_FAILURE, exec_cmdline("bin/gearman", args, true));
 
   return TEST_SUCCESS;
 }
 
-static test_return_t gearman_client_background_test(void *)
+static test_return_t gearman_client_background_test(void* object)
 {
+  cli::Context *context= (cli::Context*)object;
+
   char buffer[1024];
-  snprintf(buffer, sizeof(buffer), "-p %d", int(default_port()));
+  snprintf(buffer, sizeof(buffer), "-p %d", int(context->port()));
   const char *args[]= { buffer, "-f", WORKER_FUNCTION_NAME, "-b", "payload", 0 };
 
   // The argument doesn't exist, so we should see an error
-  test_compare(EXIT_SUCCESS, exec_cmdline("bin/gearman", args, true));
+  ASSERT_EQ(EXIT_SUCCESS, exec_cmdline("bin/gearman", args, true));
 
   return TEST_SUCCESS;
 }
 
 #define REGRESSION_FUNCTION_833394 "55_char_function_name_________________________________"
 
-static test_return_t regression_833394_test(void *)
-{
-  char buffer[1024];
-  snprintf(buffer, sizeof(buffer), "-p %d", int(default_port()));
-  const char *args[]= { buffer, "-f", REGRESSION_FUNCTION_833394, "-b", "payload", 0 };
-
-  // The argument doesn't exist, so we should see an error
-  test_compare(EXIT_SUCCESS, exec_cmdline("bin/gearman", args, true));
-
-  return TEST_SUCCESS;
-}
-
-static test_return_t gearadmin_help_test(void *)
-{
-  char buffer[1024];
-  snprintf(buffer, sizeof(buffer), "--port=%d", int(default_port()));
-  const char *args[]= { buffer, "--help", 0 };
-
-  test_compare(EXIT_SUCCESS, exec_cmdline("bin/gearadmin", args, true));
-  return TEST_SUCCESS;
-}
-
-static test_return_t gearadmin_shutdown_test(void *object)
+static test_return_t regression_833394_test(void* object)
 {
   cli::Context *context= (cli::Context*)object;
 
   char buffer[1024];
-  snprintf(buffer, sizeof(buffer), "--port=%d", int(default_port()));
+  snprintf(buffer, sizeof(buffer), "-p %d", int(context->port()));
+  const char *args[]= { buffer, "-f", REGRESSION_FUNCTION_833394, "-b", "payload", 0 };
+
+  // The argument doesn't exist, so we should see an error
+  ASSERT_EQ(EXIT_SUCCESS, exec_cmdline("bin/gearman", args, true));
+
+  return TEST_SUCCESS;
+}
+
+static test_return_t gearadmin_help_test(void* object)
+{
+  cli::Context *context= (cli::Context*)object;
+
+  char buffer[1024];
+  snprintf(buffer, sizeof(buffer), "--port=%d", int(context->port()));
+  const char *args[]= { buffer, "--help", 0 };
+
+  ASSERT_EQ(EXIT_SUCCESS, exec_cmdline("bin/gearadmin", args, true));
+  return TEST_SUCCESS;
+}
+
+static test_return_t gearadmin_shutdown_test(void* object)
+{
+  cli::Context *context= (cli::Context*)object;
+
+  char buffer[1024];
+  snprintf(buffer, sizeof(buffer), "--port=%d", int(context->port()));
   const char *args[]= { buffer, "--shutdown", 0 };
 
-  test_compare(EXIT_SUCCESS, exec_cmdline("bin/gearadmin", args, true));
+  ASSERT_EQ(EXIT_SUCCESS, exec_cmdline("bin/gearadmin", args, true));
 
   Server *server= context->servers.pop_server();
   test_true(server);
+  
+  // We will now quiet down the false error about it not being able to restart
+  server->out_of_ban_killed(true);
+
   while (server->ping()) 
   {
     // Wait out the death of the server
   }
 
   // Since we killed the server above, we need to reset it
-  server->reset();
   delete server;
 
   return TEST_SUCCESS;
 }
 
-static test_return_t gearadmin_version_test(void *)
-{
-  char buffer[1024];
-  snprintf(buffer, sizeof(buffer), "--port=%d", int(default_port()));
-  const char *args[]= { buffer, "--server-version", 0 };
-
-  test_compare(EXIT_SUCCESS, exec_cmdline("bin/gearadmin", args, true));
-  return TEST_SUCCESS;
-}
-
-static test_return_t gearadmin_verbose_test(void *)
-{
-  char buffer[1024];
-  snprintf(buffer, sizeof(buffer), "--port=%d", int(default_port()));
-  const char *args[]= { buffer, "--server-verbose", 0 };
-
-  test_compare(EXIT_SUCCESS, exec_cmdline("bin/gearadmin", args, true));
-  return TEST_SUCCESS;
-}
-
-static test_return_t gearadmin_status_test(void *)
-{
-  char buffer[1024];
-  snprintf(buffer, sizeof(buffer), "--port=%d", int(default_port()));
-  const char *args[]= { buffer, "--status", 0 };
-
-  test_compare(EXIT_SUCCESS, exec_cmdline("bin/gearadmin", args, true));
-  return TEST_SUCCESS;
-}
-
-static test_return_t gearadmin_workers_test(void *)
-{
-  char buffer[1024];
-  snprintf(buffer, sizeof(buffer), "--port=%d", int(default_port()));
-  const char *args[]= { buffer, "--workers", 0 };
-
-  test_compare(EXIT_SUCCESS, exec_cmdline("bin/gearadmin", args, true));
-  return TEST_SUCCESS;
-}
-
-static test_return_t gearadmin_create_drop_test(void *)
-{
-  char buffer[1024];
-  snprintf(buffer, sizeof(buffer), "--port=%d", int(default_port()));
-
-  const char *create_args[]= { buffer, "--create-function=test_function", 0 };
-  test_compare(EXIT_SUCCESS, exec_cmdline("bin/gearadmin", create_args, true));
-
-  const char *drop_args[]= { buffer, "--drop-function=test_function", 0 };
-  test_compare(EXIT_SUCCESS, exec_cmdline("bin/gearadmin", drop_args, true));
-
-
-  return TEST_SUCCESS;
-}
-
-static test_return_t gearadmin_getpid_test(void *)
-{
-  char buffer[1024];
-  snprintf(buffer, sizeof(buffer), "--port=%d", int(default_port()));
-  const char *args[]= { buffer, "--getpid", 0 };
-
-  test_compare(EXIT_SUCCESS, exec_cmdline("bin/gearadmin", args, true));
-  return TEST_SUCCESS;
-}
-
-static test_return_t gearadmin_unknown_test(void *)
-{
-  char buffer[1024];
-  snprintf(buffer, sizeof(buffer), "--port=%d", int(default_port()));
-  const char *args[]= { buffer, "--unknown", 0 };
-
-  // The argument doesn't exist, so we should see an error
-  test_compare(EXIT_FAILURE, exec_cmdline("bin/gearadmin", args, true));
-
-  return TEST_SUCCESS;
-}
-
-static test_return_t shutdown_workers(void *object)
+static test_return_t gearadmin_version_test(void* object)
 {
   cli::Context *context= (cli::Context*)object;
 
-  context->shutdown_workers();
+  char buffer[1024];
+  snprintf(buffer, sizeof(buffer), "--port=%d", int(context->port()));
+  const char *args[]= { buffer, "--server-version", 0 };
+
+  ASSERT_EQ(EXIT_SUCCESS, exec_cmdline("bin/gearadmin", args, true));
+  return TEST_SUCCESS;
+}
+
+static test_return_t gearadmin_verbose_test(void* object)
+{
+  cli::Context *context= (cli::Context*)object;
+
+  char buffer[1024];
+  snprintf(buffer, sizeof(buffer), "--port=%d", int(context->port()));
+  const char *args[]= { buffer, "--server-verbose", 0 };
+
+  ASSERT_EQ(EXIT_SUCCESS, exec_cmdline("bin/gearadmin", args, true));
+  return TEST_SUCCESS;
+}
+
+static test_return_t gearadmin_status_TEST(void* object)
+{
+  cli::Context *context= (cli::Context*)object;
+
+  char buffer[1024];
+  snprintf(buffer, sizeof(buffer), "--port=%d", int(context->port()));
+  const char *args[]= { buffer, "--status", 0 };
+
+  ASSERT_EQ(EXIT_SUCCESS, exec_cmdline("bin/gearadmin", args, true));
+  return TEST_SUCCESS;
+}
+
+static test_return_t gearadmin_status_with_jobs_TEST(void* object)
+{
+  cli::Context *context= (cli::Context*)object;
+
+  {
+    libgearman::Client client(context->port());
+    for (size_t x= 0; x < 10000; ++x)
+    {
+      gearman_job_handle_t job_handle;
+      ASSERT_EQ(GEARMAN_SUCCESS, gearman_client_do_background(&client,
+                                                              __func__,
+                                                              NULL, // unique
+                                                              NULL, 0, // workload
+                                                              job_handle));
+    }
+  }
+
+  char buffer[1024];
+  snprintf(buffer, sizeof(buffer), "--port=%d", int(context->port()));
+  const char *args[]= { buffer, "--status", 0 };
+
+  ASSERT_EQ(EXIT_SUCCESS, exec_cmdline("bin/gearadmin", args, true));
+
+  return TEST_SUCCESS;
+}
+
+static test_return_t gearadmin_workers_test(void* object)
+{
+  cli::Context *context= (cli::Context*)object;
+
+  libgearman::Worker worker(context->port());
+  for (int x= 0; x < 100; ++x)
+  {
+    char function_name[1024];
+    snprintf(function_name, sizeof(function_name), "function_%u", x);
+    ASSERT_EQ(GEARMAN_SUCCESS, gearman_worker_register(&worker, function_name, 0));
+  }
+
+  char buffer[1024];
+  snprintf(buffer, sizeof(buffer), "--port=%d", int(context->port()));
+  const char *args[]= { buffer, "--workers", 0 };
+
+  ASSERT_EQ(EXIT_SUCCESS, exec_cmdline("bin/gearadmin", args, true));
+
+  return TEST_SUCCESS;
+}
+
+static test_return_t gearadmin_create_drop_test(void* object)
+{
+  cli::Context *context= (cli::Context*)object;
+
+  char buffer[1024];
+  snprintf(buffer, sizeof(buffer), "--port=%d", int(context->port()));
+
+  const char *create_args[]= { buffer, "--create-function=test_function", 0 };
+  ASSERT_EQ(EXIT_SUCCESS, exec_cmdline("bin/gearadmin", create_args, true));
+
+  const char *drop_args[]= { buffer, "--drop-function=test_function", 0 };
+  ASSERT_EQ(EXIT_SUCCESS, exec_cmdline("bin/gearadmin", drop_args, true));
+
+
+  return TEST_SUCCESS;
+}
+
+static test_return_t gearadmin_getpid_test(void* object)
+{
+  cli::Context *context= (cli::Context*)object;
+
+  {
+    libgearman::Client client(context->port());
+    for (size_t x= 0; x < 4; x++)
+    {
+      gearman_job_handle_t job_handle;
+      ASSERT_EQ(GEARMAN_SUCCESS,
+                gearman_client_do_background(&client,
+                                             __func__, // function
+                                             NULL, // unique
+                                             NULL, 0, // workload
+                                             job_handle));
+    }
+  }
+
+  char buffer[1024];
+  snprintf(buffer, sizeof(buffer), "--port=%d", int(context->port()));
+  const char *args[]= { buffer, "--getpid", 0 };
+
+  ASSERT_EQ(EXIT_SUCCESS, exec_cmdline("bin/gearadmin", args, true));
+  return TEST_SUCCESS;
+}
+
+static test_return_t gearadmin_unknown_test(void* object)
+{
+  cli::Context *context= (cli::Context*)object;
+
+  char buffer[1024];
+  snprintf(buffer, sizeof(buffer), "--port=%d", int(context->port()));
+  const char *args[]= { buffer, "--unknown", 0 };
+
+  // The argument doesn't exist, so we should see an error
+  ASSERT_EQ(EXIT_FAILURE, exec_cmdline("bin/gearadmin", args, true));
+
+  return TEST_SUCCESS;
+}
+
+static test_return_t init_SETUP(void* object)
+{
+  cli::Context *context= (cli::Context*)object;
+
+  context->port(libtest::get_free_port());
+  test_true(server_startup(context->servers, "gearmand", context->port(), NULL));
+
+  // Echo function
+  gearman_function_t echo_react_fn_v2= gearman_function_create(echo_or_react_worker_v2);
+  context->push(test_worker_start(context->port(), NULL, WORKER_FUNCTION_NAME, echo_react_fn_v2, NULL, gearman_worker_options_t()));
+  context->push(test_worker_start(context->port(), NULL, REGRESSION_FUNCTION_833394, echo_react_fn_v2, NULL, gearman_worker_options_t()));
+
+  return TEST_SUCCESS;
+}
+
+static test_return_t init_TEARDOWN(void* object)
+{
+  cli::Context *context= (cli::Context*)object;
+
+  context->clear();
+
+  return TEST_SUCCESS;
+}
+
+
+static test_return_t server_SETUP(void *object)
+{
+  cli::Context *context= (cli::Context*)object;
+
+  in_port_t new_port= libtest::get_free_port();
+  test_true(server_startup(context->servers, "gearmand", new_port, NULL));
 
   return TEST_SUCCESS;
 }
@@ -262,6 +392,7 @@ static test_return_t shutdown_workers(void *object)
 test_st gearman_tests[] ={
   { "--help", 0, gearman_help_test },
   { "-H", 0, gearman_help_test },
+  { "-v", 0, gearman_verbose_TEST },
   { "--unknown", 0, gearman_unknown_test },
   { "-f echo -b payload", 0, gearman_client_background_test },
   { "lp:833394", 0, regression_833394_test },
@@ -273,7 +404,8 @@ test_st gearadmin_tests[] ={
   {"--help", 0, gearadmin_help_test},
   {"--server-version", 0, gearadmin_version_test},
   {"--server-verbose", 0, gearadmin_verbose_test},
-  {"--status", 0, gearadmin_status_test},
+  {"--status", 0, gearadmin_status_TEST},
+  {"gearman_client_do_background(100) --status", 0, gearadmin_status_with_jobs_TEST},
   {"--getpid", 0, gearadmin_getpid_test},
   {"--workers", 0, gearadmin_workers_test},
   {"--create-function and --drop-function", 0, gearadmin_create_drop_test},
@@ -287,32 +419,20 @@ test_st gearadmin_shutdown_tests[] ={
 };
 
 collection_st collection[] ={
-  {"gearman", 0, 0, gearman_tests},
-  {"gearadmin", 0, 0, gearadmin_tests},
-  {"gearadmin --shutdown", shutdown_workers, 0, gearadmin_shutdown_tests},
+  {"gearman", init_SETUP, init_TEARDOWN, gearman_tests},
+  {"gearadmin", init_SETUP, init_TEARDOWN, gearadmin_tests},
+  {"gearadmin --shutdown", server_SETUP, init_TEARDOWN, gearadmin_shutdown_tests},
   {0, 0, 0, 0}
 };
 
 static void *world_create(server_startup_st& servers, test_return_t& error)
 {
-  if (server_startup(servers, "gearmand", default_port(), 0, NULL) == false)
-  {
-    error= TEST_FAILURE;
-    return NULL;
-  }
-
   cli::Context *context= new cli::Context(servers);
-
   if (context == NULL)
   {
     error= TEST_FAILURE;
     return NULL;
   }
-  
-  // Echo function
-  gearman_function_t echo_react_fn_v2= gearman_function_create(echo_or_react_worker_v2);
-  context->push(test_worker_start(default_port(), NULL, WORKER_FUNCTION_NAME, echo_react_fn_v2, NULL, gearman_worker_options_t()));
-  context->push(test_worker_start(default_port(), NULL, REGRESSION_FUNCTION_833394, echo_react_fn_v2, NULL, gearman_worker_options_t()));
 
   return context;
 }
@@ -326,9 +446,9 @@ static bool world_destroy(void *object)
 }
 
 
-void get_world(Framework *world)
+void get_world(libtest::Framework *world)
 {
-  world->collections= collection;
-  world->_create= world_create;
-  world->_destroy= world_destroy;
+  world->collections(collection);
+  world->create(world_create);
+  world->destroy(world_destroy);
 }
