@@ -39,11 +39,13 @@
   Test that we are cycling the servers we are creating during testing.
 */
 
-#include <config.h>
+#include "gear_config.h"
 
 #include <libtest/test.hpp>
 
-#include "tests/worker.h"
+#include "libgearman/client.hpp"
+#include "libgearman/worker.hpp"
+using namespace org::gearmand;
 
 #include <fstream>
 
@@ -59,7 +61,7 @@ static test_return_t postion_TEST(void *)
 {
   const char *args[]= { "foo", 0 };
 
-  test_compare(EXIT_FAILURE, exec_cmdline(gearmand_binary(), args, true));
+  ASSERT_EQ(EXIT_FAILURE, exec_cmdline(gearmand_binary(), args, true));
   return TEST_SUCCESS;
 }
 
@@ -67,7 +69,7 @@ static test_return_t partial_TEST(void *)
 {
   const char *args[]= { "--log", 0 };
 
-  test_compare(EXIT_FAILURE, exec_cmdline(gearmand_binary(), args, true));
+  ASSERT_EQ(EXIT_FAILURE, exec_cmdline(gearmand_binary(), args, true));
   return TEST_SUCCESS;
 }
 
@@ -75,7 +77,7 @@ static test_return_t check_args_test(void *)
 {
   const char *args[]= { "--check-args", 0 };
 
-  test_compare(EXIT_SUCCESS, exec_cmdline(gearmand_binary(), args, true));
+  ASSERT_EQ(EXIT_SUCCESS, exec_cmdline(gearmand_binary(), args, true));
   return TEST_SUCCESS;
 }
 
@@ -83,7 +85,7 @@ static test_return_t long_backlog_test(void *)
 {
   const char *args[]= { "--check-args", "--backlog=10", 0 };
 
-  test_compare(EXIT_SUCCESS, exec_cmdline(gearmand_binary(), args, true));
+  ASSERT_EQ(EXIT_SUCCESS, exec_cmdline(gearmand_binary(), args, true));
   return TEST_SUCCESS;
 }
 
@@ -91,7 +93,7 @@ static test_return_t short_backlog_test(void *)
 {
   const char *args[]= { "--check-args", "-b", "10", 0 };
 
-  test_compare(EXIT_SUCCESS, exec_cmdline(gearmand_binary(), args, true));
+  ASSERT_EQ(EXIT_SUCCESS, exec_cmdline(gearmand_binary(), args, true));
   return TEST_SUCCESS;
 }
 
@@ -99,7 +101,15 @@ static test_return_t long_daemon_test(void *)
 {
   const char *args[]= { "--check-args", "--daemon", 0 };
 
-  test_compare(EXIT_SUCCESS, exec_cmdline(gearmand_binary(), args, true));
+  ASSERT_EQ(EXIT_SUCCESS, exec_cmdline(gearmand_binary(), args, true));
+  return TEST_SUCCESS;
+}
+
+static test_return_t long_coredump_TEST(void *)
+{
+  const char *args[]= { "--check-args", "--coredump", 0 };
+
+  ASSERT_EQ(EXIT_SUCCESS, exec_cmdline(gearmand_binary(), args, true));
   return TEST_SUCCESS;
 }
 
@@ -107,7 +117,7 @@ static test_return_t short_daemon_test(void *)
 {
   const char *args[]= { "--check-args", "-d", 0 };
 
-  test_compare(EXIT_SUCCESS, exec_cmdline(gearmand_binary(), args, true));
+  ASSERT_EQ(EXIT_SUCCESS, exec_cmdline(gearmand_binary(), args, true));
   return TEST_SUCCESS;
 }
 
@@ -115,7 +125,7 @@ static test_return_t long_file_descriptors_test(void *)
 {
   const char *args[]= { "--check-args", "--file-descriptors=10", 0 };
 
-  test_compare(EXIT_SUCCESS, exec_cmdline(gearmand_binary(), args, true));
+  ASSERT_EQ(EXIT_SUCCESS, exec_cmdline(gearmand_binary(), args, true));
   return TEST_SUCCESS;
 }
 
@@ -123,7 +133,7 @@ static test_return_t short_file_descriptors_test(void *)
 {
   const char *args[]= { "--check-args", "-f", "10", 0 };
 
-  test_compare(EXIT_SUCCESS, exec_cmdline(gearmand_binary(), args, true));
+  ASSERT_EQ(EXIT_SUCCESS, exec_cmdline(gearmand_binary(), args, true));
   return TEST_SUCCESS;
 }
 
@@ -131,7 +141,7 @@ static test_return_t long_help_test(void *)
 {
   const char *args[]= { "--check-args", "--help", 0 };
 
-  test_compare(EXIT_SUCCESS, exec_cmdline(gearmand_binary(), args, true));
+  ASSERT_EQ(EXIT_SUCCESS, exec_cmdline(gearmand_binary(), args, true));
   return TEST_SUCCESS;
 }
 
@@ -139,7 +149,7 @@ static test_return_t short_help_test(void *)
 {
   const char *args[]= { "--check-args", "-h", 0 };
 
-  test_compare(EXIT_SUCCESS, exec_cmdline(gearmand_binary(), args, true));
+  ASSERT_EQ(EXIT_SUCCESS, exec_cmdline(gearmand_binary(), args, true));
   return TEST_SUCCESS;
 }
 
@@ -147,7 +157,7 @@ static test_return_t long_log_file_test(void *)
 {
   const char *args[]= { "--check-args", "--log-file=\"tmp/foo\"", 0 };
 
-  test_compare(EXIT_SUCCESS, exec_cmdline(gearmand_binary(), args, true));
+  ASSERT_EQ(EXIT_SUCCESS, exec_cmdline(gearmand_binary(), args, true));
   return TEST_SUCCESS;
 }
 
@@ -155,7 +165,7 @@ static test_return_t long_log_file_stderr_TEST(void *)
 {
   const char *args[]= { "--check-args", "--log-file=stderr", 0 };
 
-  test_compare(EXIT_SUCCESS, exec_cmdline(gearmand_binary(), args, true));
+  ASSERT_EQ(EXIT_SUCCESS, exec_cmdline(gearmand_binary(), args, true));
   return TEST_SUCCESS;
 }
 
@@ -163,7 +173,7 @@ static test_return_t short_log_file_test(void *)
 {
   const char *args[]= { "--check-args", "-l", "\"tmp/foo\"", 0 };
 
-  test_compare(EXIT_SUCCESS, exec_cmdline(gearmand_binary(), args, true));
+  ASSERT_EQ(EXIT_SUCCESS, exec_cmdline(gearmand_binary(), args, true));
   return TEST_SUCCESS;
 }
 
@@ -171,7 +181,7 @@ static test_return_t long_listen_test(void *)
 {
   const char *args[]= { "--check-args", "--listen=10", 0 };
 
-  test_compare(EXIT_SUCCESS, exec_cmdline(gearmand_binary(), args, true));
+  ASSERT_EQ(EXIT_SUCCESS, exec_cmdline(gearmand_binary(), args, true));
   return TEST_SUCCESS;
 }
 
@@ -179,7 +189,7 @@ static test_return_t short_listen_test(void *)
 {
   const char *args[]= { "--check-args", "-L", "10", 0 };
 
-  test_compare(EXIT_SUCCESS, exec_cmdline(gearmand_binary(), args, true));
+  ASSERT_EQ(EXIT_SUCCESS, exec_cmdline(gearmand_binary(), args, true));
   return TEST_SUCCESS;
 }
 
@@ -187,7 +197,7 @@ static test_return_t long_port_test(void *)
 {
   const char *args[]= { "--check-args", "--port=10", 0 };
 
-  test_compare(EXIT_SUCCESS, exec_cmdline(gearmand_binary(), args, true));
+  ASSERT_EQ(EXIT_SUCCESS, exec_cmdline(gearmand_binary(), args, true));
   return TEST_SUCCESS;
 }
 
@@ -195,7 +205,7 @@ static test_return_t short_port_test(void *)
 {
   const char *args[]= { "--check-args", "-p", "10", 0 };
 
-  test_compare(EXIT_SUCCESS, exec_cmdline(gearmand_binary(), args, true));
+  ASSERT_EQ(EXIT_SUCCESS, exec_cmdline(gearmand_binary(), args, true));
   return TEST_SUCCESS;
 }
 
@@ -203,7 +213,7 @@ static test_return_t long_pid_file_test(void *)
 {
   const char *args[]= { "--check-args", "--pid-file=\"tmp/gearmand.pid\"", 0 };
 
-  test_compare(EXIT_SUCCESS, exec_cmdline(gearmand_binary(), args, true));
+  ASSERT_EQ(EXIT_SUCCESS, exec_cmdline(gearmand_binary(), args, true));
   return TEST_SUCCESS;
 }
 
@@ -211,7 +221,7 @@ static test_return_t short_pid_file_test(void *)
 {
   const char *args[]= { "--check-args", "-P", "\"tmp/gearmand.pid\"", 0 };
 
-  test_compare(EXIT_SUCCESS, exec_cmdline(gearmand_binary(), args, true));
+  ASSERT_EQ(EXIT_SUCCESS, exec_cmdline(gearmand_binary(), args, true));
   return TEST_SUCCESS;
 }
 
@@ -219,7 +229,7 @@ static test_return_t long_round_robin_test(void *)
 {
   const char *args[]= { "--check-args", "--round-robin", 0 };
 
-  test_compare(EXIT_SUCCESS, exec_cmdline(gearmand_binary(), args, true));
+  ASSERT_EQ(EXIT_SUCCESS, exec_cmdline(gearmand_binary(), args, true));
   return TEST_SUCCESS;
 }
 
@@ -227,7 +237,7 @@ static test_return_t short_round_robin_test(void *)
 {
   const char *args[]= { "--check-args", "-R", 0 };
 
-  test_compare(EXIT_SUCCESS, exec_cmdline(gearmand_binary(), args, true));
+  ASSERT_EQ(EXIT_SUCCESS, exec_cmdline(gearmand_binary(), args, true));
   return TEST_SUCCESS;
 }
 
@@ -235,7 +245,7 @@ static test_return_t long_syslog_test(void *)
 {
   const char *args[]= { "--check-args", "--syslog", 0 };
 
-  test_compare(EXIT_SUCCESS, exec_cmdline(gearmand_binary(), args, true));
+  ASSERT_EQ(EXIT_SUCCESS, exec_cmdline(gearmand_binary(), args, true));
   return TEST_SUCCESS;
 }
 
@@ -243,7 +253,7 @@ static test_return_t long_threads_test(void *)
 {
   const char *args[]= { "--check-args", "--threads=10", 0 };
 
-  test_compare(EXIT_SUCCESS, exec_cmdline(gearmand_binary(), args, true));
+  ASSERT_EQ(EXIT_SUCCESS, exec_cmdline(gearmand_binary(), args, true));
   return TEST_SUCCESS;
 }
 
@@ -251,7 +261,7 @@ static test_return_t short_threads_test(void *)
 {
   const char *args[]= { "--check-args", "-t", "8", 0 };
 
-  test_compare(EXIT_SUCCESS, exec_cmdline(gearmand_binary(), args, true));
+  ASSERT_EQ(EXIT_SUCCESS, exec_cmdline(gearmand_binary(), args, true));
   return TEST_SUCCESS;
 }
 
@@ -259,7 +269,7 @@ static test_return_t long_user_test(void *)
 {
   const char *args[]= { "--check-args", "--user=nobody", 0 };
 
-  test_compare(EXIT_SUCCESS, exec_cmdline(gearmand_binary(), args, true));
+  ASSERT_EQ(EXIT_SUCCESS, exec_cmdline(gearmand_binary(), args, true));
   return TEST_SUCCESS;
 }
 
@@ -267,7 +277,7 @@ static test_return_t short_user_test(void *)
 {
   const char *args[]= { "--check-args", "-u", "nobody", 0 };
 
-  test_compare(EXIT_SUCCESS, exec_cmdline(gearmand_binary(), args, true));
+  ASSERT_EQ(EXIT_SUCCESS, exec_cmdline(gearmand_binary(), args, true));
   return TEST_SUCCESS;
 }
 
@@ -275,7 +285,7 @@ static test_return_t long_version_test(void *)
 {
   const char *args[]= { "--check-args", "--version", 0 };
 
-  test_compare(EXIT_SUCCESS, exec_cmdline(gearmand_binary(), args, true));
+  ASSERT_EQ(EXIT_SUCCESS, exec_cmdline(gearmand_binary(), args, true));
   return TEST_SUCCESS;
 }
 
@@ -283,7 +293,7 @@ static test_return_t short_version_test(void *)
 {
   const char *args[]= { "--check-args", "-V", 0 };
 
-  test_compare(EXIT_SUCCESS, exec_cmdline(gearmand_binary(), args, true));
+  ASSERT_EQ(EXIT_SUCCESS, exec_cmdline(gearmand_binary(), args, true));
   return TEST_SUCCESS;
 }
 
@@ -291,7 +301,7 @@ static test_return_t verbose_bad_option_TEST(void *)
 {
   const char *args[]= { "--check-args", "--verbose=BAD", 0 };
 
-  test_compare(EXIT_FAILURE, exec_cmdline(gearmand_binary(), args, true));
+  ASSERT_EQ(EXIT_FAILURE, exec_cmdline(gearmand_binary(), args, true));
   return TEST_SUCCESS;
 }
 
@@ -299,7 +309,7 @@ static test_return_t verbose_DEBUG_TEST(void *)
 {
   const char *args[]= { "--check-args", "--verbose=DEBUG", 0 };
 
-  test_compare(EXIT_SUCCESS, exec_cmdline(gearmand_binary(), args, true));
+  ASSERT_EQ(EXIT_SUCCESS, exec_cmdline(gearmand_binary(), args, true));
   return TEST_SUCCESS;
 }
 
@@ -307,7 +317,7 @@ static test_return_t verbose_ERROR_TEST(void *)
 {
   const char *args[]= { "--check-args", "--verbose=ERROR", 0 };
 
-  test_compare(EXIT_SUCCESS, exec_cmdline(gearmand_binary(), args, true));
+  ASSERT_EQ(EXIT_SUCCESS, exec_cmdline(gearmand_binary(), args, true));
   return TEST_SUCCESS;
 }
 
@@ -315,7 +325,7 @@ static test_return_t verbose_ALERT_TEST(void *)
 {
   const char *args[]= { "--check-args", "--verbose=ALERT", 0 };
 
-  test_compare(EXIT_SUCCESS, exec_cmdline(gearmand_binary(), args, true));
+  ASSERT_EQ(EXIT_SUCCESS, exec_cmdline(gearmand_binary(), args, true));
   return TEST_SUCCESS;
 }
 
@@ -323,7 +333,7 @@ static test_return_t verbose_INFO_TEST(void *)
 {
   const char *args[]= { "--check-args", "--verbose=INFO", 0 };
 
-  test_compare(EXIT_SUCCESS, exec_cmdline(gearmand_binary(), args, true));
+  ASSERT_EQ(EXIT_SUCCESS, exec_cmdline(gearmand_binary(), args, true));
   return TEST_SUCCESS;
 }
 
@@ -331,7 +341,7 @@ static test_return_t verbose_WARNING_TEST(void *)
 {
   const char *args[]= { "--check-args", "--verbose=WARNING", 0 };
 
-  test_compare(EXIT_SUCCESS, exec_cmdline(gearmand_binary(), args, true));
+  ASSERT_EQ(EXIT_SUCCESS, exec_cmdline(gearmand_binary(), args, true));
   return TEST_SUCCESS;
 }
 
@@ -339,7 +349,7 @@ static test_return_t verbose_NOTICE_TEST(void *)
 {
   const char *args[]= { "--check-args", "--verbose=NOTICE", 0 };
 
-  test_compare(EXIT_SUCCESS, exec_cmdline(gearmand_binary(), args, true));
+  ASSERT_EQ(EXIT_SUCCESS, exec_cmdline(gearmand_binary(), args, true));
   return TEST_SUCCESS;
 }
 
@@ -347,7 +357,7 @@ static test_return_t verbose_FATAL_TEST(void *)
 {
   const char *args[]= { "--check-args", "--verbose=FATAL", 0 };
 
-  test_compare(EXIT_SUCCESS, exec_cmdline(gearmand_binary(), args, true));
+  ASSERT_EQ(EXIT_SUCCESS, exec_cmdline(gearmand_binary(), args, true));
   return TEST_SUCCESS;
 }
 
@@ -355,7 +365,7 @@ static test_return_t verbose_CRITICAL_TEST(void *)
 {
   const char *args[]= { "--check-args", "--verbose=CRITICAL", 0 };
 
-  test_compare(EXIT_SUCCESS, exec_cmdline(gearmand_binary(), args, true));
+  ASSERT_EQ(EXIT_SUCCESS, exec_cmdline(gearmand_binary(), args, true));
   return TEST_SUCCESS;
 }
 
@@ -363,7 +373,7 @@ static test_return_t verbose_deprecated_TEST(void *)
 {
   const char *args[]= { "--check-args", "-vvv", 0 };
 
-  test_compare(EXIT_FAILURE, exec_cmdline(gearmand_binary(), args, true));
+  ASSERT_EQ(EXIT_FAILURE, exec_cmdline(gearmand_binary(), args, true));
   return TEST_SUCCESS;
 }
 
@@ -371,7 +381,7 @@ static test_return_t long_worker_wakeup_test(void *)
 {
   const char *args[]= { "--check-args", "--worker-wakeup=4", 0 };
 
-  test_compare(EXIT_SUCCESS, exec_cmdline(gearmand_binary(), args, true));
+  ASSERT_EQ(EXIT_SUCCESS, exec_cmdline(gearmand_binary(), args, true));
   return TEST_SUCCESS;
 }
 
@@ -379,7 +389,7 @@ static test_return_t short_worker_wakeup_test(void *)
 {
   const char *args[]= { "--check-args", "-V", 0 };
 
-  test_compare(EXIT_SUCCESS, exec_cmdline(gearmand_binary(), args, true));
+  ASSERT_EQ(EXIT_SUCCESS, exec_cmdline(gearmand_binary(), args, true));
   return TEST_SUCCESS;
 }
 
@@ -387,7 +397,7 @@ static test_return_t protocol_test(void *)
 {
   const char *args[]= { "--check-args", "--protocol=http", 0 };
 
-  test_compare(EXIT_SUCCESS, exec_cmdline(gearmand_binary(), args, true));
+  ASSERT_EQ(EXIT_SUCCESS, exec_cmdline(gearmand_binary(), args, true));
   return TEST_SUCCESS;
 }
 
@@ -395,7 +405,7 @@ static test_return_t queue_test(void *)
 {
   const char *args[]= { "--check-args", "--queue-type=builtin", 0 };
 
-  test_compare(EXIT_SUCCESS, exec_cmdline(gearmand_binary(), args, true));
+  ASSERT_EQ(EXIT_SUCCESS, exec_cmdline(gearmand_binary(), args, true));
 
   return TEST_SUCCESS;
 }
@@ -404,7 +414,23 @@ static test_return_t long_job_retries_test(void *)
 {
   const char *args[]= { "--check-args", "--job-retries=4", 0 };
 
-  test_compare(EXIT_SUCCESS, exec_cmdline(gearmand_binary(), args, true));
+  ASSERT_EQ(EXIT_SUCCESS, exec_cmdline(gearmand_binary(), args, true));
+  return TEST_SUCCESS;
+}
+
+static test_return_t job_handle_prefix_TEST(void *)
+{
+  const char *args[]= { "--check-args", "--job-handle-prefix=my_own_private_handle", 0 };
+
+  ASSERT_EQ(EXIT_SUCCESS, exec_cmdline(gearmand_binary(), args, true));
+  return TEST_SUCCESS;
+}
+
+static test_return_t hashtable_buckets_TEST(void *)
+{
+  const char *args[]= { "--check-args", "--hashtable-buckets=2000", 0 };
+
+  ASSERT_EQ(EXIT_SUCCESS, exec_cmdline(gearmand_binary(), args, true));
   return TEST_SUCCESS;
 }
 
@@ -412,7 +438,7 @@ static test_return_t short_job_retries_test(void *)
 {
   const char *args[]= { "--check-args", "-j", "6", 0 };
 
-  test_compare(EXIT_SUCCESS, exec_cmdline(gearmand_binary(), args, true));
+  ASSERT_EQ(EXIT_SUCCESS, exec_cmdline(gearmand_binary(), args, true));
   return TEST_SUCCESS;
 }
 
@@ -420,15 +446,17 @@ static test_return_t http_port_test(void *)
 {
   const char *args[]= { "--check-args", "--protocol=http", "--http-port=8090",  0 };
 
-  test_compare(EXIT_SUCCESS, exec_cmdline(gearmand_binary(), args, true));
+  ASSERT_EQ(EXIT_SUCCESS, exec_cmdline(gearmand_binary(), args, true));
   return TEST_SUCCESS;
 }
 
 static test_return_t config_file_TEST(void *)
 {
+  ASSERT_EQ(-1, access("etc/gearmand.conf", R_OK));
+
   const char *args[]= { "--check-args", "--config-file=etc/gearmand.conf", 0 };
 
-  test_compare(EXIT_FAILURE, exec_cmdline(gearmand_binary(), args, true));
+  ASSERT_EQ(EXIT_FAILURE, exec_cmdline(gearmand_binary(), args, true));
   return TEST_SUCCESS;
 }
 
@@ -436,7 +464,7 @@ static test_return_t config_file_DEFAULT_TEST(void *)
 {
   const char *args[]= { "--check-args", "--config-file", 0 };
 
-  test_compare(EXIT_FAILURE, exec_cmdline(gearmand_binary(), args, true));
+  ASSERT_EQ(EXIT_FAILURE, exec_cmdline(gearmand_binary(), args, true));
   return TEST_SUCCESS;
 }
 
@@ -444,7 +472,7 @@ static test_return_t config_file_FAIL_TEST(void *)
 {
   const char *args[]= { "--check-args", "--config-file=etc/grmandfoo.conf", 0 };
 
-  test_compare(EXIT_FAILURE, exec_cmdline(gearmand_binary(), args, true));
+  ASSERT_EQ(EXIT_FAILURE, exec_cmdline(gearmand_binary(), args, true));
   return TEST_SUCCESS;
 }
 
@@ -456,20 +484,33 @@ static test_return_t config_file_SIMPLE_TEST(void *)
 
   std::string config_file= "etc/gearmand.conf";
   {
+    char current_directory_buffer[1024];
+    char *current_directory= getcwd(current_directory_buffer, sizeof(current_directory_buffer));
+
+    std::string config_path;
+
+    config_path+= current_directory;
+    config_path+= "/";
+    config_path+= config_file;
+
+
     std::fstream file_stream;
-    file_stream.open(config_file.c_str(), std::fstream::out | std::fstream::trunc);
+    file_stream.open(config_path.c_str(), std::fstream::out | std::fstream::trunc);
 
-    file_stream 
-      << "--port " << port_str << std::endl;
+    test_true(file_stream.good());
 
-    fatal_assert(file_stream.good());
+    file_stream << "--port " << port_str << std::endl;
+
+    test_true(file_stream.good());
     file_stream.close();
   }
-  test_compare(0, access(config_file.c_str(), R_OK));
+  test_zero(access(config_file.c_str(), R_OK));
 
-  const char *args[]= { "--check-args", "--config-file=etc/gearmand.conf", 0 };
+  char args_buffer[1024];
+  snprintf(args_buffer, sizeof(args_buffer), "--config-file=%s", config_file.c_str()); 
+  const char *args[]= { "--check-args", args_buffer, 0 };
 
-  test_compare(EXIT_SUCCESS, exec_cmdline(gearmand_binary(), args, true));
+  ASSERT_EQ(EXIT_SUCCESS, exec_cmdline(gearmand_binary(), args, true));
   unlink(config_file.c_str());
 
   return TEST_SUCCESS;
@@ -477,22 +518,24 @@ static test_return_t config_file_SIMPLE_TEST(void *)
 
 static test_return_t maxqueue_TEST(void *)
 {
+  return TEST_SKIPPED;
+#if 0
   in_port_t port= libtest::get_free_port();
   Application gearmand(gearmand_binary(), true);
 
-  gearmand.add_option("--check-args");
   char buffer[1024];
   test_true(snprintf(buffer, sizeof(buffer), "%d", int32_t(port)) > 0);
   gearmand.add_long_option("--port=", buffer);
 
-  test_compare(Application::SUCCESS, gearmand.run());
+  ASSERT_EQ(Application::SUCCESS, gearmand.run());
 
-  Worker worker;
-  test_compare(GEARMAN_SUCCESS, gearman_worker_register(&worker, __func__, 0));
-  test_compare(GEARMAN_SUCCESS, gearman_worker_unregister(&worker, __func__));
-  test_compare(Application::SUCCESS, gearmand.wait());
+  Worker worker(port);
+  ASSERT_EQ(GEARMAN_SUCCESS, gearman_worker_register(&worker, __func__, 0));
+  ASSERT_EQ(GEARMAN_SUCCESS, gearman_worker_unregister(&worker, __func__));
+  ASSERT_EQ(Application::SUCCESS, gearmand.join());
 
   return TEST_SUCCESS;
+#endif
 }
 
 test_st bad_option_TESTS[] ={
@@ -505,6 +548,7 @@ test_st gearmand_option_tests[] ={
   {"--check-args", 0, check_args_test},
   {"--backlog=", 0, long_backlog_test},
   {"-b", 0, short_backlog_test},
+  {"--coredump", 0, long_coredump_TEST},
   {"--daemon", 0, long_daemon_test},
   {"-d", 0, short_daemon_test},
   {"--file-descriptors=", 0, long_file_descriptors_test},
@@ -546,6 +590,8 @@ test_st gearmand_option_tests[] ={
   {"--protocol=", 0, protocol_test},
   {"--queue-type=", 0, queue_test},
   {"--job-retries=", 0, long_job_retries_test},
+  {"-hashtable-buckets", 0, hashtable_buckets_TEST},
+  {"--job-handle-prefix=", 0, job_handle_prefix_TEST},
   {"-j", 0, short_job_retries_test},
   {"--config-file=etc/gearmand.conf no file present", 0, config_file_TEST },
   {"--config-file", 0, config_file_DEFAULT_TEST },
@@ -564,15 +610,21 @@ test_st maxqueue_TESTS[] ={
   {0, 0, 0}
 };
 
+static test_return_t option_SETUP(void*)
+{
+  unlink("etc/gearmand.conf");
+  return TEST_SUCCESS;
+}
+
 collection_st collection[] ={
   { "bad options", 0, 0, bad_option_TESTS },
-  { "basic options", 0, 0, gearmand_option_tests },
+  { "basic options", option_SETUP, 0, gearmand_option_tests },
   { "httpd options", 0, 0, gearmand_httpd_option_tests },
   { "maxqueue", 0, 0, maxqueue_TESTS },
   {0, 0, 0, 0}
 };
 
-void get_world(Framework *world)
+void get_world(libtest::Framework *world)
 {
-  world->collections= collection;
+  world->collections(collection);
 }
