@@ -35,7 +35,7 @@
  *
  */
 
-#include <config.h>
+#include "gear_config.h"
 #include <libtest/test.hpp>
 
 using namespace libtest;
@@ -60,8 +60,8 @@ test_return_t gearman_execute_test(void *object)
   gearman_task_st *task;
   gearman_argument_t value= gearman_argument_make(0, 0, test_literal_param("test load"));
 
-  test_true_got(task= gearman_execute(client, test_string_make_from_cstr(worker_function), NULL, 0, NULL, &value, 0), gearman_client_error(client));
-  test_compare(test_literal_param_size("test load"), gearman_result_size(gearman_task_result(task)));
+  test_true(task= gearman_execute(client, test_string_make_from_cstr(worker_function), NULL, 0, NULL, &value, 0));
+  ASSERT_EQ(test_literal_param_size("test load"), gearman_result_size(gearman_task_result(task)));
   test_false(gearman_task_is_known(task));
   test_false(gearman_task_is_running(task));
 
@@ -77,8 +77,6 @@ test_return_t gearman_execute_NULL_workload_TEST(void *object)
   const char *worker_function= (const char *)gearman_client_context(client);
   test_true(worker_function);
 
-  gearman_argument_t value= gearman_argument_make(0, 0, test_literal_param("test load"));
-
   gearman_task_attr_t task_attr= gearman_task_attr_init_background(GEARMAN_JOB_PRIORITY_NORMAL);
 
   gearman_task_st *task;
@@ -100,8 +98,6 @@ test_return_t gearman_execute_NULL_attr_NULL_workload_TEST(void *object)
   const char *worker_function= (const char *)gearman_client_context(client);
   test_true(worker_function);
 
-  gearman_argument_t value= gearman_argument_make(0, 0, test_literal_param("test load"));
-
   gearman_task_st *task;
   test_true(task= gearman_execute(client, 
                                   test_string_make_from_cstr(worker_function),
@@ -123,8 +119,8 @@ test_return_t gearman_execute_fail_test(void *object)
   gearman_task_st *task;
   gearman_argument_t value= gearman_argument_make(0, 0, test_literal_param("fail"));
 
-  test_true_got(task= gearman_execute(client, test_string_make_from_cstr(worker_function), NULL, 0, NULL, &value, 0), gearman_client_error(client));
-  test_compare_got(GEARMAN_WORK_FAIL, gearman_task_return(task), gearman_task_error(task));
+  test_true(task= gearman_execute(client, test_string_make_from_cstr(worker_function), NULL, 0, NULL, &value, 0));
+  ASSERT_EQ(GEARMAN_WORK_FAIL, gearman_task_return(task));
   test_false(gearman_task_is_known(task));
   test_false(gearman_task_is_running(task));
 
@@ -146,7 +142,7 @@ test_return_t gearman_execute_timeout_test(void *object)
   // not exist.
   gearman_task_st *task;
   gearman_argument_t value= gearman_argument_make(0, 0, test_literal_param("test load"));
-  test_true_got(task= gearman_execute(client, test_string_make_from_cstr(worker_function), NULL, 0, NULL, &value, 0), gearman_client_error(client));
+  test_true(task= gearman_execute(client, test_string_make_from_cstr(worker_function), NULL, 0, NULL, &value, 0));
   gearman_task_free(task);
 
   return TEST_SUCCESS;
@@ -156,15 +152,15 @@ test_return_t gearman_execute_epoch_test(void *object)
 {
   gearman_client_st *client= (gearman_client_st *)object;
   const char *worker_function= (const char *)gearman_client_context(client);
-  assert(worker_function);
+  test_true(worker_function);
 
   gearman_task_attr_t task_attr= gearman_task_attr_init_epoch(time(NULL) +5, GEARMAN_JOB_PRIORITY_NORMAL);
 
   gearman_task_st *task;
   gearman_argument_t value= gearman_argument_make(0, 0, test_literal_param("test load"));
-  test_true_got(task= gearman_execute(client, test_string_make_from_cstr(worker_function), 
-                                      NULL, 0, // unique
-                                      &task_attr, &value, 0), gearman_client_error(client));
+  test_true(task= gearman_execute(client, test_string_make_from_cstr(worker_function), 
+                                  NULL, 0, // unique
+                                  &task_attr, &value, 0));
   test_truth(task);
   test_truth(gearman_task_job_handle(task));
   test_true(gearman_task_is_known(task));
@@ -184,17 +180,29 @@ test_return_t gearman_execute_epoch_check_job_handle_test(void *object)
 
   gearman_task_st *task;
   gearman_argument_t value= gearman_argument_make(0, 0, test_literal_param("test load"));
-  test_true_got(task= gearman_execute(client, test_string_make_from_cstr(worker_function), NULL, 0, &task_attr, &value, 0), gearman_client_error(client));
+  test_true(task= gearman_execute(client, test_string_make_from_cstr(worker_function), NULL, 0, &task_attr, &value, 0));
 
   test_truth(task);
   test_truth(gearman_task_job_handle(task));
 
   gearman_return_t rc;
   bool is_known;
+  size_t limit= 10;
   do {
-    rc= gearman_client_job_status(client, gearman_task_job_handle(task), &is_known, NULL, NULL, NULL);
+    if (--limit)
+    {
+      rc= gearman_client_job_status(client, gearman_task_job_handle(task), &is_known, NULL, NULL, NULL);
+    }
+    else
+    {
+      break;
+    }
   }  while (gearman_continue(rc) or is_known);
-  test_compare(GEARMAN_SUCCESS, rc);
+
+  if (limit)
+  {
+    ASSERT_EQ(GEARMAN_SUCCESS, rc);
+  }
 
   gearman_task_free(task);
 
@@ -211,14 +219,13 @@ test_return_t gearman_execute_bg_test(void *object)
 
   gearman_task_st *task;
   gearman_argument_t value= gearman_argument_make(0, 0, test_literal_param("test load"));
-  test_true_got(task= gearman_execute(client, test_string_make_from_cstr(worker_function), test_literal_param("my id"), &task_attr, &value, 0), 
-                gearman_client_error(client));
+  test_true(task= gearman_execute(client, test_string_make_from_cstr(worker_function), test_literal_param("my id"), &task_attr, &value, 0));
 
   // Lets make sure we have a task
   test_truth(task);
   test_truth(gearman_task_job_handle(task));
 
-  test_true_got(gearman_success(gearman_client_run_tasks(client)), gearman_client_error(client));
+  test_true(gearman_success(gearman_client_run_tasks(client)));
 
   gearman_task_free(task);
 
@@ -237,15 +244,14 @@ test_return_t gearman_execute_multile_bg_test(void *object)
 
     gearman_task_st *task;
     gearman_argument_t value= gearman_argument_make(0, 0, test_literal_param("test load"));
-    test_true_got(task= gearman_execute(client, test_string_make_from_cstr(worker_function), NULL, 0, &task_attr, &value, 0), 
-                  gearman_client_error(client));
+    test_true(task= gearman_execute(client, test_string_make_from_cstr(worker_function), NULL, 0, &task_attr, &value, 0));
     
     // Lets make sure we have a task
     test_truth(task);
     test_truth(gearman_task_job_handle(task));
   }
 
-  test_compare(GEARMAN_SUCCESS,
+  ASSERT_EQ(GEARMAN_SUCCESS,
                gearman_client_run_tasks(client));
 
   gearman_client_task_free_all(client);
