@@ -35,7 +35,7 @@
  *
  */
 
-#include <config.h>
+#include "gear_config.h"
 #include <libtest/test.hpp>
 
 using namespace libtest;
@@ -43,20 +43,60 @@ using namespace libtest;
 #include <cassert>
 #include <cstring>
 #include <libgearman/gearman.h>
-#include <tests/gearman_execute_partition.h>
+#include "tests/gearman_execute_partition.h"
+
+#include "tests/libgearman-1.0/client_test.h"
+
+#include "tests/workers/v2/echo_or_react.h"
+#include "tests/workers/v2/split.h"
+#include "tests/workers/aggregator/cat.h"
 
 #ifndef __INTEL_COMPILER
 #pragma GCC diagnostic ignored "-Wold-style-cast"
 #endif
 
+#define WORKER_FUNCTION_NAME "client_test"
+#define WORKER_SPLIT_FUNCTION_NAME "split_worker"
+
+test_return_t partition_SETUP(void *object)
+{
+  client_test_st *test= (client_test_st *)object;
+  test_true(test);
+
+  test->set_worker_name(WORKER_FUNCTION_NAME);
+
+  gearman_function_t echo_react_fn= gearman_function_create_v2(echo_or_react_worker_v2);
+  test->push(test_worker_start(libtest::default_port(), NULL,
+                               test->worker_name(),
+                               echo_react_fn, NULL, gearman_worker_options_t()));
+
+  gearman_function_t split_worker_fn= gearman_function_create_partition(split_worker, cat_aggregator_fn);
+  test->push(test_worker_start(libtest::default_port(), NULL,
+                               WORKER_SPLIT_FUNCTION_NAME,
+                               split_worker_fn,  NULL, GEARMAN_WORKER_GRAB_ALL));
+
+
+  return TEST_SUCCESS;
+}
+
+test_return_t partition_free_SETUP(void *object)
+{
+  client_test_st *test= (client_test_st *)object;
+
+  ASSERT_EQ(TEST_SUCCESS, partition_SETUP(object));
+
+  gearman_client_add_options(test->client(), GEARMAN_CLIENT_FREE_TASKS);
+
+  return TEST_SUCCESS;
+}
+
 test_return_t gearman_execute_partition_check_parameters(void *object)
 {
   gearman_client_st *client= (gearman_client_st *)object;
   test_true(client);
 
-  test_compare_got(GEARMAN_SUCCESS,
-                   gearman_client_echo(client, test_literal_param("this is mine")),
-                   gearman_client_error(client));
+  ASSERT_EQ(GEARMAN_SUCCESS,
+               gearman_client_echo(client, test_literal_param("this is mine")));
 
   // This just hear to make it easier to trace when gearman_execute() is
   // called (look in the log to see the failed option setting.
@@ -66,7 +106,7 @@ test_return_t gearman_execute_partition_check_parameters(void *object)
   // Test client as NULL
   gearman_task_st *task= gearman_execute_by_partition(NULL,
                                                       test_literal_param("split_worker"),
-                                                      test_literal_param("client_test"),
+                                                      test_literal_param(WORKER_FUNCTION_NAME),
                                                       NULL, 0,  // unique
                                                       NULL,
                                                       &workload, 0);
@@ -88,29 +128,30 @@ test_return_t gearman_execute_partition_basic(void *object)
 {
   gearman_client_st *client= (gearman_client_st *)object;
 
-  test_compare_hint(GEARMAN_SUCCESS,
-                    gearman_client_echo(client, test_literal_param("this is mine")),
-                    gearman_client_error(client));
+  ASSERT_EQ(GEARMAN_SUCCESS,
+               gearman_client_echo(client, test_literal_param("this is mine")));
 
   // This just hear to make it easier to trace when
   // gearman_execute_partition() is called (look in the log to see the
   // failed option setting.
   gearman_client_set_server_option(client, test_literal_param("should fail"));
+
+  // This is the real work
   gearman_argument_t workload= gearman_argument_make(0, 0, test_literal_param("this dog does not hunt"));
 
   gearman_task_st *task= gearman_execute_by_partition(client,
                                                       test_literal_param("split_worker"),
-                                                      test_literal_param("client_test"),
+                                                      test_literal_param(WORKER_FUNCTION_NAME),
                                                       NULL, 0,  // unique
                                                       NULL,
                                                       &workload, 0);
-  test_true_hint(task, gearman_client_error(client));
-  test_compare(GEARMAN_SUCCESS, gearman_task_return(task));
+  ASSERT_TRUE(task);
+  ASSERT_EQ(GEARMAN_SUCCESS, gearman_task_return(task));
   gearman_result_st *result= gearman_task_result(task);
-  test_truth(result);
+  ASSERT_TRUE(result);
   const char *value= gearman_result_value(result);
   test_truth(value);
-  test_compare(18UL, gearman_result_size(result));
+  ASSERT_EQ(18UL, gearman_result_size(result));
 
   gearman_task_free(task);
   gearman_client_task_free_all(client);
@@ -123,9 +164,8 @@ test_return_t gearman_execute_partition_workfail(void *object)
   gearman_client_st *client= (gearman_client_st *)object;
   const char *worker_function= (const char *)gearman_client_context(client);
 
-  test_compare_hint(GEARMAN_SUCCESS,
-                    gearman_client_echo(client, test_literal_param("this is mine")),
-                    gearman_client_error(client));
+  ASSERT_EQ(GEARMAN_SUCCESS,
+               gearman_client_echo(client, test_literal_param("this is mine")));
 
   gearman_argument_t workload= gearman_argument_make(0, 0, test_literal_param("this dog does not hunt mapper_fail"));
 
@@ -135,9 +175,9 @@ test_return_t gearman_execute_partition_workfail(void *object)
                                                       NULL, 0,  // unique
                                                       NULL,
                                                       &workload, 0);
-  test_true_hint(task, gearman_client_error(client));
+  test_true(task);
 
-  test_compare_got(GEARMAN_WORK_FAIL, gearman_task_return(task), gearman_task_error(task));
+  ASSERT_EQ(GEARMAN_WORK_FAIL, gearman_task_return(task));
 
   gearman_task_free(task);
   gearman_client_task_free_all(client);
@@ -150,7 +190,7 @@ test_return_t gearman_execute_partition_fail_in_reduction(void *object)
   gearman_client_st *client= (gearman_client_st *)object;
   const char *worker_function= (const char *)gearman_client_context(client);
 
-  test_true_got(gearman_success(gearman_client_echo(client, test_literal_param("this is mine"))), gearman_client_error(client));
+  ASSERT_EQ(gearman_client_echo(client, test_literal_param("this is mine")), GEARMAN_SUCCESS);
 
   gearman_argument_t workload= gearman_argument_make(0, 0, test_literal_param("this dog does not hunt fail"));
 
@@ -160,9 +200,9 @@ test_return_t gearman_execute_partition_fail_in_reduction(void *object)
                                                       NULL, 0,  // unique
                                                       NULL,
                                                       &workload, 0);
-  test_true_hint(task, gearman_client_error(client));
+  test_true(task);
 
-  test_compare_got(GEARMAN_WORK_FAIL, gearman_task_return(task), gearman_task_error(task));
+  ASSERT_EQ(GEARMAN_WORK_FAIL, gearman_task_return(task));
 
   gearman_task_free(task);
   gearman_client_task_free_all(client);
@@ -174,7 +214,7 @@ test_return_t gearman_execute_partition_use_as_function(void *object)
 {
   gearman_client_st *client= (gearman_client_st *)object;
 
-  test_true_got(gearman_success(gearman_client_echo(client, test_literal_param("this is mine"))), gearman_client_error(client));
+  ASSERT_EQ(gearman_client_echo(client, test_literal_param("this is mine")), GEARMAN_SUCCESS);
 
   // This just hear to make it easier to trace when
   // gearman_execute_partition() is called (look in the log to see the
@@ -187,14 +227,14 @@ test_return_t gearman_execute_partition_use_as_function(void *object)
                                          NULL, 0,  // unique
                                          NULL,
                                          &workload, 0);
-  test_true_hint(task, gearman_client_error(client));
+  test_true(task);
 
-  test_compare(GEARMAN_SUCCESS, gearman_task_return(task));
+  ASSERT_EQ(GEARMAN_SUCCESS, gearman_task_return(task));
   gearman_result_st *result= gearman_task_result(task);
   test_truth(result);
   const char *value= gearman_result_value(result);
   test_truth(value);
-  test_compare(18UL, gearman_result_size(result));
+  ASSERT_EQ(18UL, gearman_result_size(result));
 
   gearman_task_free(task);
   gearman_client_task_free_all(client);
@@ -209,18 +249,17 @@ test_return_t gearman_execute_partition_no_aggregate(void *object)
   gearman_argument_t workload= gearman_argument_make(0, 0, test_literal_param("this dog does not hunt"));
 
   gearman_task_st *task= gearman_execute_by_partition(client,
-                                                      test_literal_param("client_test"),
+                                                      test_literal_param(WORKER_FUNCTION_NAME),
                                                       test_literal_param("count"),
                                                       NULL, 0,  // unique
                                                       NULL,
                                                       &workload, 0);
-  test_true_hint(task, gearman_client_error(client));
+  test_true(task);
 
-  test_compare_got(GEARMAN_SUCCESS, 
-                   gearman_task_return(task), 
-                   gearman_client_error(client));
+  ASSERT_EQ(GEARMAN_SUCCESS, 
+               gearman_task_return(task));
 
-  test_compare(GEARMAN_SUCCESS, gearman_task_return(task));
+  ASSERT_EQ(GEARMAN_SUCCESS, gearman_task_return(task));
   test_false(gearman_task_result(task));
 
   gearman_task_free(task);
