@@ -2,7 +2,7 @@
  * 
  *  Gearmand client and server library.
  *
- *  Copyright (C) 2011 Data Differential, http://datadifferential.com/
+ *  Copyright (C) 2011-2013 Data Differential, http://datadifferential.com/
  *  Copyright (C) 2008 Brian Aker, Eric Day
  *  All rights reserved.
  *
@@ -38,23 +38,22 @@
 
 
 
-#include <config.h>
+#include "gear_config.h"
 #include <libtest/test.hpp>
 
 using namespace libtest;
 
-#include <cassert>
 #include <cstdio>
 #include <cstdlib>
 #include <cstring>
 
 #define GEARMAN_CORE
-#include <libgearman/common.h>
-#include <libgearman/packet.hpp>
+#include "libgearman/common.h"
+#include "libgearman/packet.hpp"
 
-#include <libgearman/universal.hpp>
+#include "libgearman/universal.hpp"
 
-#include <tests/regression.h>
+#include "tests/regression.h"
 
 #ifndef __INTEL_COMPILER
 #pragma GCC diagnostic ignored "-Wold-style-cast"
@@ -78,10 +77,8 @@ static test_return_t init_test(void *)
   gearman_universal_st gear;
 
   gearman_universal_initialize(gear);
-  test_false(gear.options.dont_track_packets);
-  test_false(gear.options.non_blocking);
-  test_false(gear.options.stored_non_blocking);
-  test_false(gear._namespace);
+  ASSERT_FALSE(gear.options.non_blocking);
+  ASSERT_FALSE(gear._namespace);
 
   gearman_universal_free(gear);
 
@@ -100,9 +97,7 @@ static test_return_t clone_test(void *)
     gearman_universal_clone(destination, gear);
 
     { // Test all of the flags
-      test_truth(destination.options.dont_track_packets == gear.options.dont_track_packets);
       test_truth(destination.options.non_blocking == gear.options.non_blocking);
-      test_truth(destination.options.stored_non_blocking == gear.options.stored_non_blocking);
     }
     test_truth(destination._namespace == gear._namespace);
     test_truth(destination.verbose == gear.verbose);
@@ -134,13 +129,13 @@ static test_return_t set_timout_test(void *)
 
   gearman_universal_initialize(universal);
 
-  test_compare(-1, gearman_universal_timeout(universal)); // Current default
+  ASSERT_EQ(-1, gearman_universal_timeout(universal)); // Current default
 
   gearman_universal_set_timeout(universal, 20);
-  test_compare(20, gearman_universal_timeout(universal)); // New value of 20
+  ASSERT_EQ(20, gearman_universal_timeout(universal)); // New value of 20
 
   gearman_universal_set_timeout(universal, 10);
-  test_compare(10, gearman_universal_timeout(universal)); // New value of 10
+  ASSERT_EQ(10, gearman_universal_timeout(universal)); // New value of 10
 
   gearman_universal_free(universal);
 
@@ -154,9 +149,9 @@ static test_return_t basic_error_test(void *)
   gearman_universal_initialize(universal);
 
   const char *error= gearman_universal_error(universal);
-  test_false(error);
+  ASSERT_FALSE(error);
 
-  test_compare(0, gearman_universal_errno(universal));
+  ASSERT_EQ(0, gearman_universal_errno(universal));
 
   gearman_universal_free(universal);
 
@@ -170,9 +165,7 @@ static test_return_t state_option_test(void *)
 
   gearman_universal_initialize(universal);
   { // Initial Allocated, no changes
-    test_false(universal.options.dont_track_packets);
-    test_false(universal.options.non_blocking);
-    test_false(universal.options.stored_non_blocking);
+    ASSERT_FALSE(universal.options.non_blocking);
   }
   gearman_universal_free(universal);
 
@@ -186,9 +179,7 @@ static test_return_t state_option_on_create_test(void *)
 
   gearman_universal_initialize(universal, options);
   { // Initial Allocated, no changes
-    test_truth(universal.options.dont_track_packets);
     test_truth(universal.options.non_blocking);
-    test_false(universal.options.stored_non_blocking);
   }
   gearman_universal_free(universal);
 
@@ -201,10 +192,10 @@ static test_return_t gearman_universal_set_namespace_test(void *)
   gearman_universal_st universal;
   gearman_universal_initialize(universal);
 
-  test_false(universal._namespace);
+  ASSERT_FALSE(universal._namespace);
 
   gearman_universal_set_namespace(universal, gearman_literal_param("foo23"));
-  test_truth(universal._namespace);
+  ASSERT_TRUE(universal._namespace);
 
   gearman_universal_free(universal);
 
@@ -216,7 +207,7 @@ static test_return_t clone_gearman_universal_set_namespace_test(void *)
   gearman_universal_st universal;
   gearman_universal_initialize(universal);
 
-  test_false(universal._namespace);
+  ASSERT_FALSE(universal._namespace);
 
   gearman_universal_set_namespace(universal, gearman_literal_param("my_dog"));
   test_truth(universal._namespace);
@@ -238,32 +229,24 @@ static test_return_t state_option_set_test(void *)
 
   gearman_universal_initialize(universal, options);
   { // Initial Allocated, no changes
-    test_truth(universal.options.dont_track_packets);
     test_truth(universal.options.non_blocking);
-    test_false(universal.options.stored_non_blocking);
   }
 
   test_truth(gearman_universal_is_non_blocking(universal));
 
   gearman_universal_initialize(universal);
   { // Initial Allocated, no changes
-    test_false(universal.options.dont_track_packets);
-    test_false(universal.options.non_blocking);
-    test_false(universal.options.stored_non_blocking);
+    ASSERT_FALSE(universal.options.non_blocking);
   }
 
   gearman_universal_add_options(universal, GEARMAN_DONT_TRACK_PACKETS);
   { // Initial Allocated, no changes
-    test_truth(universal.options.dont_track_packets);
-    test_false(universal.options.non_blocking);
-    test_false(universal.options.stored_non_blocking);
+    ASSERT_FALSE(universal.options.non_blocking);
   }
 
   gearman_universal_remove_options(universal, GEARMAN_DONT_TRACK_PACKETS);
   { // Initial Allocated, no changes
-    test_false(universal.options.dont_track_packets);
-    test_false(universal.options.non_blocking);
-    test_false(universal.options.stored_non_blocking);
+    ASSERT_FALSE(universal.options.non_blocking);
   }
 
   gearman_universal_free(universal);
@@ -294,8 +277,8 @@ static test_return_t connection_init_test(void *)
   gearman_connection_st *connection_ptr= gearman_connection_create(universal, NULL);
   test_truth(connection_ptr);
 
-  test_false(connection_ptr->options.ready);
-  test_false(connection_ptr->options.packet_in_use);
+  ASSERT_FALSE(connection_ptr->options.ready);
+  ASSERT_FALSE(connection_ptr->options.packet_in_use);
 
   delete connection_ptr;
 
@@ -311,8 +294,8 @@ static test_return_t connection_alloc_test(void *)
   gearman_connection_st *connection_ptr= gearman_connection_create(universal, NULL);
   test_truth(connection_ptr);
 
-  test_false(connection_ptr->options.ready);
-  test_false(connection_ptr->options.packet_in_use);
+  ASSERT_FALSE(connection_ptr->options.ready);
+  ASSERT_FALSE(connection_ptr->options.packet_in_use);
 
   delete connection_ptr;
 
@@ -335,22 +318,24 @@ static test_return_t packet_init_test(void *)
   gearman_universal_initialize(universal);
 
   packet_ptr= gearman_packet_create(universal, &packet);
-  test_false(packet.options.allocated);
-  test_false(packet_ptr->options.allocated);
+  ASSERT_FALSE(packet.options.allocated);
+  ASSERT_FALSE(packet_ptr->options.allocated);
 
-  test_false(packet.options.complete);
-  test_false(packet.options.free_data);
+  ASSERT_FALSE(packet.options.complete);
+  ASSERT_FALSE(packet.options.free_data);
 
   test_truth(packet_ptr == &packet);
 
   gearman_packet_free(packet_ptr);
-  test_false(packet.options.allocated);
+  ASSERT_FALSE(packet.options.allocated);
 
   return TEST_SUCCESS;
 }
 
 static test_return_t gearman_packet_give_data_test(void *)
 {
+  // @note Since this is a give data, ignore any errors that believe there is
+  // an implicit memory leak.
   size_t data_size= test_literal_param_size("Mine!");
   char *data= (char *)calloc(sizeof(char), data_size +1);
   test_true(data);
@@ -367,7 +352,7 @@ static test_return_t gearman_packet_give_data_test(void *)
   gearman_packet_give_data(packet, data, data_size);
 
   test_truth(packet.data == data);
-  test_truth(packet.data_size == data_size);
+  ASSERT_EQ(packet.data_size, data_size);
   test_truth(packet.options.free_data);
 
   gearman_packet_free(&packet);
@@ -378,6 +363,8 @@ static test_return_t gearman_packet_give_data_test(void *)
 
 static test_return_t gearman_packet_take_data_test(void *)
 {
+  // Since this is a take data, ignore any errors that believe there is an
+  // implicit memory leak.
   size_t data_size= test_literal_param_size("Mine!");
   char *data= (char *)calloc(sizeof(char), data_size +1);
   test_true(data);
@@ -395,18 +382,18 @@ static test_return_t gearman_packet_take_data_test(void *)
   gearman_packet_give_data(packet, data, data_size);
 
   test_truth(packet_ptr->data == data);
-  test_compare(data_size, packet_ptr->data_size);
+  ASSERT_EQ(data_size, packet_ptr->data_size);
   test_truth(packet_ptr->options.free_data);
 
   size_t mine_size;
   char *mine= (char *)gearman_packet_take_data(packet, &mine_size);
 
-  test_false(packet_ptr->data);
+  ASSERT_FALSE(packet_ptr->data);
   test_zero(packet_ptr->data_size);
-  test_false(packet_ptr->options.free_data);
+  ASSERT_FALSE(packet_ptr->options.free_data);
 
   test_strcmp(mine, "Mine!");
-  test_compare(data_size, mine_size);
+  ASSERT_EQ(data_size, mine_size);
 
   gearman_packet_free(packet_ptr);
   gearman_universal_free(universal);
@@ -441,17 +428,17 @@ static void *world_create(server_startup_st& servers, test_return_t& error)
   /**
     We start up everything before we allocate so that we don't have to track memory in the forked process.
   */
-  if (server_startup(servers, "gearmand", libtest::default_port(), 0, NULL) == false)
+  if (server_startup(servers, "gearmand", libtest::default_port(), NULL) == false)
   {
-    error= TEST_FAILURE;
+    error= TEST_SKIPPED;
     return NULL;
   }
 
   return NULL;
 }
 
-void get_world(Framework *world)
+void get_world(libtest::Framework *world)
 {
-  world->collections= collection;
-  world->_create= world_create;
+  world->collections(collection);
+  world->create(world_create);
 }
