@@ -35,13 +35,12 @@
  *
  */
 
-#include <config.h>
+#include "gear_config.h"
 #include <libtest/test.hpp>
 
 using namespace libtest;
 
 #include <tests/limits.h>
-#include "tests/workers.h"
 #include "tests/start_worker.h"
 
 
@@ -60,7 +59,7 @@ test_return_t function_name_limit_test(void *object)
                                              NULL, 
                                              test_literal_param("reset"),
                                              &result_length, &rc);
-  test_compare_got(GEARMAN_INVALID_ARGUMENT, rc, gearman_strerror(rc));
+  ASSERT_EQ(GEARMAN_INVALID_ARGUMENT, rc);
   test_null(job_result);
 
   return TEST_SUCCESS;
@@ -85,7 +84,7 @@ test_return_t unique_name_limit_test(void *object)
                                              &unique_name[0],
                                              test_literal_param("reset"),
                                              &result_length, &rc);
-  test_compare_got(GEARMAN_INVALID_ARGUMENT, rc, gearman_strerror(rc));
+  ASSERT_EQ(GEARMAN_INVALID_ARGUMENT, rc);
   test_null(job_result);
 
   return TEST_SUCCESS;
