@@ -35,35 +35,142 @@
  *
  */
 
-#include <config.h>
+#include "gear_config.h"
 #include <libtest/test.hpp>
 
 using namespace libtest;
 
-#include <cassert>
-#include <libgearman/gearman.h>
-#include <libgearman/actions.hpp>
-#include <tests/task.h>
+#include <libgearman-1.0/gearman.h>
+#include "libgearman/actions.hpp"
+#include "tests/task.h"
 
+#include "tests/workers/v2/call_exception.h"
+#include "tests/start_worker.h"
+
+#include "libgearman/client.hpp"
+#include "libgearman/worker.hpp"
+using namespace org::gearmand;
+
+#include <cassert>
 #include <iostream>
+#include <memory>
 
 #ifndef __INTEL_COMPILER
 #pragma GCC diagnostic ignored "-Wold-style-cast"
 #endif
 
-test_return_t gearman_client_add_task_test(void *object)
+test_return_t gearman_client_add_task_status_by_unique_NOT_FOUND_TEST(void *object)
 {
   gearman_client_st *client= (gearman_client_st *)object;
   const char *worker_function= (const char *)gearman_client_context(client);
 
-  assert(worker_function);
+  const char* unique_key= __func__;
+
+  fatal_assert(worker_function);
 
   gearman_return_t ret;
-  gearman_task_st *task= gearman_client_add_task(client, NULL, NULL,
-                                                 worker_function, NULL, "dog", 3,
+
+  gearman_task_st* unique_task= gearman_client_add_task_status_by_unique(client,
+                                                                         NULL, // context
+                                                                         unique_key,
+                                                                         &ret);
+  ASSERT_EQ(GEARMAN_SUCCESS, ret);
+  test_truth(unique_task);
+  test_true(gearman_task_unique(unique_task));
+  ASSERT_EQ(strlen(unique_key), strlen(gearman_task_unique(unique_task)));
+
+  libtest::dream(1, 0);
+  test_false(gearman_task_is_running(unique_task));
+  test_false(gearman_task_is_known(unique_task));
+
+  gearman_task_free(unique_task);
+
+  return TEST_SUCCESS;
+}
+
+test_return_t gearman_client_add_task_status_by_unique_TEST(void *object)
+{
+  gearman_client_st *client= (gearman_client_st *)object;
+  const char *worker_function= (const char *)gearman_client_context(client);
+
+  const char* unique_key= __func__;
+
+  fatal_assert(worker_function);
+
+  gearman_return_t ret;
+  gearman_task_st *task= gearman_client_add_task(client,
+                                                 NULL, // preallocated task
+                                                 NULL, // context 
+                                                 worker_function, // function
+                                                 unique_key, // unique
+                                                 test_literal_param("sleep"), // workload
                                                  &ret);
-  test_true_got(gearman_success(ret), gearman_strerror(ret));
+  ASSERT_EQ(GEARMAN_SUCCESS, ret);
   test_truth(task);
+  test_true(gearman_task_unique(task));
+  ASSERT_EQ(strlen(unique_key), strlen(gearman_task_unique(task)));
+
+  do
+  {
+    // just for async IO
+    size_t limit= 10;
+    do {
+      if (--limit)
+      {
+        break;
+      }
+      ret= gearman_client_run_tasks(client);
+    } while (gearman_continue(ret));
+
+    if (limit)
+    {
+      ASSERT_EQ(ret, GEARMAN_SUCCESS);
+    }
+
+    // If the task has been built to be freed, we won't have it to test
+    if (gearman_client_has_option(client, GEARMAN_CLIENT_FREE_TASKS))
+    {
+      return TEST_SUCCESS;
+    }
+
+  } while (gearman_task_is_running(task));
+
+  gearman_task_st* unique_task= gearman_client_add_task_status_by_unique(client,
+                                                                         NULL, // context
+                                                                         unique_key,
+                                                                         &ret);
+  ASSERT_EQ(GEARMAN_SUCCESS, ret);
+  test_truth(unique_task);
+  test_true(gearman_task_unique(unique_task));
+  ASSERT_EQ(strlen(unique_key), strlen(gearman_task_unique(unique_task)));
+
+  gearman_task_free(unique_task);
+  gearman_task_free(task);
+
+  return TEST_SUCCESS;
+}
+
+test_return_t gearman_client_add_task_test(void *object)
+{
+  SKIP_IF(HAVE_UUID_UUID_H != 1);
+
+  gearman_client_st *client= (gearman_client_st *)object;
+  const char *worker_function= (const char *)gearman_client_context(client);
+
+  fatal_assert(worker_function);
+
+  gearman_return_t ret;
+  gearman_task_st *task= gearman_client_add_task(client,
+                                                 NULL, // preallocated task
+                                                 NULL, // context 
+                                                 worker_function, // function
+                                                 NULL, // unique
+                                                 test_literal_param("dog"), // workload
+                                                 &ret);
+  ASSERT_EQ(ret, GEARMAN_SUCCESS);
+  test_truth(task);
+  test_true(gearman_task_unique(task));
+  ASSERT_EQ(size_t(GEARMAN_MAX_UUID_SIZE), strlen(gearman_task_unique(task)));
 
   do
   {
@@ -73,7 +180,7 @@ test_return_t gearman_client_add_task_test(void *object)
       ret= gearman_client_run_tasks(client);
     } while (gearman_continue(ret));
 
-    test_compare_got(GEARMAN_SUCCESS, ret, gearman_client_error(client));
+    ASSERT_EQ(ret, GEARMAN_SUCCESS);
 
     // If the task has been built to be freed, we won't have it to test
     if (gearman_client_has_option(client, GEARMAN_CLIENT_FREE_TASKS))
@@ -94,14 +201,14 @@ test_return_t gearman_client_add_task_test_fail(void *object)
   gearman_client_st *client= (gearman_client_st *)object;
   const char *worker_function= (const char *)gearman_client_context(client);
 
-  assert(worker_function);
+  fatal_assert(worker_function);
 
   gearman_return_t ret;
   gearman_task_st *task= gearman_client_add_task(client, NULL, NULL,
                                                  worker_function, NULL,
                                                  test_literal_param("fail"),
                                                  &ret);
-  test_compare(GEARMAN_SUCCESS,ret);
+  ASSERT_EQ(GEARMAN_SUCCESS,ret);
   test_truth(task);
   test_truth(task->client);
 
@@ -109,7 +216,7 @@ test_return_t gearman_client_add_task_test_fail(void *object)
     ret= gearman_client_run_tasks(client);
   } while (gearman_continue(ret));
 
-  test_compare_got(GEARMAN_SUCCESS, ret, gearman_client_error(client));
+  ASSERT_EQ(ret, GEARMAN_SUCCESS);
 
   // If the task has been free() then we can't check anything about it
   if (gearman_client_has_option(client, GEARMAN_CLIENT_FREE_TASKS))
@@ -119,7 +226,7 @@ test_return_t gearman_client_add_task_test_fail(void *object)
 
   test_truth(task->client);
 
-  test_compare_got(GEARMAN_WORK_FAIL, gearman_task_return(task), gearman_task_error(task));
+  ASSERT_EQ(gearman_task_return(task), GEARMAN_WORK_FAIL);
 
   test_truth(task->client);
   gearman_task_free(task);
@@ -132,7 +239,7 @@ test_return_t gearman_client_add_task_test_bad_workload(void *object)
   gearman_client_st *client= (gearman_client_st *)object;
   const char *worker_function= (const char *)gearman_client_context(client);
 
-  assert(worker_function);
+  fatal_assert(worker_function);
 
   gearman_return_t ret;
 
@@ -140,14 +247,14 @@ test_return_t gearman_client_add_task_test_bad_workload(void *object)
   gearman_task_st *task= gearman_client_add_task(client, NULL, NULL,
                                                  worker_function, NULL, "fail", 0,
                                                  &ret);
-  test_true_got(ret == GEARMAN_INVALID_ARGUMENT, gearman_strerror(ret));
+  ASSERT_EQ(ret, GEARMAN_INVALID_ARGUMENT);
   test_false(task);
 
   // We test for NULL with size
   task= gearman_client_add_task(client, NULL, NULL,
                                 worker_function, NULL, NULL, 5,
                                 &ret);
-  test_true_got(ret == GEARMAN_INVALID_ARGUMENT, gearman_strerror(ret));
+  ASSERT_EQ(ret, GEARMAN_INVALID_ARGUMENT);
   test_false(task);
 
   return TEST_SUCCESS;
@@ -170,7 +277,7 @@ test_return_t gearman_client_add_task_exception(void *object)
   gearman_client_st *client= (gearman_client_st *)object;
   const char *worker_function= (const char *)gearman_client_context(client);
 
-  assert(worker_function);
+  fatal_assert(worker_function);
 
   if (gearman_client_has_option(client, GEARMAN_CLIENT_FREE_TASKS))
   {
@@ -188,13 +295,13 @@ test_return_t gearman_client_add_task_exception(void *object)
                                                  worker_function, NULL,
                                                  test_literal_param("exception"),
                                                  &ret);
-  test_compare_got(GEARMAN_SUCCESS, ret, gearman_strerror(ret));
+  ASSERT_EQ(ret, GEARMAN_SUCCESS);
   test_truth(task);
 
   do {
     ret= gearman_client_run_tasks(client);
   } while (gearman_continue(ret));
-  test_compare_got(GEARMAN_SUCCESS, ret, gearman_client_error(client));
+  ASSERT_EQ(ret, GEARMAN_SUCCESS);
 
   test_truth(exception_success);
 
@@ -206,7 +313,73 @@ test_return_t gearman_client_add_task_exception(void *object)
     return TEST_SUCCESS;
   }
 
-  assert(client->task_list);
+  fatal_assert(client->task_list);
+  gearman_task_free(task);
+
+  return TEST_SUCCESS;
+}
+
+static gearman_return_t check_exception_function(gearman_task_st *task)
+{
+  std::string *exception_str= (std::string *)gearman_task_context(task);
+  if (exception_str == NULL)
+  {
+    Error << "Programmer error, null std::string passed";
+    return GEARMAN_WORK_FAIL;
+  }
+
+  assert(task);
+#if 0
+  exception_str->append(task->recv->arg[1], task->recv->arg_size[1]);
+#endif
+
+  return GEARMAN_SUCCESS;
+}
+
+test_return_t gearman_client_add_task_check_exception_TEST(void*)
+{
+  libgearman::Client client(libtest::default_port());
+
+  if (gearman_client_has_option(&client, GEARMAN_CLIENT_FREE_TASKS))
+  {
+    return TEST_SKIPPED;
+  }
+
+  gearman_client_set_exception_fn(&client, check_exception_function);
+
+  gearman_function_t func= gearman_function_create_v2(call_exception_WORKER);
+  std::auto_ptr<worker_handle_st> call_exception_worker(test_worker_start(libtest::default_port(), NULL,
+                                                                          __func__,
+                                                                          func,
+                                                                          NULL, gearman_worker_options_t()));
+
+  std::string exception_string;
+  gearman_return_t ret;
+  gearman_task_st *task= gearman_client_add_task(&client, NULL, &exception_string,
+                                                 __func__, NULL,
+                                                 test_literal_param("exception test"),
+                                                 &ret);
+  ASSERT_EQ(ret, GEARMAN_SUCCESS);
+  test_truth(task);
+
+  do {
+    ret= gearman_client_run_tasks(&client);
+  } while (gearman_continue(ret));
+  ASSERT_EQ(ret, GEARMAN_SUCCESS);
+
+  // This is a defect, from what I understand we should be passing along the
+  // exception.
+#if 0
+  test_true(exception_string.compare("exception test") == 0);
+#endif
+
+  // If the task has been free() then we can't check anything about it
+  if (gearman_client_has_option(&client, GEARMAN_CLIENT_FREE_TASKS))
+  {
+    return TEST_SUCCESS;
+  }
+
+  fatal_assert(&client->task_list);
   gearman_task_free(task);
 
   return TEST_SUCCESS;
@@ -217,13 +390,13 @@ test_return_t gearman_client_add_task_background_test(void *object)
   gearman_client_st *client= (gearman_client_st *)object;
   const char *worker_function= (const char *)gearman_client_context(client);
 
-  assert(worker_function);
+  fatal_assert(worker_function);
 
   gearman_return_t ret;
   gearman_task_st *task= gearman_client_add_task_background(client, NULL, NULL,
                                                             worker_function, NULL, "dog", 3,
                                                             &ret);
-  test_compare_got(GEARMAN_SUCCESS, ret, gearman_strerror(ret));
+  ASSERT_EQ(ret, GEARMAN_SUCCESS);
   test_truth(task);
 
   do
@@ -234,7 +407,7 @@ test_return_t gearman_client_add_task_background_test(void *object)
       ret= gearman_client_run_tasks(client);
     } while (gearman_continue(ret));
 
-    test_compare_got(GEARMAN_SUCCESS, ret, gearman_client_error(client));
+    ASSERT_EQ(ret, GEARMAN_SUCCESS);
 
     // If the task has been built to be freed, we won't have it to test
     if (gearman_client_has_option(client, GEARMAN_CLIENT_FREE_TASKS))
@@ -244,7 +417,7 @@ test_return_t gearman_client_add_task_background_test(void *object)
 
   } while (gearman_task_is_running(task));
 
-  assert(client->task_list);
+  fatal_assert(client->task_list);
   gearman_task_free(task);
 
   return TEST_SUCCESS;
@@ -255,13 +428,13 @@ test_return_t gearman_client_add_task_high_background_test(void *object)
   gearman_client_st *client= (gearman_client_st *)object;
   const char *worker_function= (const char *)gearman_client_context(client);
 
-  assert(worker_function);
+  fatal_assert(worker_function);
 
   gearman_return_t ret;
   gearman_task_st *task= gearman_client_add_task_high_background(client, NULL, NULL,
                                                                  worker_function, NULL, "dog", 3,
                                                                  &ret);
-  test_compare_got(GEARMAN_SUCCESS, ret, gearman_strerror(ret));
+  ASSERT_EQ(ret, GEARMAN_SUCCESS);
   test_truth(task);
 
   do
@@ -271,7 +444,7 @@ test_return_t gearman_client_add_task_high_background_test(void *object)
     do {
       ret= gearman_client_run_tasks(client);
     } while (gearman_continue(ret));
-    test_compare_got(GEARMAN_SUCCESS, ret, gearman_client_error(client));
+    ASSERT_EQ(ret, GEARMAN_SUCCESS);
 
     // If the task has been built to be freed, we won't have it to test
     if (gearman_client_has_option(client, GEARMAN_CLIENT_FREE_TASKS))
@@ -290,13 +463,13 @@ test_return_t gearman_client_add_task_low_background_test(void *object)
   gearman_client_st *client= (gearman_client_st *)object;
   const char *worker_function= (const char *)gearman_client_context(client);
 
-  assert(worker_function);
+  fatal_assert(worker_function);
 
   gearman_return_t ret;
   gearman_task_st *task= gearman_client_add_task_high_background(client, NULL, NULL,
                                                                  worker_function, NULL, "dog", 3,
                                                                  &ret);
-  test_compare_got(GEARMAN_SUCCESS, ret, gearman_strerror(ret));
+  ASSERT_EQ(ret, GEARMAN_SUCCESS);
   test_truth(task);
 
   do
@@ -307,7 +480,7 @@ test_return_t gearman_client_add_task_low_background_test(void *object)
       ret= gearman_client_run_tasks(client);
     } while (gearman_continue(ret));
 
-    test_compare_got(GEARMAN_SUCCESS, ret, gearman_client_error(client));
+    ASSERT_EQ(ret, GEARMAN_SUCCESS);
 
     // If the task has been built to be freed, we won't have it to test
     if (gearman_client_has_option(client, GEARMAN_CLIENT_FREE_TASKS))
@@ -340,7 +513,7 @@ test_return_t gearman_client_add_task_warning(void *object)
   gearman_client_st *client= (gearman_client_st *)object;
   const char *worker_function= (const char *)gearman_client_context(client);
 
-  assert(worker_function);
+  fatal_assert(worker_function);
 
   gearman_return_t ret;
 
@@ -351,16 +524,18 @@ test_return_t gearman_client_add_task_warning(void *object)
                                                  worker_function, NULL,
                                                  test_literal_param("warning"),
                                                  &ret);
-  test_compare_got(GEARMAN_SUCCESS, ret, gearman_strerror(ret));
+  ASSERT_EQ(ret, GEARMAN_SUCCESS);
   test_truth(task);
 
-  test_compare(GEARMAN_SUCCESS, 
+  ASSERT_EQ(GEARMAN_SUCCESS, 
                gearman_client_run_tasks(client));
   test_truth(warning_success);
 
   gearman_client_set_warning_fn(client, NULL);
   if (not gearman_client_has_option(client, GEARMAN_CLIENT_FREE_TASKS))
+  {
     gearman_task_free(task);
+  }
 
   return TEST_SUCCESS;
 }
@@ -375,13 +550,13 @@ test_return_t gearman_client_add_task_no_servers(void *)
                                                  "does not exist", NULL,
                                                  test_literal_param("dog"),
                                                  &ret);
-  test_compare_got(GEARMAN_SUCCESS, ret, gearman_strerror(ret));
+  ASSERT_EQ(ret, GEARMAN_SUCCESS);
   test_truth(task);
 
-  test_compare(GEARMAN_NO_SERVERS, 
+  ASSERT_EQ(GEARMAN_NO_SERVERS, 
                gearman_client_run_tasks(client));
 
-  test_compare(GEARMAN_NO_SERVERS, 
+  ASSERT_EQ(GEARMAN_NO_SERVERS, 
                gearman_client_run_tasks(client));
 
   gearman_client_free(client);
@@ -392,10 +567,11 @@ test_return_t gearman_client_add_task_no_servers(void *)
 test_return_t gearman_client_add_task_pause_test(void *object)
 {
   gearman_client_st *client= (gearman_client_st *)object;
-  assert(client);
+  fatal_assert(client);
+
   const char *worker_function= (const char *)gearman_client_context(client);
+  fatal_assert(worker_function);
 
-  assert(worker_function);
   // Don't do this.
   gearman_actions_t pause_actions= gearman_actions_pause();
   client->actions= pause_actions;
@@ -405,8 +581,10 @@ test_return_t gearman_client_add_task_pause_test(void *object)
                                                  worker_function, NULL, "dog", 3,
                                                  &ret);
   test_true(client->actions.data_fn == pause_actions.data_fn);
-  test_compare_got(GEARMAN_SUCCESS, ret, gearman_strerror(ret));
+  ASSERT_EQ(ret, GEARMAN_SUCCESS);
   test_truth(task);
+  test_true(gearman_task_unique(task));
+  ASSERT_EQ(size_t(GEARMAN_MAX_UUID_SIZE), strlen(gearman_task_unique(task)));
 
   do
   {
@@ -419,7 +597,7 @@ test_return_t gearman_client_add_task_pause_test(void *object)
       test_true(client->actions.data_fn == pause_actions.data_fn);
     } while (gearman_continue(ret));
 
-    test_compare_got(GEARMAN_SUCCESS, ret, gearman_client_error(client));
+    ASSERT_EQ(ret, GEARMAN_SUCCESS);
     test_true(count > 1);
 
     // If the task has been built to be freed, we won't have it to test
@@ -468,10 +646,10 @@ struct _task_free_st {
 
 static void test_task_free_fn(gearman_task_st *task, void *context)
 {
-  assert(task);
+  fatal_assert(task);
 
   _task_free_st *foo= (_task_free_st *)context;
-  assert(foo->test());
+  fatal_assert(foo->test());
 
   foo->add();
 }
@@ -479,10 +657,10 @@ static void test_task_free_fn(gearman_task_st *task, void *context)
 test_return_t gearman_client_set_task_context_free_fn_test(void *object)
 {
   gearman_client_st *client= (gearman_client_st *)object;
-  assert(client);
+  fatal_assert(client);
 
   const char *worker_function= (const char *)gearman_client_context(client);
-  assert(worker_function);
+  fatal_assert(worker_function);
 
   struct _task_free_st task_free_foo;
 
@@ -493,8 +671,10 @@ test_return_t gearman_client_set_task_context_free_fn_test(void *object)
   gearman_task_st *task= gearman_client_add_task(client, NULL, NULL,
                                                  worker_function, NULL, "dog", 3,
                                                  &ret);
-  test_true_got(gearman_success(ret), gearman_strerror(ret));
+  ASSERT_EQ(ret, GEARMAN_SUCCESS);
   test_truth(task);
+  test_true(gearman_task_unique(task));
+  ASSERT_EQ(size_t(GEARMAN_MAX_UUID_SIZE), strlen(gearman_task_unique(task)));
   gearman_task_set_context(task, &task_free_foo);
 
   do
@@ -505,7 +685,7 @@ test_return_t gearman_client_set_task_context_free_fn_test(void *object)
       ret= gearman_client_run_tasks(client);
     } while (gearman_continue(ret));
 
-    test_compare_got(GEARMAN_SUCCESS, ret, gearman_client_error(client));
+    ASSERT_EQ(ret, GEARMAN_SUCCESS);
 
     // If the task has been built to be freed, we won't have it to test
     if (gearman_client_has_option(client, GEARMAN_CLIENT_FREE_TASKS))
