@@ -6,7 +6,7 @@
  * the COPYING file in the parent directory for full text.
  */
 
-#include <config.h>
+#include "gear_config.h"
 #include <libtest/test.hpp>
 
 using namespace libtest;
@@ -35,7 +35,7 @@ static test_return_t gearmand_basic_option_test(void *)
     "--redis-port=6379",
     0 };
 
-  test_compare(EXIT_SUCCESS, exec_cmdline(gearmand_binary(), args, true));
+  ASSERT_EQ(EXIT_SUCCESS, exec_cmdline(gearmand_binary(), args, true));
   return TEST_SUCCESS;
 }
 
@@ -46,7 +46,7 @@ static test_return_t collection_init(void *object)
 
   const char *argv[]= { "--queue-type=redis", 0 };
 
-  test->initialize(1, argv);
+  test->initialize(argv);
 
   return TEST_SUCCESS;
 }
@@ -60,13 +60,15 @@ static test_return_t collection_cleanup(void *object)
 }
 
 
-static void *world_create(server_startup_st& servers, test_return_t& error)
+static void *world_create(server_startup_st& servers, test_return_t&)
 {
   return new Context(default_port(), servers);
 }
 
 static bool world_destroy(void *object)
 {
+  SKIP_IF(HAVE_UUID_UUID_H != 1);
+
   Context *test= (Context *)object;
 
   delete test;
@@ -100,9 +102,9 @@ collection_st collection[] ={
   {0, 0, 0, 0}
 };
 
-void get_world(Framework *world)
+void get_world(libtest::Framework *world)
 {
-  world->collections= collection;
-  world->_create= world_create;
-  world->_destroy= world_destroy;
+  world->collections(collection);
+  world->create(world_create);
+  world->destroy(world_destroy);
 }
