@@ -36,7 +36,7 @@
  */
 
 
-#include <config.h>
+#include "gear_config.h"
 
 #include "util/instance.hpp"
 
@@ -196,7 +196,7 @@ bool Instance::run()
             switch(errno)
             {
             default:
-              std::cerr << "Failed during send(" << strerror(errno) << ")" << std::endl;
+              std::cerr << "Failed dureng send(" << strerror(errno) << ")" << std::endl;
               break;
             }
           }
@@ -211,7 +211,6 @@ bool Instance::run()
     case READING:
       if (operation->has_response())
       {
-        size_t total_read;
         ssize_t read_length;
 
         do
@@ -240,7 +239,6 @@ bool Instance::run()
           }
 
           operation->push(buffer, static_cast<size_t>(read_length));
-          total_read+= static_cast<size_t>(read_length);
 
         } while (more_to_read());
       } // end has_response
